@@ -10,11 +10,11 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: arch.ml,v 1.5 1999/11/17 18:56:40 xleroy Exp $ *)
+(* $Id: arch.ml,v 1.8 2000/04/21 08:10:48 weis Exp $ *)
 
 (* Specific operations for the Alpha processor *)
 
-open Formatmsg
+open Format
 
 (* Addressing modes *)
 
@@ -27,6 +27,7 @@ type addressing_mode =
 type specific_operation =
     Iadd4 | Iadd8 | Isub4 | Isub8       (* Scaled adds and subs *)
   | Ireloadgp of bool                   (* The ldgp instruction *)
+  | Itrunc32                            (* Truncate 64-bit int to 32 bit *)
 
 (* Sizes, endianness *)
 
@@ -51,22 +52,23 @@ let num_args_addressing = function
 
 (* Printing operations and addressing modes *)
 
-let print_addressing printreg addr arg =
+let print_addressing printreg addr ppf arg =
   match addr with
-    Ibased(s, n) ->
-      print_string "\""; print_string s; print_string "\"";
-      if n <> 0 then begin print_string " + "; print_int n end
+  | Ibased(s, n) ->
+      fprintf ppf "\"%s\"%s" s
+      (if n <> 0 then Printf.sprintf " + %i" n else "")
   | Iindexed n ->
-      printreg arg.(0);
-      if n <> 0 then begin print_string " + "; print_int n end
+      fprintf ppf "%a%s" printreg arg.(0)
+      (if n <> 0 then Printf.sprintf " + %i" n else "")
 
-let print_specific_operation printreg op arg =
+let print_specific_operation printreg op ppf arg =
   match op with
-    Iadd4 -> printreg arg.(0); print_string " * 4 + "; printreg arg.(1)
-  | Iadd8 -> printreg arg.(0); print_string " * 8 + "; printreg arg.(1)
-  | Isub4 -> printreg arg.(0); print_string " * 4 - "; printreg arg.(1)
-  | Isub8 -> printreg arg.(0); print_string " * 8 - "; printreg arg.(1)
-  | Ireloadgp _ -> print_string "ldgp"
+  | Iadd4 -> fprintf ppf "%a  * 4 + %a" printreg arg.(0) printreg arg.(1)
+  | Iadd8 -> fprintf ppf "%a  * 8 + %a" printreg arg.(0) printreg arg.(1)
+  | Isub4 -> fprintf ppf "%a  * 4 - %a" printreg arg.(0) printreg arg.(1)
+  | Isub8 -> fprintf ppf "%a  * 8 - %a" printreg arg.(0) printreg arg.(1)
+  | Ireloadgp _ -> fprintf ppf "ldgp"
+  | Itrunc32 -> fprintf ppf "truncate32 %a" printreg arg.(0)
 
 (* Distinguish between the Digital assembler and other assemblers (e.g. gas) *)
 
