@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: proc.ml,v 1.4 1999/11/17 18:56:41 xleroy Exp $ *)
+(* $Id: proc.ml,v 1.6 2000/04/06 14:19:46 xleroy Exp $ *)
 
 (* Description of the ARM processor *)
 
@@ -37,15 +37,16 @@ let word_addressed = false
     r14                         return address
     r15                         program counter
 
-    f0 - f7                     general purpose (f4 - f7 preserved by C)
+    f0 - f6                     general purpose (f4 - f6 preserved by C)
+    f7                          temporary
 *)
 
 let int_reg_name = [|
   "r0"; "r1"; "r2"; "r3"; "r4"; "r5"; "r6"; "r7"; "r10"; "r12"
 |]
-  
+
 let float_reg_name = [|
-  "f0"; "f1"; "f2"; "f3"; "f4"; "f5"; "f6"; "f7"
+  "f0"; "f1"; "f2"; "f3"; "f4"; "f5"; "f6"
 |]
 
 let num_register_classes = 2
@@ -56,7 +57,7 @@ let register_class r =
   | Addr -> 0
   | Float -> 1
 
-let num_available_registers = [| 10; 8 |]
+let num_available_registers = [| 10; 7 |]
 
 let first_available_register = [| 0; 100 |]
 
@@ -73,8 +74,8 @@ let hard_int_reg =
   v
 
 let hard_float_reg =
-  let v = Array.create 8 Reg.dummy in
-  for i = 0 to 7 do v.(i) <- Reg.at_location Float (Reg(100 + i)) done;
+  let v = Array.create 7 Reg.dummy in
+  for i = 0 to 6 do v.(i) <- Reg.at_location Float (Reg(100 + i)) done;
   v
 
 let all_phys_regs =
@@ -161,13 +162,13 @@ let loc_exn_bucket = phys_reg 0
 
 (* Registers destroyed by operations *)
 
-let destroyed_at_c_call =               (* r4-r9, f4-f7 preserved *)
+let destroyed_at_c_call =               (* r4-r9, f4-f6 preserved *)
   Array.of_list(List.map phys_reg [0;1;2;3;8;9; 100;101;102;103])
 
 let destroyed_at_oper = function
     Iop(Icall_ind | Icall_imm _ | Iextcall(_, true)) -> all_phys_regs
   | Iop(Iextcall(_, false)) -> destroyed_at_c_call
-  | Iop(Ialloc(_)) -> [|phys_reg 8|]	(* r10 destroyed *)
+  | Iop(Ialloc(_)) -> [|phys_reg 8|]    (* r10 destroyed *)
   | _ -> [||]
 
 let destroyed_at_raise = all_phys_regs
@@ -176,10 +177,10 @@ let destroyed_at_raise = all_phys_regs
 
 let safe_register_pressure = function
     Iextcall(_, _) -> 4
-  | _ -> 8
+  | _ -> 7
 let max_register_pressure = function
     Iextcall(_, _) -> [| 4; 4 |]
-  | _ -> [| 10; 8 |]
+  | _ -> [| 10; 7 |]
 
 (* Layout of the stack *)
 
