@@ -10,16 +10,18 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: asmlink.mli,v 1.5 1999/11/17 18:56:30 xleroy Exp $ *)
+(* $Id: asmlink.mli,v 1.7 2000/04/21 08:10:28 weis Exp $ *)
 
 (* Link a set of .cmx/.o files and produce an executable *)
 
-val link: string list -> unit
+open Format
+
+val link: formatter -> string list -> unit
 
 type error =
     File_not_found of string
   | Not_an_object_file of string
-  | Missing_implementations of string list
+  | Missing_implementations of (string * string list) list
   | Inconsistent_interface of string * string * string
   | Inconsistent_implementation of string * string * string
   | Assembler_error of string
@@ -27,4 +29,4 @@ type error =
 
 exception Error of error
 
-val report_error: error -> unit
+val report_error: formatter -> error -> unit
