@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: clambda.ml,v 1.12 1999/11/17 18:56:30 xleroy Exp $ *)
+(* $Id: clambda.ml,v 1.15 2001/02/19 20:15:36 maranget Exp $ *)
 
 (* A variant of the "lambda" code with direct / indirect calls explicit
    and closures explicit too *)
@@ -32,8 +32,8 @@ type ulambda =
   | Uletrec of (Ident.t * ulambda) list * ulambda
   | Uprim of primitive * ulambda list
   | Uswitch of ulambda * ulambda_switch
-  | Ustaticfail
-  | Ucatch of ulambda * ulambda
+  | Ustaticfail of int * ulambda list
+  | Ucatch of int * Ident.t list * ulambda * ulambda
   | Utrywith of ulambda * Ident.t * ulambda
   | Uifthenelse of ulambda * ulambda * ulambda
   | Usequence of ulambda * ulambda
@@ -44,10 +44,9 @@ type ulambda =
 
 and ulambda_switch =
   { us_index_consts: int array;
-    us_cases_consts: ulambda array;
+    us_actions_consts : ulambda array;
     us_index_blocks: int array;
-    us_cases_blocks: ulambda array;
-    us_checked: bool }
+    us_actions_blocks: ulambda array}
 
 (* Description of known functions *)
 
