@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: closure.ml,v 1.33 1999/12/06 16:59:05 xleroy Exp $ *)
+(* $Id: closure.ml,v 1.40 2001/02/19 20:15:37 maranget Exp $ *)
 
 (* Introduction of closures, uncurrying, recognition of direct calls *)
 
@@ -18,6 +18,7 @@ open Misc
 open Asttypes
 open Primitive
 open Lambda
+open Switch
 open Clambda
 
 (* Auxiliaries for compiling functions *)
@@ -42,27 +43,27 @@ let occurs_var var u =
       Uvar v -> v = var
     | Uconst cst -> false
     | Udirect_apply(lbl, args) -> List.exists occurs args
-    | Ugeneric_apply(funct, args) -> occurs funct or List.exists occurs args
+    | Ugeneric_apply(funct, args) -> occurs funct || List.exists occurs args
     | Uclosure(fundecls, clos) -> List.exists occurs clos
     | Uoffset(u, ofs) -> occurs u
-    | Ulet(id, def, body) -> occurs def or occurs body
+    | Ulet(id, def, body) -> occurs def || occurs body
     | Uletrec(decls, body) ->
-        List.exists (fun (id, u) -> occurs u) decls or occurs body
+        List.exists (fun (id, u) -> occurs u) decls || occurs body
     | Uprim(p, args) -> List.exists occurs args
     | Uswitch(arg, s) ->
-        occurs arg or occurs_array s.us_cases_consts
-                   or occurs_array s.us_cases_blocks
-    | Ustaticfail -> false
-    | Ucatch(body, hdlr) -> occurs body or occurs hdlr
-    | Utrywith(body, exn, hdlr) -> occurs body or occurs hdlr
+        occurs arg ||
+        occurs_array s.us_actions_consts || occurs_array s.us_actions_blocks
+    | Ustaticfail (_, args) -> List.exists occurs args
+    | Ucatch(_, _, body, hdlr) -> occurs body || occurs hdlr
+    | Utrywith(body, exn, hdlr) -> occurs body || occurs hdlr
     | Uifthenelse(cond, ifso, ifnot) ->
-        occurs cond or occurs ifso or occurs ifnot
-    | Usequence(u1, u2) -> occurs u1 or occurs u2
-    | Uwhile(cond, body) -> occurs cond or occurs body
-    | Ufor(id, lo, hi, dir, body) -> occurs lo or occurs hi or occurs body
-    | Uassign(id, u) -> id = var or occurs u
+        occurs cond || occurs ifso || occurs ifnot
+    | Usequence(u1, u2) -> occurs u1 || occurs u2
+    | Uwhile(cond, body) -> occurs cond || occurs body
+    | Ufor(id, lo, hi, dir, body) -> occurs lo || occurs hi || occurs body
+    | Uassign(id, u) -> id = var || occurs u
     | Usend(met, obj, args) -> 
-        occurs met or occurs obj or List.exists occurs args
+        occurs met || occurs obj || List.exists occurs args
   and occurs_array a =
     try
       for i = 0 to Array.length a - 1 do
@@ -97,8 +98,12 @@ let prim_size prim args =
   | Parrayrefs kind -> if kind = Pgenarray then 18 else 8
   | Parraysets kind -> if kind = Pgenarray then 22 else 10
   | Pbittest -> 3
+  | Pbigarrayref(ndims, _, _) -> 4 + ndims * 6
+  | Pbigarrayset(ndims, _, _) -> 4 + ndims * 6
   | _ -> 2 (* arithmetic and comparisons *)
 
+(* Very raw approximation of switch cost *)
+  
 let lambda_smaller lam threshold =
   let size = ref 0 in
   let rec lambda_size lam =
@@ -125,14 +130,13 @@ let lambda_smaller lam threshold =
         size := !size + prim_size prim args;
         lambda_list_size args
     | Uswitch(lam, cases) ->
-        if Array.length cases.us_cases_consts > 0 then size := !size + 5;
-        if Array.length cases.us_cases_blocks > 0 then size := !size + 5;
-        if cases.us_checked then size := !size + 2;
+        if Array.length cases.us_actions_consts > 1 then size := !size + 5 ;
+        if Array.length cases.us_actions_blocks > 1 then size := !size + 5 ;
         lambda_size lam;
-        lambda_array_size cases.us_cases_consts;
-        lambda_array_size cases.us_cases_blocks
-    | Ustaticfail -> ()
-    | Ucatch(body, handler) ->
+        lambda_array_size cases.us_actions_consts ;
+        lambda_array_size cases.us_actions_blocks
+    | Ustaticfail (_,args) -> lambda_list_size args
+    | Ucatch(_, _, body, handler) ->
         incr size; lambda_size body; lambda_size handler
     | Utrywith(body, id, handler) ->
         size := !size + 8; lambda_size body; lambda_size handler
@@ -165,7 +169,7 @@ let rec is_pure_clambda = function
   | Uconst cst -> true
   | Uprim((Psetglobal _ | Psetfield _ | Psetfloatfield _ |
            Pccall _ | Praise | Poffsetref _ | Pstringsetu | Pstringsets |
-           Parraysetu _ | Parraysets _), _) -> false
+           Parraysetu _ | Parraysets _ | Pbigarrayset _), _) -> false
   | Uprim(p, args) -> List.for_all is_pure_clambda args
   | _ -> false
 
@@ -229,8 +233,10 @@ let simplif_prim p (args, approxs as args_approxs) =
   then simplif_prim_pure p args_approxs
   else (Uprim(p, args), Value_unknown)
 
-(* Substitute variables in a [ulambda] term and perform
-   some more simplifications on integer primitives.
+(* Substitute variables in a [ulambda] term (a body of an inlined function)
+   and perform some more simplifications on integer primitives.
+   Also perform alpha-conversion on let-bound identifiers to avoid
+   clashes with locally-generated identifiers.
    The variables must not be assigned in the term.
    This is used to substitute "trivial" arguments for parameters
    during inline expansion. *)
@@ -241,44 +247,66 @@ let approx_ulam = function
   | Uconst(Const_pointer n) -> Value_constptr n
   | _ -> Value_unknown
 
-let substitute sb ulam =
-  let rec subst ulam =
-    match ulam with
-      Uvar v ->
-        begin try Tbl.find v sb with Not_found -> ulam end
-    | Uconst cst -> ulam
-    | Udirect_apply(lbl, args) -> Udirect_apply(lbl, List.map subst args)
-    | Ugeneric_apply(fn, args) -> Ugeneric_apply(subst fn, List.map subst args)
-    | Uclosure(defs, env) -> Uclosure(defs, List.map subst env)
-    | Uoffset(u, ofs) -> Uoffset(subst u, ofs)
-    | Ulet(id, u1, u2) -> Ulet(id, subst u1, subst u2)
-    | Uletrec(bindings, body) ->
-        Uletrec(List.map (fun (id, u) -> (id, subst u)) bindings, subst body)
-    | Uprim(p, args) ->
-        let sargs = List.map subst args in
-        let (res, _) = simplif_prim p (sargs, List.map approx_ulam sargs) in
-        res
-    | Uswitch(arg, sw) ->
-        Uswitch(subst arg,
-                { us_index_consts = sw.us_index_consts;
-                  us_cases_consts = Array.map subst sw.us_cases_consts;
-                  us_index_blocks = sw.us_index_blocks;
-                  us_cases_blocks = Array.map subst sw.us_cases_blocks;
-                  us_checked = sw.us_checked })
-    | Ustaticfail -> Ustaticfail
-    | Ucatch(u1, u2) -> Ucatch(subst u1, subst u2)
-    | Utrywith(u1, id, u2) -> Utrywith(subst u1, id, subst u2)
-    | Uifthenelse(u1, u2, u3) ->
-        begin match subst u1 with
-          Uconst(Const_pointer n) -> if n <> 0 then subst u2 else subst u3
-        | su1 -> Uifthenelse(su1, subst u2, subst u3)
-        end
-    | Usequence(u1, u2) -> Usequence(subst u1, subst u2)
-    | Uwhile(u1, u2) -> Uwhile(subst u1, subst u2)
-    | Ufor(id, u1, u2, dir, u3) -> Ufor(id, subst u1, subst u2, dir, subst u3)
-    | Uassign(id, u) -> Uassign(id, subst u)
-    | Usend(u1, u2, ul) -> Usend(subst u1, subst u2, List.map subst ul)
-  in subst ulam
+let rec substitute sb ulam =
+  match ulam with
+    Uvar v ->
+      begin try Tbl.find v sb with Not_found -> ulam end
+  | Uconst cst -> ulam
+  | Udirect_apply(lbl, args) ->
+      Udirect_apply(lbl, List.map (substitute sb) args)
+  | Ugeneric_apply(fn, args) ->
+      Ugeneric_apply(substitute sb fn, List.map (substitute sb) args)
+  | Uclosure(defs, env) ->
+      (* never present in an inlined function body; painful to get right *)
+      assert false
+  | Uoffset(u, ofs) -> Uoffset(substitute sb u, ofs)
+  | Ulet(id, u1, u2) ->
+      let id' = Ident.rename id in
+      Ulet(id', substitute sb u1, substitute (Tbl.add id (Uvar id') sb) u2)
+  | Uletrec(bindings, body) ->
+      (* never present in an inlined function body; painful to get right *)
+      assert false
+  | Uprim(p, args) ->
+      let sargs = List.map (substitute sb) args in
+      let (res, _) = simplif_prim p (sargs, List.map approx_ulam sargs) in
+      res
+  | Uswitch(arg, sw) ->
+      Uswitch(substitute sb arg,
+              { sw with
+                us_actions_consts =
+                  Array.map (substitute sb) sw.us_actions_consts;
+                us_actions_blocks =
+                  Array.map (substitute sb) sw.us_actions_blocks;
+               })
+  | Ustaticfail (nfail, args) ->
+      Ustaticfail (nfail, List.map (substitute sb) args)
+  | Ucatch(nfail, ids, u1, u2) ->
+      Ucatch(nfail, ids, substitute sb u1, substitute sb u2)
+  | Utrywith(u1, id, u2) ->
+      let id' = Ident.rename id in
+      Utrywith(substitute sb u1, id', substitute (Tbl.add id (Uvar id') sb) u2)
+  | Uifthenelse(u1, u2, u3) ->
+      begin match substitute sb u1 with
+        Uconst(Const_pointer n) ->
+          if n <> 0 then substitute sb u2 else substitute sb u3
+      | su1 ->
+          Uifthenelse(su1, substitute sb u2, substitute sb u3)
+      end
+  | Usequence(u1, u2) -> Usequence(substitute sb u1, substitute sb u2)
+  | Uwhile(u1, u2) -> Uwhile(substitute sb u1, substitute sb u2)
+  | Ufor(id, u1, u2, dir, u3) ->
+      let id' = Ident.rename id in
+      Ufor(id', substitute sb u1, substitute sb u2, dir,
+           substitute (Tbl.add id (Uvar id') sb) u3)
+  | Uassign(id, u) ->
+      let id' =
+        try
+          match Tbl.find id sb with Uvar i -> i | _ -> assert false
+        with Not_found ->
+          id in
+      Uassign(id', substitute sb u)
+  | Usend(u1, u2, ul) ->
+      Usend(substitute sb u1, substitute sb u2, List.map (substitute sb) ul)
 
 (* Perform an inline expansion *)
 
@@ -300,8 +328,9 @@ let rec bind_params subst params args body =
       if is_simple_argument a1 then
         bind_params (Tbl.add p1 a1 subst) pl al body
       else begin
-        let body' = bind_params subst pl al body in
-        if occurs_var p1 body then Ulet(p1, a1, body')
+        let p1' = Ident.rename p1 in
+        let body' = bind_params (Tbl.add p1 (Uvar p1') subst) pl al body in
+        if occurs_var p1 body then Ulet(p1', a1, body')
         else if no_effects a1 then body'
         else Usequence(a1, body')
       end
@@ -364,6 +393,11 @@ let sequence_constant_expr lam ulam1 (ulam2, approx2 as res2) =
 
 let global_approx = ref([||] : value_approximation array)
 
+(* Maintain the nesting depth for functions *)
+
+let function_nesting_depth = ref 0
+let excessive_function_nesting_depth = 5
+
 (* Uncurry an expression and explicitate closures.
    Also return the approximation of the expression.
    The approximation environment [fenv] maps idents to approximations.
@@ -385,6 +419,8 @@ let close_approx_var fenv cenv id =
 let close_var fenv cenv id =
   let (ulam, app) = close_approx_var fenv cenv id in ulam
 
+exception Found of int
+
 let rec close fenv cenv = function
     Lvar id ->
       close_approx_var fenv cenv id
@@ -490,25 +526,25 @@ let rec close fenv cenv = function
        Value_unknown)
   | Lprim(p, args) ->
       simplif_prim p (close_list_approx fenv cenv args)
-  | Lswitch(arg, sw) ->
+  | Lswitch(arg, sw) as l ->
+(* NB: failaction might get copied, thus it should be some Lstaticraise *)
       let (uarg, _) = close fenv cenv arg in
-      let (const_index, const_cases) =
-        close_switch fenv cenv sw.sw_numconsts sw.sw_consts in
-      let (block_index, block_cases) =
-        close_switch fenv cenv sw.sw_numblocks sw.sw_blocks in
+      let const_index, const_actions =
+        close_switch fenv cenv sw.sw_consts sw.sw_numconsts sw.sw_failaction
+      and block_index, block_actions =
+        close_switch fenv cenv sw.sw_blocks sw.sw_numblocks sw.sw_failaction in
       (Uswitch(uarg, 
                {us_index_consts = const_index;
-                us_cases_consts = const_cases;
+                us_actions_consts = const_actions;
                 us_index_blocks = block_index;
-                us_cases_blocks = block_cases;
-                us_checked = sw.sw_checked}),
+                us_actions_blocks = block_actions}),
        Value_unknown)
-  | Lstaticfail ->
-      (Ustaticfail, Value_unknown)
-  | Lcatch(body, handler) ->
+  | Lstaticraise (i, args) ->
+      (Ustaticfail (i, close_list fenv cenv args), Value_unknown)
+  | Lstaticcatch(body, (i, vars), handler) ->
       let (ubody, _) = close fenv cenv body in
       let (uhandler, _) = close fenv cenv handler in
-      (Ucatch(ubody, uhandler), Value_unknown)
+      (Ucatch(i, vars, ubody, uhandler), Value_unknown)
   | Ltrywith(body, id, handler) ->
       let (ubody, _) = close fenv cenv body in
       let (uhandler, _) = close fenv cenv handler in
@@ -563,6 +599,10 @@ and close_named fenv cenv id = function
 (* Build a shared closure for a set of mutually recursive functions *)
 
 and close_functions fenv cenv fun_defs =
+  (* Update and check nesting depth *)
+  incr function_nesting_depth;
+  let initially_closed =
+    !function_nesting_depth < excessive_function_nesting_depth in
   (* Determine the free variables of the functions *)
   let fv =
     IdentSet.elements (free_variables (Lletrec(fun_defs, lambda_unit))) in
@@ -579,7 +619,7 @@ and close_functions fenv cenv fun_defs =
             let fundesc =
               {fun_label = label;
                fun_arity = (if kind = Tupled then -arity else arity);
-               fun_closed = true;
+               fun_closed = initially_closed;
                fun_inline = None } in
             (id, params, body, fundesc)
         | (_, _) -> fatal_error "Closure.close_functions")
@@ -602,7 +642,7 @@ and close_functions fenv cenv fun_defs =
   let fv_pos = !env_pos in
   (* This reference will be set to false if the hypothesis that a function
      does not use its environment parameter is invalidated. *)
-  let useless_env = ref true in
+  let useless_env = ref initially_closed in
   (* Translate each function definition *)
   let clos_fundef (id, params, body, fundesc) env_pos =
     let env_param = Ident.create "env" in
@@ -614,22 +654,29 @@ and close_functions fenv cenv fun_defs =
           Tbl.add id (Uoffset(Uvar env_param, pos - env_pos)) env)
         uncurried_defs clos_offsets cenv_fv in
     let (ubody, approx) = close fenv_rec cenv_body body in
-    if !useless_env & occurs_var env_param ubody then useless_env := false;
+    if !useless_env && occurs_var env_param ubody then useless_env := false;
     let fun_params = if !useless_env then params else params @ [env_param] in
     ((fundesc.fun_label, fundesc.fun_arity, fun_params, ubody),
      (id, env_pos, Value_closure(fundesc, approx))) in
   (* Translate all function definitions. *)
-  let clos_info_list = 
-    let cl = List.map2 clos_fundef uncurried_defs clos_offsets in
-    (* If the hypothesis that the environment parameters are useless has been
-       invalidated, then set [fun_closed] to false in all descriptions and
-       recompile *)
-    if !useless_env then cl else begin
-      List.iter
-        (fun (id, params, body, fundesc) -> fundesc.fun_closed <- false)
-        uncurried_defs;
-      List.map2 clos_fundef uncurried_defs clos_offsets
-    end in
+  let clos_info_list =
+    if initially_closed then begin
+      let cl = List.map2 clos_fundef uncurried_defs clos_offsets in
+      (* If the hypothesis that the environment parameters are useless has been
+         invalidated, then set [fun_closed] to false in all descriptions and
+         recompile *)
+      if !useless_env then cl else begin
+        List.iter
+          (fun (id, params, body, fundesc) -> fundesc.fun_closed <- false)
+          uncurried_defs;
+        List.map2 clos_fundef uncurried_defs clos_offsets
+      end
+    end else
+      (* Excessive closure nesting: assume environment parameter is used *)
+        List.map2 clos_fundef uncurried_defs clos_offsets
+    in
+  (* Update nesting depth *)
+  decr function_nesting_depth;
   (* Return the Uclosure node and the list of all identifiers defined,
      with offsets and approximations. *)
   let (clos, infos) = List.split clos_info_list in
@@ -649,26 +696,37 @@ and close_one_function fenv cenv id funct =
 
 (* Close a switch *)
 
-and close_switch fenv cenv num_keys cases =
-  let index = Array.create num_keys 0 in
-  let ucases = ref []
-  and num_cases = ref 0 in
-  if List.length cases < num_keys then begin
-    num_cases := 1;
-    ucases := [Ustaticfail]
-  end;
+and close_switch fenv cenv cases num_keys default =
+  let index = Array.create num_keys 0
+  and store = mk_store Pervasives.(=) in
+
+  (* First default case *)
+  begin match default with
+  | Some def when List.length cases < num_keys ->
+      ignore (store.act_store def)
+  | _ -> ()
+  end ;
+  (* Then all other cases *)
   List.iter
-    (function (key, lam) ->
-        let (ulam, _) = close fenv cenv lam in
-        ucases := ulam :: !ucases;
-        index.(key) <- !num_cases;
-        incr num_cases)
-    cases;
-  (index, Array.of_list(List.rev !ucases))
+    (fun (key,lam) ->
+     index.(key) <- store.act_store lam)
+    cases ;
+  (* Compile action *)
+  let actions =
+    Array.map
+      (fun lam ->
+        let ulam,_ = close fenv cenv lam in
+        ulam)
+      (store.act_get ()) in
+  match actions with
+  | [| |] -> [| |], [| |] (* May happen when default is None *)
+  | _     -> index, actions
+
 
 (* The entry point *)
 
 let intro size lam =
+  function_nesting_depth := 0;
   global_approx := Array.create size Value_unknown;
   Compilenv.set_global_approx(Value_tuple !global_approx);
   let (ulam, approx) = close Tbl.empty Tbl.empty lam in
