@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: cmm.mli,v 1.17 2000/03/17 13:24:17 xleroy Exp $ *)
+(* $Id: cmm.mli,v 1.19 2000/10/02 14:08:16 maranget Exp $ *)
 
 (* Second intermediate language (machine independent) *)
 
@@ -86,8 +86,8 @@ type expression =
   | Cifthenelse of expression * expression * expression
   | Cswitch of expression * int array * expression array
   | Cloop of expression
-  | Ccatch of expression * expression
-  | Cexit
+  | Ccatch of int * Ident.t list * expression * expression
+  | Cexit of int * expression list
   | Ctrywith of expression * Ident.t * expression
 
 type fundecl =
