@@ -10,11 +10,11 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: codegen.ml,v 1.5 1999/11/17 18:56:31 xleroy Exp $ *)
+(* $Id: codegen.ml,v 1.7 2000/04/21 08:10:29 weis Exp $ *)
 
 (* From C-- to assembly code *)
 
-open Formatmsg
+open Format
 open Cmm
 
 let dump_cmm = ref false
@@ -43,10 +43,10 @@ let rec regalloc fd =
   then begin Reg.reinit(); Liveness.fundecl newfd; regalloc newfd end
   else newfd
 
-let fundecl fd_cmm =
+let fundecl ppf fd_cmm =
   if !dump_cmm then begin
-    print_string "*** C-- code"; print_newline();
-    Printcmm.fundecl fd_cmm; print_newline()
+    fprintf ppf "*** C-- code@.";
+    fprintf ppf "%a@." Printcmm.fundecl fd_cmm
   end;
   Reg.reset();
   let fd_sel = Sequence.fundecl fd_cmm in
@@ -65,7 +65,7 @@ let fundecl fd_cmm =
   let fd_reload = regalloc fd_split in
   let fd_linear = Linearize.fundecl fd_reload in
   if !dump_linear then begin
-    print_string "*** Linearized code"; print_newline();
+    printf "*** Linearized code@.";
     Printlinear.fundecl fd_linear; print_newline()
   end;
   Emit.fundecl fd_linear
