@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: compilenv.ml,v 1.13 1999/11/17 18:56:32 xleroy Exp $ *)
+(* $Id: compilenv.ml,v 1.16 2000/04/21 08:10:30 weis Exp $ *)
 
 (* Compilation environments for compilation units *)
 
@@ -44,6 +44,14 @@ type unit_infos =
     mutable ui_apply_fun: int list;             (* Apply functions needed *)
     mutable ui_force_link: bool }               (* Always linked *)
 
+(* Each .a library has a matching .cmxa file that provides the following
+   infos on the library: *)
+
+type library_infos =
+  { lib_units: (unit_infos * Digest.t) list;  (* List of unit infos w/ CRCs *)
+    lib_ccobjs: string list;            (* C object files needed *)
+    lib_ccopts: string list }           (* Extra opts to C compiler *)
+
 let global_approx_table =
   (Hashtbl.create 17 : (string, value_approximation) Hashtbl.t)
 
@@ -104,7 +112,7 @@ let global_approx global_ident =
             find_in_path !load_path (String.uncapitalize modname ^ ".cmx") in
           let (ui, crc) = read_unit_info filename in
           if ui.ui_name <> modname then
-            raise(Error(Illegal_renaming(modname, filename)));
+            raise(Error(Illegal_renaming(ui.ui_name, filename)));
           (ui.ui_approx, crc)
         with Not_found ->
           (Value_unknown, cmx_not_found_crc) in
@@ -143,17 +151,13 @@ let save_unit_info filename =
 
 (* Error report *)
 
-open Formatmsg
+open Format
 
-let report_error = function
-    Not_a_unit_info filename ->
-      print_string filename; print_space();
-      print_string "is not a compilation unit description."
+let report_error ppf = function
+  | Not_a_unit_info filename ->
+      fprintf ppf "%s@ is not a compilation unit description." filename
   | Corrupted_unit_info filename ->
-      print_string "Corrupted compilation unit description"; print_space();
-      print_string filename
+      fprintf ppf "Corrupted compilation unit description@ %s" filename
   | Illegal_renaming(modname, filename) ->
-      print_string filename; print_space();
-      print_string "contains the description for unit"; print_space();
-      print_string modname
+      fprintf ppf "%s@ contains the description for unit@ %s" filename modname
 
