@@ -10,14 +10,14 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: emitaux.mli,v 1.8 1999/11/17 18:56:33 xleroy Exp $ *)
+(* $Id: emitaux.mli,v 1.9 2000/02/21 19:38:06 xleroy Exp $ *)
 
 (* Common functions for emitting assembly code *)
 
 val output_channel: out_channel ref
 val emit_string: string -> unit
 val emit_int: int -> unit
-val emit_nativeint: Nativeint.t -> unit
+val emit_nativeint: nativeint -> unit
 val emit_symbol: char -> string -> unit
 val emit_printf: ('a, out_channel, unit) format -> 'a
 val emit_char: char -> unit
