@@ -10,10 +10,12 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: arch.ml,v 1.4 1999/11/17 18:56:42 xleroy Exp $ *)
+(* $Id: arch.ml,v 1.9 2000/04/21 08:11:02 weis Exp $ *)
 
 (* Specific operations for the Intel 386 processor *)
 
+open Format
+
 type addressing_mode =
     Ibased of string * int              (* symbol + displ *)
   | Iindexed of int                     (* reg + displ *)
@@ -23,17 +25,18 @@ type addressing_mode =
 
 type specific_operation =
     Ilea of addressing_mode             (* Lea gives scaled adds *)
-  | Istore_int of Nativeint.t * addressing_mode (* Store an integer constant *)
+  | Istore_int of nativeint * addressing_mode (* Store an integer constant *)
   | Istore_symbol of string * addressing_mode (* Store a symbol *)
   | Ioffset_loc of int * addressing_mode (* Add a constant to a location *)
   | Ipush                               (* Push regs on stack *)
-  | Ipush_int of Nativeint.t            (* Push an integer constant *)
+  | Ipush_int of nativeint            (* Push an integer constant *)
   | Ipush_symbol of string              (* Push a symbol *)
   | Ipush_load of addressing_mode       (* Load a scalar and push *)
   | Ipush_load_float of addressing_mode (* Load a float and push *)
   | Isubfrev | Idivfrev                 (* Reversed float sub and div *)
-  | Ifloatarithmem of float_operation * addressing_mode (* float arith w/mem *)
-
+  | Ifloatarithmem of bool * float_operation * addressing_mode
+                                        (* Float arith operation with memory *)
+                                        (* bool: true=64 bits, false=32 *)
 and float_operation =
     Ifloatadd | Ifloatsub | Ifloatsubrev | Ifloatmul | Ifloatdiv | Ifloatdivrev
 
@@ -66,70 +69,61 @@ let num_args_addressing = function
 
 (* Printing operations and addressing modes *)
 
-open Formatmsg
-
-let print_addressing printreg addr arg =
+let print_addressing printreg addr ppf arg =
   match addr with
-    Ibased(s, 0) ->
-      print_string "\""; print_string s; print_string "\""
+  | Ibased(s, 0) ->
+      fprintf ppf "\"%s\"" s
   | Ibased(s, n) ->
-      print_string "\""; print_string s; print_string "\" + "; print_int n
+      fprintf ppf "\"%s\" + %i" s n
   | Iindexed n ->
-      printreg arg.(0);
-      if n <> 0 then begin print_string " + "; print_int n end
+      let idx = if n <> 0 then Printf.sprintf " + %i" n else "" in
+      fprintf ppf "%a%s" printreg arg.(0) idx
   | Iindexed2 n ->
-      printreg arg.(0); print_string " + "; printreg arg.(1);
-      if n <> 0 then begin print_string " + "; print_int n end
+      let idx = if n <> 0 then Printf.sprintf " + %i" n else "" in
+      fprintf ppf "%a + %a%s" printreg arg.(0) printreg arg.(1) idx
   | Iscaled(scale, n) ->
-      printreg arg.(0); print_string " * "; print_int scale;
-      if n <> 0 then begin print_string " + "; print_int n end
+      let idx = if n <> 0 then Printf.sprintf " + %i" n else "" in
+      fprintf ppf "%a  * %i%s" printreg arg.(0) scale idx
   | Iindexed2scaled(scale, n) ->
-      printreg arg.(0); print_string " + "; printreg arg.(1);
-      print_string " * "; print_int scale;
-      if n <> 0 then begin print_string " + "; print_int n end
+      let idx = if n <> 0 then Printf.sprintf " + %i" n else "" in
+      fprintf ppf "%a + %a * %i%s" printreg arg.(0) printreg arg.(1) scale idx
 
-let print_specific_operation printreg op arg =
+let print_specific_operation printreg op ppf arg =
   match op with
-    Ilea addr -> print_addressing printreg addr arg
+  | Ilea addr -> print_addressing printreg addr ppf arg
   | Istore_int(n, addr) ->
-      print_string "["; print_addressing printreg addr arg;
-      print_string "] := "; print_string (Nativeint.to_string n)
+      fprintf ppf "[%a] := %s" (print_addressing printreg addr) arg
+                               (Nativeint.to_string n)
   | Istore_symbol(lbl, addr) ->
-      print_string "["; print_addressing printreg addr arg;
-      print_string "] := \""; print_string lbl; print_string "\""
+      fprintf ppf "[%a] := \"%s\"" (print_addressing printreg addr) arg lbl
   | Ioffset_loc(n, addr) ->
-      print_string "["; print_addressing printreg addr arg;
-      print_string "] +:= "; print_int n
+      fprintf ppf "[%a] +:= %i" (print_addressing printreg addr) arg n
   | Ipush ->
-      print_string "push ";
+      fprintf ppf "push ";
       for i = 0 to Array.length arg - 1 do
-        if i > 0 then print_string ", ";
-        printreg arg.(i)
+        if i > 0 then fprintf ppf ", ";
+        printreg ppf arg.(i)
       done
   | Ipush_int n ->
-      print_string "push "; print_string (Nativeint.to_string n)
+      fprintf ppf "push %s" (Nativeint.to_string n)
   | Ipush_symbol s ->
-      print_string "push \""; print_string s; print_string "\""
+      fprintf ppf "push \"%s\"" s
   | Ipush_load addr ->
-      print_string "push ["; print_addressing printreg addr arg;
-      print_string "]"
+      fprintf ppf "push [%a]" (print_addressing printreg addr) arg
   | Ipush_load_float addr ->
-      print_string "pushfloat ["; print_addressing printreg addr arg;
-      print_string "]"
+      fprintf ppf "pushfloat [%a]" (print_addressing printreg addr) arg
   | Isubfrev ->
-      printreg arg.(0); print_string " -f(rev) "; printreg arg.(1)
+      fprintf ppf "%a -f(rev) %a" printreg arg.(0) printreg arg.(1)
   | Idivfrev ->
-      printreg arg.(0); print_string " /f(rev) "; printreg arg.(1)
-  | Ifloatarithmem(op, addr) ->
-      printreg arg.(0);
-      begin match op with
-        Ifloatadd -> print_string " +f "
-      | Ifloatsub -> print_string " -f "
-      | Ifloatsubrev -> print_string " -f(rev) "
-      | Ifloatmul -> print_string " *f "
-      | Ifloatdiv -> print_string " /f "
-      | Ifloatdivrev -> print_string " /f(rev) "
-      end;
-      print_string "[";
-      print_addressing printreg addr (Array.sub arg 1 (Array.length arg - 1));
-      print_string "]"
+      fprintf ppf "%a /f(rev) %a" printreg arg.(0) printreg arg.(1)
+  | Ifloatarithmem(double, op, addr) ->
+      let op_name = function
+      | Ifloatadd -> "+f"
+      | Ifloatsub -> "-f"
+      | Ifloatsubrev -> "-f(rev)"
+      | Ifloatmul -> "*f"
+      | Ifloatdiv -> "/f"
+      | Ifloatdivrev -> "/f(rev)" in
+      let long = if double then "float64" else "float32" in
+      fprintf ppf "%a %s %s[%a]" printreg arg.(0) (op_name op) long
+       (print_addressing printreg addr) (Array.sub arg 1 (Array.length arg - 1))
