@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: selection.ml,v 1.6 1999/11/17 18:56:43 xleroy Exp $ *)
+(* $Id: selection.ml,v 1.11 2000/03/17 13:24:17 xleroy Exp $ *)
 
 (* Instruction selection for the Intel x86 *)
 
@@ -115,25 +115,28 @@ let pseudoregs_for_operation op arg res =
      Keep it simple, force it in edx. *)
   | Iintop_imm(Imod, _) ->
       ([| edx |], [| edx |], true)
-  (* For floating-point operations, the result is always left at the
-     top of the floating-point stack *)
+  (* For floating-point operations and floating-point loads,
+     the result is always left at the top of the floating-point stack *)
   | Iconst_float _ | Inegf | Iabsf | Iaddf | Isubf | Imulf | Idivf
-  | Ifloatofint | Ispecific(Isubfrev | Idivfrev | Ifloatarithmem(_, _)) ->
+  | Ifloatofint | Iload((Single | Double | Double_u), _)
+  | Ispecific(Isubfrev | Idivfrev | Ifloatarithmem(_, _, _)) ->
       (arg, [| tos |], false)           (* don't move it immediately *)
-  (* Same for a floating-point load *)
-  | Iload(Word, addr) when res.(0).typ = Float ->
-      (arg, [| tos |], false)
   (* For storing a byte, the argument must be in eax...edx.
-     For storing a halfword, any reg is ok.
-     Keep it simple, just force it to be in edx in both cases. *)
-  | Istore(Word, addr) -> raise Use_default
-  | Istore(chunk, addr) ->
+     (But for a short, any reg will do!)
+     Keep it simple, just force the argument to be in edx. *)
+  | Istore((Byte_unsigned | Byte_signed), addr) ->
       let newarg = Array.copy arg in
       newarg.(0) <- edx;
       (newarg, res, false)
   (* Other instructions are regular *)
   | _ -> raise Use_default
 
+let chunk_double = function
+    Single -> false
+  | Double -> true
+  | Double_u -> true
+  | _ -> assert false
+
 (* The selector class *)
 
 class selector = object (self)
@@ -158,11 +161,13 @@ method select_addressing exp =
 method select_store addr exp =
   match exp with
     Cconst_int n ->
-      (Ispecific(Istore_int(Nativeint.from n, addr)), Ctuple [])
+      (Ispecific(Istore_int(Nativeint.of_int n, addr)), Ctuple [])
   | Cconst_natint n ->
       (Ispecific(Istore_int(n, addr)), Ctuple [])
   | Cconst_pointer n ->
-      (Ispecific(Istore_int(Nativeint.from n, addr)), Ctuple [])
+      (Ispecific(Istore_int(Nativeint.of_int n, addr)), Ctuple [])
+  | Cconst_natpointer n ->
+      (Ispecific(Istore_int(n, addr)), Ctuple [])
   | Cconst_symbol s ->
       (Ispecific(Istore_symbol(s, addr)), Ctuple [])
   | _ ->
@@ -201,7 +206,7 @@ method select_operation op args =
   | Cdivf ->
       self#select_floatarith Idivf (Ispecific Idivfrev) Ifloatdiv Ifloatdivrev args
   (* Recognize store instructions *)
-  | Cstore ->
+  | Cstore Word ->
       begin match args with
         [loc; Cop(Caddi, [Cop(Cload _, [loc']); Cconst_int n])]
         when loc = loc' ->
@@ -216,12 +221,14 @@ method select_operation op args =
 
 method select_floatarith regular_op reversed_op mem_op mem_rev_op args =
   match args with
-    [arg1; Cop(Cload _, [loc2])] ->
+    [arg1; Cop(Cload chunk, [loc2])] ->
       let (addr, arg2) = self#select_addressing loc2 in
-      (Ispecific(Ifloatarithmem(mem_op, addr)), [arg1; arg2])
-  | [Cop(Cload _, [loc1]); arg2] ->
+      (Ispecific(Ifloatarithmem(chunk_double chunk, mem_op, addr)),
+                 [arg1; arg2])
+  | [Cop(Cload chunk, [loc1]); arg2] ->
       let (addr, arg1) = self#select_addressing loc1 in
-      (Ispecific(Ifloatarithmem(mem_rev_op, addr)), [arg2; arg1])
+      (Ispecific(Ifloatarithmem(chunk_double chunk, mem_rev_op, addr)),
+                 [arg2; arg1])
   | [arg1; arg2] ->
       (* Evaluate bigger subexpression first to minimize stack usage.
          Because of right-to-left evaluation, rightmost arg is evaluated
@@ -251,16 +258,17 @@ method insert_op op rs rd =
 
 method select_push exp =
   match exp with
-    Cconst_int n -> (Ispecific(Ipush_int(Nativeint.from n)), Ctuple [])
+    Cconst_int n -> (Ispecific(Ipush_int(Nativeint.of_int n)), Ctuple [])
   | Cconst_natint n -> (Ispecific(Ipush_int n), Ctuple [])
-  | Cconst_pointer n -> (Ispecific(Ipush_int(Nativeint.from n)), Ctuple [])
+  | Cconst_pointer n -> (Ispecific(Ipush_int(Nativeint.of_int n)), Ctuple [])
+  | Cconst_natpointer n -> (Ispecific(Ipush_int n), Ctuple [])
   | Cconst_symbol s -> (Ispecific(Ipush_symbol s), Ctuple [])
-  | Cop(Cload ty, [loc]) when ty = typ_float ->
-      let (addr, arg) = self#select_addressing loc in
-      (Ispecific(Ipush_load_float addr), arg)
-  | Cop(Cload ty, [loc]) when ty = typ_addr or ty = typ_int ->
+  | Cop(Cload Word, [loc]) ->
       let (addr, arg) = self#select_addressing loc in
       (Ispecific(Ipush_load addr), arg)
+  | Cop(Cload Double_u, [loc]) ->
+      let (addr, arg) = self#select_addressing loc in
+      (Ispecific(Ipush_load_float addr), arg)
   | _ -> (Ispecific(Ipush), exp)
 
 method emit_extcall_args env args =
