@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: linearize.ml,v 1.20 2000/01/07 15:50:42 doligez Exp $ *)
+(* $Id: linearize.ml,v 1.23 2001/02/05 08:49:10 xleroy Exp $ *)
 
 (* Transformation of Mach code into a list of pseudo-instructions. *)
 
@@ -45,6 +45,11 @@ and instruction_desc =
   | Lpoptrap
   | Lraise
 
+let has_fallthrough = function
+  | Lreturn | Lbranch _ | Lswitch _ | Lraise
+  | Lop Itailcall_ind | Lop (Itailcall_imm _) -> false
+  | _ -> true 
+
 type fundecl =
   { fun_name: string;
     fun_body: instruction;
@@ -91,15 +96,24 @@ let copy_instr d i n =
   { desc = d; next = n;
     arg = i.Mach.arg; res = i.Mach.res; live = i.Mach.live }
 
-(* Label the beginning of the given instruction sequence.
-   If the sequence starts with a branch, jump over it. *)
+(*
+   Label the beginning of the given instruction sequence.
+   - If the sequence starts with a branch, jump over it.
+   - If the sequence is the end, (tail call position), just do nothing
+*)
 
-let get_label n =
-  match n.desc with
+let get_label n = match n.desc with
     Lbranch lbl -> (lbl, n)
   | Llabel lbl -> (lbl, n)
+  | Lend -> (-1, n)
   | _ -> let lbl = new_label() in (lbl, cons_instr (Llabel lbl) n)
 
+(* Check the fallthrough label *)
+let check_label n = match n.desc with
+  | Lbranch lbl -> lbl
+  | Llabel lbl -> lbl
+  | _ -> -1
+
 (* Discard all instructions up to the next label.
    This function is to be called before adding a non-terminating
    instruction. *)
@@ -114,19 +128,35 @@ let rec discard_dead_code n =
   | Lop(Istackoffset _) -> n
   | _ -> discard_dead_code n.next
 
-(* Add a branch in front of a continuation.
+(*
+   Add a branch in front of a continuation.
    Discard dead code in the continuation.
-   Does not insert anything if we're just falling through. *)
+   Does not insert anything if we're just falling through
+   or if we jump to dead code after the end of function (lbl=-1)
+*)
 
 let add_branch lbl n =
-  let n1 = discard_dead_code n in
-  match n1.desc with
-    Llabel lbl1 when lbl1 = lbl -> n1
-  | _ -> cons_instr (Lbranch lbl) n1
+  if lbl >= 0 then
+    let n1 = discard_dead_code n in
+    match n1.desc with
+    | Llabel lbl1 when lbl1 = lbl -> n1
+    | _ -> cons_instr (Lbranch lbl) n1
+  else
+    discard_dead_code n
 
-(* Current label for exit handler *)
+(* Current labels for exit handler *)
 
-let exit_label = ref None
+let exit_label = ref []
+
+let find_exit_label k =
+  try
+    List.assoc k !exit_label
+  with
+  | Not_found -> Misc.fatal_error "Linearize.find_exit_label"
+
+let is_next_catch n = match !exit_label with
+| (n0,_)::_  when n0=n -> true
+| _ -> false
 
 (* Linearize an instruction [i]: add it in front of the continuation [n] *)
 
@@ -152,16 +182,19 @@ let rec linear i n =
           copy_instr (Lcondbranch(test, lbl)) i (linear ifnot n1)
       | _, Iend, Lbranch lbl ->
           copy_instr (Lcondbranch(invert_test test, lbl)) i (linear ifso n1)
-      | Iexit, _, _ ->
-          let n2 = linear ifnot n1 in
-          begin match !exit_label with None -> n2
-          | Some lbl -> copy_instr (Lcondbranch(test, lbl)) i n2
-          end
-      | _,  Iexit, _ ->
+      | Iexit nfail1, Iexit nfail2, _ 
+            when is_next_catch nfail1 ->
+          let lbl2 = find_exit_label nfail2 in
+          copy_instr
+            (Lcondbranch (invert_test test, lbl2)) i (linear ifso n1)
+      | Iexit nfail, _, _ ->
+          let n2 = linear ifnot n1
+          and lbl = find_exit_label nfail in
+          copy_instr (Lcondbranch(test, lbl)) i n2
+      | _,  Iexit nfail, _ ->
           let n2 = linear ifso n1 in
-          begin match !exit_label with None -> n2
-          | Some lbl -> copy_instr (Lcondbranch(invert_test test, lbl)) i n2
-          end
+          let lbl = find_exit_label nfail in
+          copy_instr (Lcondbranch(invert_test test, lbl)) i n2
       | Iend, _, _ ->
           let (lbl_end, n2) = get_label n1 in
           copy_instr (Lcondbranch(test, lbl_end)) i (linear ifnot n2)
@@ -189,8 +222,7 @@ let rec linear i n =
       (* Switches with 1 and 2 branches have been eliminated earlier.
          Here, we do something for switches with 3 branches. *)
       if Array.length index = 3 then begin
-        let fallthrough_lbl =
-          match !n2.desc with Llabel lbl -> lbl | _ -> -1 in
+        let fallthrough_lbl = check_label !n2 in
         let find_label n =
           let lbl = lbl_cases.(index.(n)) in
           if lbl = fallthrough_lbl then None else Some lbl in
@@ -203,19 +235,17 @@ let rec linear i n =
       let n1 = linear i.Mach.next n in
       let n2 = linear body (cons_instr (Lbranch lbl_head) n1) in
       cons_instr (Llabel lbl_head) n2
-  | Icatch(body, handler) ->
+  | Icatch(io, body, handler) ->
       let (lbl_end, n1) = get_label(linear i.Mach.next n) in
       let (lbl_handler, n2) = get_label(linear handler n1) in
-      let saved_exit_label = !exit_label in
-      exit_label := Some lbl_handler;
+      exit_label := (io, lbl_handler) :: !exit_label ;
       let n3 = linear body (add_branch lbl_end n2) in
-      exit_label := saved_exit_label;
+      exit_label := List.tl !exit_label;
       n3
-  | Iexit ->
+  | Iexit nfail ->
       let n1 = linear i.Mach.next n in
-      begin match !exit_label with None -> n1
-      | Some lbl -> add_branch lbl n1
-      end
+      let lbl = find_exit_label nfail in
+      add_branch lbl n1
   | Itrywith(body, handler) ->
       let (lbl_join, n1) = get_label (linear i.Mach.next n) in
       let (lbl_body, n2) =
