@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: mach.ml,v 1.16 2000/02/21 19:38:07 xleroy Exp $ *)
+(* $Id: mach.ml,v 1.17 2000/08/11 19:50:52 maranget Exp $ *)
 
 (* Representation of machine code by sequences of pseudoinstructions *)
 
@@ -69,8 +69,8 @@ and instruction_desc =
   | Iifthenelse of test * instruction * instruction
   | Iswitch of int array * instruction array
   | Iloop of instruction
-  | Icatch of instruction * instruction
-  | Iexit
+  | Icatch of int * instruction * instruction
+  | Iexit of int
   | Itrywith of instruction * instruction
   | Iraise
 
@@ -117,9 +117,9 @@ let rec instr_iter f i =
           instr_iter f i.next
       | Iloop(body) ->
           instr_iter f body; instr_iter f i.next
-      | Icatch(body, handler) ->
+      | Icatch(_, body, handler) ->
           instr_iter f body; instr_iter f handler; instr_iter f i.next
-      | Iexit -> ()
+      | Iexit _ -> ()
       | Itrywith(body, handler) ->
           instr_iter f body; instr_iter f handler; instr_iter f i.next
       | Iraise -> ()
