@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: printcmm.ml,v 1.21 2000/04/21 08:10:33 weis Exp $ *)
+(* $Id: printcmm.ml,v 1.23 2000/10/02 14:08:20 maranget Exp $ *)
 
 (* Pretty-printing of C-- code *)
 
@@ -144,10 +144,19 @@ let rec expr ppf = function
       fprintf ppf "@[<v 0>@[<2>(switch@ %a@ @]%t)@]" expr e1 print_cases 
   | Cloop e ->
       fprintf ppf "@[<2>(loop@ %a)@]" sequence e
-  | Ccatch(e1, e2) ->
-      fprintf ppf "@[<2>(catch@ %a@;<1 -2>with@ %a)@]" sequence e1 sequence e2
-  | Cexit ->
-      fprintf ppf "exit"
+  | Ccatch(i, ids, e1, e2) ->
+      fprintf ppf
+        "@[<2>(catch@ %a@;<1 -2>with(%d%a)@ %a)@]"
+        sequence e1 i
+        (fun ppf ids ->
+          List.iter
+            (fun id -> fprintf ppf " %a" Ident.print id)
+            ids) ids
+        sequence e2
+  | Cexit (i, el) ->
+      fprintf ppf "@[<2>(exit %d" i ;
+      List.iter (fun e -> fprintf ppf "@ %a" expr e) el;
+      fprintf ppf ")@]"
   | Ctrywith(e1, id, e2) ->
       fprintf ppf "@[<2>(try@ %a@;<1 -2>with@ %a@ %a)@]"
              sequence e1 Ident.print id sequence e2
