@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: reloadgen.ml,v 1.4 1999/11/17 18:56:36 xleroy Exp $ *)
+(* $Id: reloadgen.ml,v 1.5 2000/08/11 19:50:55 maranget Exp $ *)
 
 (* Insert load/stores for pseudoregs that got assigned to stack locations. *)
 
@@ -120,11 +120,12 @@ method private reload i =
           (self#reload i.next))
   | Iloop body ->
       instr_cons (Iloop(self#reload body)) [||] [||] (self#reload i.next)
-  | Icatch(body, handler) ->
-      instr_cons (Icatch(self#reload body, self#reload handler)) [||] [||]
+  | Icatch(nfail, body, handler) ->
+      instr_cons
+        (Icatch(nfail, self#reload body, self#reload handler)) [||] [||]
         (self#reload i.next)
-  | Iexit ->
-      instr_cons Iexit [||] [||] dummy_instr
+  | Iexit i ->
+      instr_cons (Iexit i) [||] [||] dummy_instr
   | Itrywith(body, handler) ->
       instr_cons (Itrywith(self#reload body, self#reload handler)) [||] [||]
         (self#reload i.next)
