@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: selectgen.ml,v 1.22 2000/04/13 12:17:26 xleroy Exp $ *)
+(* $Id: selectgen.ml,v 1.28 2001/02/05 08:48:44 xleroy Exp $ *)
 
 (* Selection of pseudo-instructions, assignment of pseudo-registers,
    sequentialization. *)
@@ -174,6 +174,12 @@ let join_array rs =
     res
   end
 
+(* Registers for catch constructs *)
+let catch_regs = ref []
+
+(* Name of function being compiled *)
+let current_function_name = ref ""
+
 (* The default instruction selection class *)
 
 class virtual selector_generic = object (self)
@@ -269,7 +275,7 @@ method private select_arith op = function
       (Iintop op, args)
 
 method private select_shift op = function
-    [arg; Cconst_int n] when n >= 0 & n < Arch.size_int * 8 ->
+    [arg; Cconst_int n] when n >= 0 && n < Arch.size_int * 8 ->
       (Iintop_imm(op, n), [arg])
   | args ->
       (Iintop op, args)
@@ -301,8 +307,12 @@ method select_condition = function
       (Iinttest(Isigned cmp), Ctuple args)
   | Cop(Ccmpa cmp, [arg1; Cconst_pointer n]) when self#is_immediate n ->
       (Iinttest_imm(Iunsigned cmp, n), arg1)
+  | Cop(Ccmpa cmp, [arg1; Cconst_int n]) when self#is_immediate n ->
+      (Iinttest_imm(Iunsigned cmp, n), arg1)
   | Cop(Ccmpa cmp, [Cconst_pointer n; arg2]) when self#is_immediate n ->
       (Iinttest_imm(Iunsigned(swap_comparison cmp), n), arg2)
+  | Cop(Ccmpa cmp, [Cconst_int n; arg2]) when self#is_immediate n ->
+      (Iinttest_imm(Iunsigned(swap_comparison cmp), n), arg2)
   | Cop(Ccmpa cmp, args) ->
       (Iinttest(Iunsigned cmp), Ctuple args)
   | Cop(Ccmpf cmp, args) ->
@@ -448,8 +458,7 @@ method emit_expr env exp =
           let rd = Reg.createv typ_addr in
           let size = size_expr env (Ctuple new_args) in
           self#insert (Iop(Ialloc size)) [||] rd;
-          self#emit_stores env new_args rd 
-            (Arch.offset_addressing Arch.identity_addressing (-Arch.size_int));
+          self#emit_stores env new_args rd;
           rd
       | op ->
           let r1 = self#emit_tuple env new_args in
@@ -478,14 +487,35 @@ method emit_expr env exp =
       let (rarg, sbody) = self#emit_sequence env ebody in
       self#insert (Iloop(sbody#extract)) [||] [||];
       [||]
-  | Ccatch(e1, e2) ->
+  | Ccatch(nfail, ids, e1, e2) ->
+      let rs =
+        List.map
+          (fun id ->
+            let r = Reg.createv typ_addr in
+            name_regs id r  ;
+            r)
+          ids in
+      catch_regs := (nfail, Array.concat rs) :: !catch_regs ; 
       let (r1, s1) = self#emit_sequence env e1 in
-      let (r2, s2) = self#emit_sequence env e2 in
+      catch_regs := List.tl !catch_regs ;
+      let new_env =
+        List.fold_left
+        (fun env (id,r) -> Tbl.add id r env)
+        env (List.combine ids rs) in
+      let (r2, s2) = self#emit_sequence new_env e2 in
       let r = join r1 s1 r2 s2 in
-      self#insert (Icatch(s1#extract, s2#extract)) [||] [||];
+      self#insert (Icatch(nfail, s1#extract, s2#extract)) [||] [||];
       r
-  | Cexit ->
-      self#insert Iexit [||] [||];
+  | Cexit (nfail,args) ->
+      let (simple_list, ext_env) = self#emit_parts_list env args in
+      let src = self#emit_tuple ext_env simple_list in
+      let dest =
+        try List.assoc nfail !catch_regs with
+        | Not_found ->
+            Misc.fatal_error
+              ("Selectgen.emit_expr, on exit("^string_of_int nfail^")") in
+      self#insert_moves src dest ;
+      self#insert (Iexit nfail) [||] [||];
       [||]
   | Ctrywith(e1, v, e2) ->
       Proc.contains_calls := true;
@@ -572,8 +602,9 @@ method emit_extcall_args env args =
   self#insert_move_args r1 loc_arg stack_ofs;
   arg_stack
 
-method private emit_stores env data regs_addr addr =
-  let a = ref addr in
+method emit_stores env data regs_addr =
+  let a =
+    ref (Arch.offset_addressing Arch.identity_addressing (-Arch.size_int)) in
   List.iter
     (fun e ->
       let (op, arg) = self#select_store !a e in
@@ -632,6 +663,10 @@ method emit_tail env exp =
           if stack_ofs = 0 then begin
             self#insert_moves r1 loc_arg;
             self#insert (Iop(Itailcall_imm lbl)) loc_arg [||]
+          end else if lbl = !current_function_name then begin
+            let loc_arg' = Proc.loc_parameters r1 in
+            self#insert_moves r1 loc_arg';
+            self#insert (Iop(Itailcall_imm lbl)) loc_arg' [||]
           end else begin
             Proc.contains_calls := true;
             let rd = Reg.createv ty in
@@ -648,6 +683,7 @@ method emit_tail env exp =
       let rd = [|Proc.loc_exn_bucket|] in
       self#insert (Iop Imove) r1 rd;
       self#insert Iraise rd [||]
+  | Cexit (_,_) -> ignore (self#emit_expr env exp)
   | Csequence(e1, e2) ->
       let _ = self#emit_expr env e1 in
       self#emit_tail env e2
@@ -662,12 +698,23 @@ method emit_tail env exp =
       self#insert
         (Iswitch(index, Array.map (self#emit_tail_sequence env) ecases))
         rsel [||]
-  | Ccatch(e1, e2) ->
-      self#insert (Icatch(self#emit_tail_sequence env e1,
-                          self#emit_tail_sequence env e2))
-                  [||] [||]
-  | Cexit ->
-      self#insert Iexit [||] [||]
+  | Ccatch(nfail, ids, e1, e2) ->
+       let rs =
+        List.map
+          (fun id ->
+            let r = Reg.createv typ_addr in
+            name_regs id r  ;
+            r)
+          ids in
+      catch_regs := (nfail, Array.concat rs) :: !catch_regs ; 
+      let s1 = self#emit_tail_sequence env e1 in
+      catch_regs := List.tl !catch_regs ;
+      let new_env =
+        List.fold_left
+        (fun env (id,r) -> Tbl.add id r env)
+        env (List.combine ids rs) in
+      let s2 = self#emit_tail_sequence new_env e2 in
+      self#insert (Icatch(nfail, s1, s2)) [||] [||]
   | Ctrywith(e1, v, e2) ->
       Proc.contains_calls := true;
       let (r1, s1) = self#emit_sequence env e1 in
@@ -692,6 +739,7 @@ method private emit_tail_sequence env exp =
 
 method emit_fundecl f =
   Proc.contains_calls := false;
+  current_function_name := f.Cmm.fun_name;
   let rargs =
     List.map
       (fun (id, ty) -> let r = Reg.createv ty in name_regs id r; r)
