@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: selectgen.mli,v 1.4 1999/11/17 18:56:37 xleroy Exp $ *)
+(* $Id: selectgen.mli,v 1.5 2000/06/29 11:44:36 xleroy Exp $ *)
 
 (* Selection of pseudo-instructions, assignment of pseudo-registers,
    sequentialization. *)
@@ -44,6 +44,10 @@ class virtual selector_generic : object
   method emit_extcall_args :
     environment -> Cmm.expression list -> Reg.t array * int
     (* Can be overriden to deal with stack-based calling conventions *)
+  method emit_stores :
+    environment -> Cmm.expression list -> Reg.t array -> unit
+    (* Fill a freshly allocated block.  Can be overriden for architectures
+       that do not provide Arch.offset_addressing. *)
 
   (* The following method is the entry point and should not be overriden *)
   method emit_fundecl : Cmm.fundecl -> Mach.fundecl
