@@ -10,11 +10,11 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: arch.ml,v 1.4 1999/11/17 18:56:47 xleroy Exp $ *)
+(* $Id: arch.ml,v 1.6 2000/04/21 08:11:14 weis Exp $ *)
 
 (* Specific operations for the Sparc processor *)
 
-open Formatmsg
+open Format
 
 type specific_operation = unit          (* None worth mentioning *)
 
@@ -47,14 +47,14 @@ let num_args_addressing = function
 
 (* Printing operations and addressing modes *)
 
-let print_addressing printreg addr arg =
+let print_addressing printreg addr ppf arg =
   match addr with
-    Ibased(s, n) ->
-      print_string "\""; print_string s; print_string "\"";
-      if n <> 0 then begin print_string " + "; print_int n end
+  | Ibased(s, n) ->
+      let idx = if n <> 0 then Printf.sprintf " + %i" n else "" in
+      fprintf ppf "\"%s\"%s" s idx
   | Iindexed n ->
-      printreg arg.(0);
-      if n <> 0 then begin print_string " + "; print_int n end
+      let idx = if n <> 0 then Printf.sprintf " + %i" n else "" in
+      fprintf ppf "%a%s" printreg arg.(0) idx
 
-let print_specific_operation printreg op arg =
+let print_specific_operation printreg op ppf arg =
   Misc.fatal_error "Arch_sparc.print_specific_operation"
