@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: selection.ml,v 1.6 1999/11/17 18:56:47 xleroy Exp $ *)
+(* $Id: selection.ml,v 1.7 2000/12/28 13:03:03 weis Exp $ *)
 
 (* Instruction selection for the Sparc processor *)
 
@@ -49,12 +49,12 @@ method select_operation op args =
   | (Cmuli, _) -> 
       (Iextcall(".umul", false), args)
   | (Cdivi, [arg; Cconst_int n])
-    when self#is_immediate n & n = 1 lsl (Misc.log2 n) ->
+    when self#is_immediate n && n = 1 lsl (Misc.log2 n) ->
       (Iintop_imm(Idiv, n), [arg])
   | (Cdivi, _) -> 
       (Iextcall(".div", false), args)
   | (Cmodi, [arg; Cconst_int n])
-    when self#is_immediate n & n = 1 lsl (Misc.log2 n) ->
+    when self#is_immediate n && n = 1 lsl (Misc.log2 n) ->
       (Iintop_imm(Imod, n), [arg])
   | (Cmodi, _) ->
       (Iextcall(".rem", false), args)
