@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: fail.c,v 1.28 2001/06/15 14:22:49 xleroy Exp $ */
+/* $Id: fail.c,v 1.30 2001/12/07 13:39:19 xleroy Exp $ */
 
 /* Raising exceptions from C. */
 
@@ -32,7 +33,7 @@ typedef value caml_generated_constant[1];
 
 extern caml_generated_constant Out_of_memory, Sys_error, Failure,
   Invalid_argument, End_of_file, Division_by_zero, Not_found,
-  Match_failure, Sys_blocked_io;
+  Match_failure, Sys_blocked_io, Stack_overflow;
 
 /* Exception raising */
 
@@ -99,12 +100,13 @@ void invalid_argument (char *msg)
    because it allocates and we're out of memory...
    We therefore build the bucket by hand.
    This works OK because the exception value for Out_of_memory is also
-   statically allocated out of the heap. */
+   statically allocated out of the heap.
+   The same applies to Stack_overflow. */
 
 static struct {
   header_t hdr;
   value exn;
-} out_of_memory_bucket;
+} out_of_memory_bucket, stack_overflow_bucket;
 
 void raise_out_of_memory(void)
 {
@@ -113,6 +115,13 @@ void raise_out_of_memory(void)
   mlraise((value) &(out_of_memory_bucket.exn));
 }
 
+void raise_stack_overflow(void)
+{
+  stack_overflow_bucket.hdr = Make_header(1, 0, Caml_white);
+  stack_overflow_bucket.exn = (value) Stack_overflow;
+  mlraise((value) &(stack_overflow_bucket.exn));
+}
+
 void raise_sys_error(value msg)
 {
   raise_with_arg((value) Sys_error, msg);
