@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: bytelink.ml,v 1.59 2001/02/20 09:40:52 xleroy Exp $ *)
+(* $Id: bytelink.ml,v 1.70 2001/11/29 09:54:21 xleroy Exp $ *)
 
 (* Link a set of .cmo files and produce a bytecode executable. *)
 
@@ -27,6 +27,7 @@ type error =
   | Inconsistent_import of string * string * string
   | Custom_runtime
   | File_exists of string
+  | Cannot_open_dll of string
 
 exception Error of error
 
@@ -36,19 +37,22 @@ type link_action =
   | Link_archive of string * compilation_unit list
       (* Name of .cma file and descriptors of the units to be linked. *)
 
-(* Add C objects and options and "custom" info from a library descriptor *)
-(* Ignore them if -noautolink or -use-runtime were given *)
+(* Add C objects and options from a library descriptor *)
+(* Ignore them if -noautolink or -use-runtime or -use-prim was given *)
 
 let lib_ccobjs = ref []
 let lib_ccopts = ref []
+let lib_dllibs = ref []
 
 let add_ccobjs l =
-  if not !Clflags.no_auto_link && String.length !Clflags.use_runtime = 0
-     && String.length !Clflags.use_prims = 0
+  if not !Clflags.no_auto_link
+      && String.length !Clflags.use_runtime = 0
+      && String.length !Clflags.use_prims = 0
   then begin
     if l.lib_custom then Clflags.custom_runtime := true;
     lib_ccobjs := l.lib_ccobjs @ !lib_ccobjs;
-    lib_ccopts := l.lib_ccopts @ !lib_ccopts
+    lib_ccopts := l.lib_ccopts @ !lib_ccopts;
+    lib_dllibs := l.lib_dllibs @ !lib_dllibs
   end
 
 (* A note on ccobj ordering:
@@ -65,8 +69,8 @@ let add_ccobjs l =
      and b.cma was built with ocamlc -i ... objb1 objb2
    lib_ccobjs starts as [],
    becomes objb2 objb1 when b.cma is scanned,
-   then obja2 obja1 objb2 objb1 when b.cma is scanned.
-   Clflags.ccobjs was initially obj2 obj1,
+   then obja2 obja1 objb2 objb1 when a.cma is scanned.
+   Clflags.ccobjs was initially obj2 obj1.
    and is set to obj2 obj1 obja2 obja1 objb2 objb1.
    Finally, the C compiler is given objb1 objb2 obja1 obja2 obj1 obj2,
    which is what we need.  (If b depends on a, a.cma must appear before
@@ -249,6 +253,11 @@ let output_debug_info oc =
     !debug_info;
   debug_info := []
 
+(* Output a list of strings with 0-termination *)
+
+let output_stringlist oc l =
+  List.iter (fun s -> output_string oc s; output_byte oc 0) l
+
 (* Transform a file name into an absolute file name *)
 
 let make_absolute file =
@@ -258,7 +267,7 @@ let make_absolute file =
 
 (* Create a bytecode executable file *)
 
-let link_bytecode tolink exec_name copy_header =
+let link_bytecode tolink exec_name standalone =
   if Sys.os_type = "MacOS" then begin
     (* Create it as a text file for bytecode scripts *)
     let c = open_out_gen [Open_wronly; Open_creat] 0o777 exec_name in
@@ -267,8 +276,8 @@ let link_bytecode tolink exec_name copy_header =
   let outchan = open_out_gen [Open_wronly; Open_trunc; Open_creat; Open_binary]
                              0o777 exec_name in
   try
-    (* Copy the header *)
-    if copy_header then begin
+    if standalone then begin
+      (* Copy the header *)
       try
         let header =
           if String.length !Clflags.use_runtime > 0
@@ -289,13 +298,30 @@ let link_bytecode tolink exec_name copy_header =
     let start_code = pos_out outchan in
     Symtable.init();
     Hashtbl.clear crc_interfaces;
+    let sharedobjs = List.map Dll.extract_dll_name !Clflags.dllibs in
+    if standalone then begin
+      (* Initialize the DLL machinery *)
+      Dll.add_path !load_path;
+      try Dll.open_dlls sharedobjs
+      with Failure reason -> raise(Error(Cannot_open_dll reason))
+    end;
     let output_fun = output_string outchan
     and currpos_fun () = pos_out outchan - start_code in
     List.iter (link_file output_fun currpos_fun) tolink;
+    if standalone then Dll.close_all_dlls();
     (* The final STOP instruction *)
     output_byte outchan Opcodes.opSTOP;
     output_byte outchan 0; output_byte outchan 0; output_byte outchan 0;
     Bytesections.record outchan "CODE";
+    (* DLL stuff *)
+    if standalone then begin
+      (* The extra search path for DLLs *)
+      output_stringlist outchan !Clflags.dllpaths;
+      Bytesections.record outchan "DLPT";
+      (* The names of the DLLs *)
+      output_stringlist outchan sharedobjs;
+      Bytesections.record outchan "DLLS"
+    end;
     (* The names of all primitives *)
     Symtable.output_primitive_names outchan;
     Bytesections.record outchan "PRIM";
@@ -424,19 +450,13 @@ let build_custom_runtime prim_name exec_name =
                          (List.rev_map Ccomp.expand_libname !Clflags.ccobjs))
           (Ccomp.expand_libname "-lcamlrun")
           Config.bytecomp_c_libraries) in
-      (* C compiler doesn't clean up after itself *)
-      remove_file (Filename.chop_suffix prim_name ".c" ^ ".obj");
+      (* C compiler doesn't clean up after itself.  Note that the .obj
+         file is created in the current working directory. *)
+      remove_file
+        (Filename.chop_suffix (Filename.basename prim_name) ".c" ^ ".obj");
       retcode
   | "MacOS" ->
-      let c68k = "sc"
-      and libs68k = "\"{libraries}IntEnv.far.o\" \
-                     \"{libraries}MacRuntime.o\" \
-                     \"{clibraries}StdCLib.far.o\" \
-                     \"{libraries}MathLib.far.o\" \
-                     \"{libraries}ToolLibs.o\" \
-                     \"{libraries}Interface.o\""
-      and link68k = "ilink -compact -state nouse -model far -msg nodup"
-      and cppc = "mrc"
+      let cppc = "mrc"
       and libsppc = "\"{sharedlibraries}MathLib\" \
                      \"{ppclibraries}PPCCRuntime.o\" \
                      \"{ppclibraries}PPCToolLibs.o\" \
@@ -444,18 +464,11 @@ let build_custom_runtime prim_name exec_name =
                      \"{ppclibraries}StdCRuntime.o\" \
                      \"{sharedlibraries}InterfaceLib\""
       and linkppc = "ppclink -d"
-      and objs68k = extract ".o" (List.rev !Clflags.ccobjs)
       and objsppc = extract ".x" (List.rev !Clflags.ccobjs)
       and q_prim_name = Filename.quote prim_name
       and q_stdlib = Filename.quote Config.standard_library
       and q_exec_name = Filename.quote exec_name
       in
-      Ccomp.run_command (Printf.sprintf "%s -i %s %s %s -o %s.o"
-        c68k
-        q_stdlib
-        (String.concat " " (List.rev_map Filename.quote !Clflags.ccopts))
-        q_prim_name
-        q_prim_name);
       Ccomp.run_command (Printf.sprintf "%s -i %s %s %s -o %s.x"
         cppc
         q_stdlib
@@ -463,15 +476,6 @@ let build_custom_runtime prim_name exec_name =
         q_prim_name
         q_prim_name);
       Ccomp.run_command ("delete -i " ^ q_exec_name);
-      Ccomp.run_command (Printf.sprintf
-        "%s -t MPST -c 'MPS ' -o %s %s.o %s %s %s"
-        link68k
-        q_exec_name
-        q_prim_name
-        (String.concat " " (List.map Filename.quote objs68k))
-        (Filename.quote
-            (Filename.concat Config.standard_library "libcamlrun.o"))
-        libs68k);
       Ccomp.command (Printf.sprintf
         "%s -t MPST -c 'MPS ' -o %s %s.x %s %s %s"
         linkppc
@@ -504,11 +508,14 @@ let fix_exec_name name =
 (* Main entry point (build a custom runtime if needed) *)
 
 let link objfiles =
-  let objfiles = if !Clflags.nopervasives then objfiles
-                 else "stdlib.cma" :: (objfiles @ ["std_exit.cmo"]) in
+  let objfiles =
+    if !Clflags.nopervasives then objfiles
+    else if !Clflags.output_c_object then "stdlib.cma" :: objfiles
+    else "stdlib.cma" :: (objfiles @ ["std_exit.cmo"]) in
   let tolink = List.fold_right scan_file objfiles [] in
-  Clflags.ccobjs := !Clflags.ccobjs @ !lib_ccobjs;
+  Clflags.ccobjs := !Clflags.ccobjs @ !lib_ccobjs; (* put user's libs last *)
   Clflags.ccopts := !lib_ccopts @ !Clflags.ccopts; (* put user's opts first *)
+  Clflags.dllibs := !lib_dllibs @ !Clflags.dllibs; (* put user's DLLs first *)
   if not !Clflags.custom_runtime then
     link_bytecode tolink !Clflags.exec_name true
   else if not !Clflags.output_c_object then begin
@@ -558,10 +565,12 @@ let report_error ppf = function
       Symtable.report_error err
   | Inconsistent_import(intf, file1, file2) ->
       fprintf ppf
-        "@[<hv 0>Files %s and %s@ \
+        "@[<hov>Files %s@ and %s@ \
                  make inconsistent assumptions over interface %s@]"
         file1 file2 intf
   | Custom_runtime ->
       fprintf ppf "Error while building custom runtime system"
   | File_exists file ->
       fprintf ppf "Cannot overwrite existing file %s" file
+  | Cannot_open_dll file ->
+      fprintf ppf "Error on dynamically loaded library: %s" file
