@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: emitcode.mli,v 1.11 2000/03/09 09:12:26 xleroy Exp $ *)
+(* $Id: emitcode.mli,v 1.12 2001/10/30 09:32:30 xleroy Exp $ *)
 
 (* Generation of bytecode for .cmo files *)
 
@@ -49,8 +49,9 @@ type compilation_unit =
 type library =
   { lib_units: compilation_unit list;   (* List of compilation units *)
     lib_custom: bool;                   (* Requires custom mode linking? *)
-    lib_ccobjs: string list;            (* C object files needed *)
-    lib_ccopts: string list }           (* Extra opts to C compiler *)
+    lib_ccobjs: string list;            (* C object files needed for -custom *)
+    lib_ccopts: string list;            (* Extra opts to C compiler *)
+    lib_dllibs: string list }           (* DLLs needed *)
 
 (* Format of a .cma file:
      magic number (Config.cma_magic_number)
