@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: instruct.ml,v 1.17 2001/02/19 12:28:04 xleroy Exp $ *)
+(* $Id: instruct.ml,v 1.18 2001/11/26 12:47:01 xleroy Exp $ *)
 
 open Lambda
 
@@ -86,7 +86,7 @@ type instruction =
   | Kboolnot
   | Kpushtrap of label
   | Kpoptrap
-  | Kraise | Kreraise
+  | Kraise
   | Kcheck_signals
   | Kccall of string * int
   | Knegint | Kaddint | Ksubint | Kmulint | Kdivint | Kmodint
