@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: lambda.ml,v 1.34 2000/03/05 19:17:53 xleroy Exp $ *)
+(* $Id: lambda.ml,v 1.37 2001/02/19 20:27:30 maranget Exp $ *)
 
 open Misc
 open Path
@@ -57,6 +57,8 @@ type primitive =
   | Parraysets of array_kind
   (* Test if the argument is a block or an immediate integer *)
   | Pisint
+  (* Test if the (integer) argument is outside an interval *)
+  | Pisout
   (* Bitvect operations *)
   | Pbittest
   (* Operations on boxed integers (Nativeint.t, Int32.t, Int64.t) *)
@@ -123,8 +125,8 @@ type lambda =
   | Lletrec of (Ident.t * lambda) list * lambda
   | Lprim of primitive * lambda list
   | Lswitch of lambda * lambda_switch
-  | Lstaticfail
-  | Lcatch of lambda * lambda
+  | Lstaticraise of int * lambda list
+  | Lstaticcatch of lambda * (int * Ident.t list) * lambda
   | Ltrywith of lambda * Ident.t * lambda
   | Lifthenelse of lambda * lambda * lambda
   | Lsequence of lambda * lambda
@@ -140,7 +142,7 @@ and lambda_switch =
     sw_consts: (int * lambda) list;
     sw_numblocks: int;
     sw_blocks: (int * lambda) list;
-    sw_checked: bool }
+    sw_failaction : lambda option}
 
 and lambda_event =
   { lev_loc: int;
@@ -200,10 +202,16 @@ let free_variables l =
   | Lswitch(arg, sw) ->
       freevars arg; 
       List.iter (fun (key, case) -> freevars case) sw.sw_consts;
-      List.iter (fun (key, case) -> freevars case) sw.sw_blocks
-  | Lstaticfail -> ()
-  | Lcatch(e1, e2) ->
-      freevars e1; freevars e2
+      List.iter (fun (key, case) -> freevars case) sw.sw_blocks;
+      begin match sw.sw_failaction with
+      | None -> ()
+      | Some l -> freevars l
+      end
+  | Lstaticraise (_,args) ->
+      List.iter freevars args
+  | Lstaticcatch(e1, (_,vars), e2) ->
+      freevars e1; freevars e2 ;
+      List.iter (fun id -> fv := IdentSet.remove id !fv) vars        
   | Ltrywith(e1, exn, e2) ->
       freevars e1; freevars e2; fv := IdentSet.remove exn !fv
   | Lifthenelse(e1, e2, e3) ->
@@ -225,13 +233,30 @@ let free_variables l =
   in freevars l; !fv
 
 (* Check if an action has a "when" guard *)
+let raise_count = ref 0
+
+let next_raise_count () =
+  incr raise_count ;
+  !raise_count
+
+(* Anticipated staticraise, for guards *)
+let staticfail = Lstaticraise (0,[])
 
 let rec is_guarded = function
-    Lifthenelse(cond, body, Lstaticfail) -> true
+  | Lifthenelse( cond, body, Lstaticraise (0,[])) -> true
   | Llet(str, id, lam, body) -> is_guarded body
   | Levent(lam, ev) -> is_guarded lam
   | _ -> false
 
+let rec patch_guarded patch = function
+  | Lifthenelse (cond, body, Lstaticraise (0,[])) ->
+      Lifthenelse (cond, body, patch)
+  | Llet(str, id, lam, body) ->
+      Llet (str, id, lam, patch_guarded patch body)
+  | Levent(lam, ev) ->
+      Levent (patch_guarded patch lam, ev)
+  | _ -> fatal_error "Lambda.patch_guarded"
+
 (* Translate an access path *)
 
 let rec transl_path = function
@@ -269,9 +294,14 @@ let subst_lambda s lam =
   | Lswitch(arg, sw) ->
       Lswitch(subst arg,
               {sw with sw_consts = List.map subst_case sw.sw_consts;
-                       sw_blocks = List.map subst_case sw.sw_blocks})
-  | Lstaticfail -> Lstaticfail
-  | Lcatch(e1, e2) -> Lcatch(subst e1, subst e2)
+                       sw_blocks = List.map subst_case sw.sw_blocks;
+                       sw_failaction =
+                         match sw.sw_failaction with
+                         | None -> None
+                         | Some l -> Some (subst l)})
+                   
+  | Lstaticraise (i,args) ->  Lstaticraise (i, List.map subst args)
+  | Lstaticcatch(e1, io, e2) -> Lstaticcatch(subst e1, io, subst e2)
   | Ltrywith(e1, exn, e2) -> Ltrywith(subst e1, exn, subst e2)
   | Lifthenelse(e1, e2, e3) -> Lifthenelse(subst e1, subst e2, subst e3)
   | Lsequence(e1, e2) -> Lsequence(subst e1, subst e2)
@@ -284,3 +314,23 @@ let subst_lambda s lam =
   and subst_decl (id, exp) = (id, subst exp)
   and subst_case (key, case) = (key, subst case)
   in subst lam
+
+
+(* To let-bind expressions to variables *)
+
+let bind str var exp body =
+  match exp with
+    Lvar var' when Ident.same var var' -> body
+  | _ -> Llet(str, var, exp, body)
+
+and commute_comparison = function
+| Ceq -> Ceq| Cneq -> Cneq
+| Clt -> Cgt | Cle -> Cge
+| Cgt -> Clt | Cge -> Cle
+
+and negate_comparison = function
+| Ceq -> Cneq| Cneq -> Ceq
+| Clt -> Cge | Cle -> Cgt
+| Cgt -> Cle | Cge -> Clt
+
+
