@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: lambda.mli,v 1.32 2000/03/05 19:17:53 xleroy Exp $ *)
+(* $Id: lambda.mli,v 1.35 2001/02/19 20:27:31 maranget Exp $ *)
 
 (* The "lambda" intermediate code *)
 
@@ -57,6 +57,8 @@ type primitive =
   | Parraysets of array_kind
   (* Test if the argument is a block or an immediate integer *)
   | Pisint
+  (* Test if the (integer) argument is outside an interval *)
+  | Pisout
   (* Bitvect operations *)
   | Pbittest
   (* Operations on boxed integers (Nativeint.t, Int32.t, Int64.t) *)
@@ -132,8 +134,8 @@ type lambda =
   | Lletrec of (Ident.t * lambda) list * lambda
   | Lprim of primitive * lambda list
   | Lswitch of lambda * lambda_switch
-  | Lstaticfail
-  | Lcatch of lambda * lambda
+  | Lstaticraise of int * lambda list
+  | Lstaticcatch of lambda * (int * Ident.t list) * lambda
   | Ltrywith of lambda * Ident.t * lambda
   | Lifthenelse of lambda * lambda * lambda
   | Lsequence of lambda * lambda
@@ -149,8 +151,7 @@ and lambda_switch =
     sw_consts: (int * lambda) list;     (* Integer cases *)
     sw_numblocks: int;                  (* Number of tag block cases *)
     sw_blocks: (int * lambda) list;     (* Tag block cases *)
-    sw_checked: bool }                  (* True if bound checks needed *)
-
+    sw_failaction : lambda option}      (* Action to take if failure *)
 and lambda_event =
   { lev_loc: int;
     lev_kind: lambda_event_kind;
@@ -167,6 +168,7 @@ val lambda_unit: lambda
 val name_lambda: lambda -> (Ident.t -> lambda) -> lambda
 val name_lambda_list: lambda list -> (lambda list -> lambda) -> lambda
 val is_guarded: lambda -> bool
+val patch_guarded : lambda -> lambda -> lambda
 
 module IdentSet: Set.S with type elt = Ident.t
 val free_variables: lambda -> IdentSet.t
@@ -175,3 +177,22 @@ val transl_path: Path.t -> lambda
 val make_sequence: ('a -> lambda) -> 'a list -> lambda
 
 val subst_lambda: lambda Ident.tbl -> lambda -> lambda
+val bind : let_kind -> Ident.t -> lambda -> lambda -> lambda
+
+val commute_comparison : comparison -> comparison
+val negate_comparison : comparison -> comparison
+
+(***********************)
+(* For static failures *)
+(***********************)
+
+(* Get a new static failure ident *)
+val next_raise_count : unit -> int
+
+
+val staticfail : lambda (* Anticipated static failure *)
+
+(* Check anticipated failure, substitute its final value *)
+val is_guarded: lambda -> bool 
+val patch_guarded : lambda -> lambda -> lambda 
+
