@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: matching.ml,v 1.48 2001/07/27 14:40:57 maranget Exp $ *)
+(* $Id: matching.ml,v 1.50 2001/09/25 09:54:13 garrigue Exp $ *)
 
 (* Compilation of pattern matching *)
 
@@ -127,7 +127,7 @@ let filter_matrix matcher pss =
   let rec filter_rec = function
     | (p::ps)::rem ->
         begin match p.pat_desc with
-        | Tpat_or (p1,p2) ->
+        | Tpat_or (p1,p2,_) ->
             filter_rec ((p1::ps)::(p2::ps)::rem)
         | Tpat_alias (p,_) ->
             filter_rec ((p::ps)::rem)
@@ -203,7 +203,7 @@ let filter_ctx q ctx =
   let rec filter_rec = function
     | ({right=p::ps} as l)::rem ->
         begin match p.pat_desc with
-        | Tpat_or (p1,p2) ->
+        | Tpat_or (p1,p2,_) ->
             filter_rec ({l with right=p1::ps}::{l with right=p2::ps}::rem)
         | Tpat_alias (p,_) ->
             filter_rec ({l with right=p::ps}::rem)
@@ -477,7 +477,7 @@ let up_ok (ps,act_p) l =
 exception Same
 
 let rec what_is_or = function
-  | {pat_desc = Tpat_or (p,_)} -> what_is_or p
+  | {pat_desc = Tpat_or (p,_,_)} -> what_is_or p
   | {pat_desc = (Tpat_alias (p,_))} -> what_is_or p        
   | {pat_desc=(Tpat_var _|Tpat_any)} -> fatal_error "Matching.what_is_or"
   | p -> p
@@ -501,8 +501,8 @@ let simplify_or p =
         with
         | Var q -> raise (Var {p with pat_desc = Tpat_alias (q,id)})
         end
-    | {pat_desc = Tpat_or (p1,p2)} ->
-        {p with pat_desc = Tpat_or (simpl_rec p1, simpl_rec p2)}
+    | {pat_desc = Tpat_or (p1,p2,_)} ->
+        {p with pat_desc = Tpat_or (simpl_rec p1, simpl_rec p2, None)}
     | {pat_desc = Tpat_record lbls} ->
         let all_lbls = all_record_args lbls in
         {p with pat_desc=Tpat_record all_lbls}        
@@ -540,10 +540,10 @@ let simplify_matching m = match m.args with
               record_ex_pat full_pat ;
               (full_pat::patl,action)::
               simplify rem
-          | Tpat_or (_,_) ->
+          | Tpat_or _ ->
               let pat_simple  = simplify_or pat in
               begin match pat_simple.pat_desc with
-              | Tpat_or (_,_) ->
+              | Tpat_or _ ->
                   let ex_pat = what_is_or pat_simple in
                   record_ex_pat ex_pat ;
                   (pat_simple :: patl, action) ::
@@ -611,7 +611,7 @@ let rec extract_vars r p = match p.pat_desc with
 | Tpat_array pats ->
     List.fold_left extract_vars r pats
 | Tpat_variant (_,Some p, _) -> extract_vars r p
-| Tpat_or (p,_) -> extract_vars r p
+| Tpat_or (p,_,_) -> extract_vars r p
 | Tpat_constant _|Tpat_any|Tpat_variant (_,None,_) -> r
 
 exception Cannot_flatten
@@ -628,7 +628,7 @@ let mk_alpha_env arg aliases ids =
     ids
 
 let rec explode_or_pat arg patl mk_action rem vars aliases = function
-  | {pat_desc = Tpat_or (p1,p2)} ->
+  | {pat_desc = Tpat_or (p1,p2,_)} ->
       explode_or_pat
         arg patl mk_action
         (explode_or_pat arg patl mk_action rem vars aliases p2)
@@ -661,7 +661,7 @@ let compile_or argo cl clor al def = match clor with
       to_catch = []}
 | _  ->
     let rec do_cases = function
-      | ({pat_desc=Tpat_or (_,_)} as orp::patl, action)::rem ->
+      | ({pat_desc=Tpat_or _} as orp::patl, action)::rem ->
           let others,rem = get_equiv orp rem in          
           let orpm =
             {cases =
@@ -757,7 +757,7 @@ let all_vars _ = false
 *)
 
 let is_or p = match p.pat_desc with
-| Tpat_or (_,_) -> true
+| Tpat_or _ -> true
 | _ -> false
 
 (* Conditions for appending to the Or matrix *)
@@ -767,7 +767,7 @@ and condb act ps qs =  not (is_guarded act) && Parmatch.le_pats qs ps
 let or_ok p ps l =
   List.for_all
     (function
-      | ({pat_desc=Tpat_or (_,_)} as q::qs,act) ->
+      | ({pat_desc=Tpat_or _} as q::qs,act) ->
           conda p q || condb act ps qs
       | _ -> true)
     l
@@ -1409,6 +1409,15 @@ open Switch
 
 let lambda_of_int i =  Lconst (Const_base (Const_int i))
 
+let rec last def = function
+| [] -> def
+| [x,_] -> x
+| _::rem -> last def rem
+
+let get_edges low high l = match l with
+| [] -> low, high
+| (x,_)::_ -> x, last high l
+
   
 let as_interval_canfail fail low high l =
   let store = mk_store equal_action in
@@ -1453,7 +1462,7 @@ let as_interval_canfail fail low high l =
             nofail_rec i i index rem in
 
   ignore (store.act_store fail) ; (* fail has action index 0 *)
-  let r = init_rec (sort_lambda_list l) in
+  let r = init_rec l in
   Array.of_list r,  store.act_get ()
 
 let as_interval_nofail l =
@@ -1469,21 +1478,25 @@ let as_interval_nofail l =
         else
           (cur_low, cur_high, cur_act)::
           i_rec i i act_index rem in
-  let inters = match sort_lambda_list l with
+  let inters = match l with
   | (i,act)::rem ->
       let act_index = store.act_store act in
       i_rec i i act_index rem
   | _ -> assert false in
+
   Array.of_list inters, store.act_get ()
 
-let as_interval fail low high l = match fail with
-| None -> as_interval_nofail l
-| Some act -> as_interval_canfail act low high l
+let as_interval fail low high l =
+  let l = sort_lambda_list l in
+  get_edges low high l,
+  (match fail with
+  | None -> as_interval_nofail l
+  | Some act -> as_interval_canfail act low high l)
 
 let call_switcher konst fail arg low high int_lambda_list =
-  let cases, actions =
+  let edges, (cases, actions) =
     as_interval fail low high int_lambda_list in
-  Switcher.zyva konst arg cases actions
+  Switcher.zyva edges konst arg cases actions
 
 
 let exists_ctx ok ctx =
@@ -1497,11 +1510,11 @@ let rec list_as_pat = function
   | [] -> fatal_error "Matching.list_as_pat"
   | [pat] -> pat
   | pat::rem ->
-      {pat with pat_desc = Tpat_or (pat,list_as_pat rem)}
+      {pat with pat_desc = Tpat_or (pat,list_as_pat rem,None)}
 
 
 let rec pat_as_list k = function
-  | {pat_desc=Tpat_or (p1,p2)} ->
+  | {pat_desc=Tpat_or (p1,p2,_)} ->
       pat_as_list (pat_as_list k p2) p1
   | p -> p::k
 
@@ -1509,7 +1522,7 @@ let rec pat_as_list k = function
 exception All
 
 let rec extract_pat seen k p = match p.pat_desc with
-| Tpat_or (p1,p2) ->
+| Tpat_or (p1,p2,_) ->
     let k1,seen1 = extract_pat seen k p1 in
     extract_pat seen1 k1 p2
 | Tpat_alias (p,_) ->
@@ -1735,7 +1748,7 @@ let combine_constructor arg ex_pat cstr partial ctx def
   end
 
 let make_test_sequence_variant_constant fail arg int_lambda_list =
-  let cases, actions =
+  let _, (cases, actions) =
     as_interval fail min_int max_int int_lambda_list in
   Switcher.test_sequence
     (fun i -> Lconst (Const_base (Const_int i))) arg cases actions
@@ -1745,6 +1758,7 @@ let call_switcher_variant_constant fail arg int_lambda_list =
     (fun i -> Lconst (Const_base (Const_int i)))
     fail arg min_int max_int int_lambda_list
 
+
 let call_switcher_variant_constr fail arg int_lambda_list =
   let v = Ident.create "variant" in
   Llet(Alias, v, Lprim(Pfield 0, [arg]),
@@ -2161,7 +2175,7 @@ let flatten_pattern size p =
 let rec flatten_pat_line size p k = match p.pat_desc with
 | Tpat_any ->  omegas size::k
 | Tpat_tuple args -> args::k
-| Tpat_or (p1,p2) ->
+| Tpat_or (p1,p2,_) ->
     flatten_pat_line size p1 (flatten_pat_line size p2 k)
 | _ -> fatal_error "Matching.flatten_pat_line"
 
