@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: symtable.mli,v 1.10 1999/11/17 18:57:02 xleroy Exp $ *)
+(* $Id: symtable.mli,v 1.11 2000/03/06 22:11:11 weis Exp $ *)
 
 (* Assign locations and numbers to globals and primitives *)
 
@@ -50,4 +50,6 @@ type error =
 
 exception Error of error
 
-val report_error: error -> unit
+open Format
+
+val report_error: formatter -> error -> unit
