@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: translclass.mli,v 1.6 1999/11/17 18:57:02 xleroy Exp $ *)
+(* $Id: translclass.mli,v 1.7 2000/03/06 22:11:12 weis Exp $ *)
 
 open Typedtree
 open Lambda
@@ -23,4 +23,6 @@ type error = Illegal_class_expr
 
 exception Error of Location.t * error
 
-val report_error: error -> unit
+open Format
+
+val report_error: formatter -> error -> unit
