@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: translcore.ml,v 1.71 2000/03/06 22:11:13 weis Exp $ *)
+(* $Id: translcore.ml,v 1.77 2001/02/19 20:27:40 maranget Exp $ *)
 
 (* Translation from typed abstract syntax to lambda terms,
    for the core language *)
@@ -239,7 +239,7 @@ let transl_prim prim args =
     | [{exp_desc = Texp_construct({cstr_tag = Cstr_constant _}, _)}; arg2] ->
         intcomp
     | [arg1; arg2] when has_base_type arg1 Predef.path_int
-                     or has_base_type arg1 Predef.path_char ->
+                     || has_base_type arg1 Predef.path_char ->
         intcomp
     | [arg1; arg2] when has_base_type arg1 Predef.path_float ->
         floatcomp
@@ -276,6 +276,7 @@ let transl_prim prim args =
   with Not_found ->
     Pccall prim
 
+
 (* Eta-expand a primitive without knowing the types of its arguments *)
 
 let transl_primitive p =
@@ -401,8 +402,10 @@ let rec push_defaults loc bindings pat_expr_list partial =
 
 (* Insertion of debugging events *)
 
-let event_before exp lam =
-  if !Clflags.debug && lam <> Lstaticfail
+let event_before exp lam = match lam with
+| Lstaticraise (_,_) -> lam
+| _ ->
+  if !Clflags.debug
   then Levent(lam, {lev_loc = exp.exp_loc.Location.loc_start;
                     lev_kind = Lev_before;
                     lev_repr = None;
@@ -430,6 +433,15 @@ let event_function exp lam =
     lam None
 
 
+let assert_failed loc =
+  Lprim(Praise, [Lprim(Pmakeblock(0, Immutable),
+          [transl_path Predef.path_assert_failure;
+           Lconst(Const_block(0,
+              [Const_base(Const_string !Location.input_name);
+               Const_base(Const_int loc.Location.loc_start);
+               Const_base(Const_int loc.Location.loc_end)]))])])
+;;
+
 (* Translation of expressions *)
 
 let rec transl_exp e =
@@ -453,11 +465,18 @@ let rec transl_exp e =
       in
       Lfunction(kind, params, body)
   | Texp_apply({exp_desc = Texp_ident(path, {val_kind = Val_prim p})}, args)
-    when List.length args = p.prim_arity && List.for_all ((<>) None) args ->
-      let args = List.map (function Some x -> x | None -> assert false) args in
+    when List.length args = p.prim_arity
+    && List.for_all (fun (arg,_) -> arg <> None) args ->
+      let args = List.map (function Some x, _ -> x | _ -> assert false) args in
       let prim = transl_prim p args in
-      let lam = Lprim(prim, transl_list args) in
-      begin match prim with Pccall _ -> event_after e lam | _ -> lam end
+      begin match (prim, args) with
+        (Praise, [arg1]) ->
+          Lprim(Praise, [event_after arg1 (transl_exp arg1)])
+      | (Pccall _, _) ->
+          event_after e (Lprim(prim, transl_list args))
+      | (_, _) ->
+          Lprim(prim, transl_list args)
+      end
   | Texp_apply(funct, oargs) ->
       event_after e (transl_apply (transl_exp funct) oargs)
   | Texp_match({exp_desc = Texp_tuple argl} as arg, pat_expr_list, partial) ->
@@ -558,7 +577,7 @@ let rec transl_exp e =
   | Texp_when(cond, body) ->
       event_before cond
         (Lifthenelse(transl_exp cond, event_before body (transl_exp body),
-                     Lstaticfail))
+                     staticfail))
   | Texp_send(expr, met) ->
       let met_id =
         match met with
@@ -583,6 +602,11 @@ let rec transl_exp e =
              (Lvar cpy))
   | Texp_letmodule(id, modl, body) ->
       Llet(Strict, id, !transl_module Tcoerce_none None modl, transl_exp body)
+  | Texp_assert (cond) ->
+      if !Clflags.noassert
+      then lambda_unit
+      else Lifthenelse (transl_exp cond, lambda_unit, assert_failed e.exp_loc)
+  | Texp_assertfalse -> assert_failed e.exp_loc
   | _ ->
       fatal_error "Translcore.transl"
 
@@ -610,7 +634,7 @@ and transl_apply lam sargs =
         Lapply(lexp, args)
   in
   let rec build_apply lam args = function
-      None :: l ->
+      (None, optional) :: l ->
         let defs = ref [] in
         let protect name lam =
           match lam with
@@ -620,13 +644,16 @@ and transl_apply lam sargs =
               defs := (id, lam) :: !defs;
               Lvar id
         in
+        let args, args' =
+          if List.for_all (fun (_,opt) -> opt = Optional) args then [], args
+          else args, [] in
         let lam =
-          if args = [] then lam else lapply lam (List.rev args) in
+          if args = [] then lam else lapply lam (List.rev_map fst args) in
         let handle = protect "func" lam
-        and l = List.map (may_map (protect "arg")) l
+        and l = List.map (fun (arg, opt) -> may_map (protect "arg") arg, opt) l
         and id_arg = Ident.create "param" in
         let body =
-          match build_apply handle [Lvar id_arg] l with
+          match build_apply handle ((Lvar id_arg, optional)::args') l with
             Lfunction(Curried, ids, lam) ->
               Lfunction(Curried, id_arg::ids, lam)
           | Levent(Lfunction(Curried, ids, lam), _) ->
@@ -637,12 +664,12 @@ and transl_apply lam sargs =
         List.fold_left
           (fun body (id, lam) -> Llet(Strict, id, lam, body))
           body !defs
-    | Some arg :: l ->
-        build_apply lam (arg :: args) l
+    | (Some arg, optional) :: l ->
+        build_apply lam ((arg, optional) :: args) l
     | [] ->
-        lapply lam (List.rev args)
+        lapply lam (List.rev_map fst args)
   in
-  build_apply lam [] (List.map (may_map transl_exp) sargs)
+  build_apply lam [] (List.map (fun (x,o) -> may_map transl_exp x, o) sargs)
 
 and transl_function loc untuplify_fn repr partial pat_expr_list =
   match pat_expr_list with
@@ -736,7 +763,7 @@ and transl_record all_labels repres lbl_expr_list opt_init_expr =
   then begin
     (* Allocate new record with given fields (and remaining fields
        taken from init_expr if any *)
-    let lv = Array.create (Array.length all_labels) Lstaticfail in
+    let lv = Array.create (Array.length all_labels) staticfail in
     let init_id = Ident.create "init" in
     begin match opt_init_expr with
       None -> ()
