@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: typeopt.ml,v 1.5 2000/02/28 15:45:50 xleroy Exp $ *)
+(* $Id: typeopt.ml,v 1.6 2001/01/13 11:55:09 xleroy Exp $ *)
 
 (* Auxiliaries for type-based optimizations, e.g. array kinds *)
 
@@ -59,7 +59,10 @@ let array_element_kind env ty =
       else if Path.same p Predef.path_float then
         Pfloatarray
       else if Path.same p Predef.path_string
-           || Path.same p Predef.path_array then
+           || Path.same p Predef.path_array 
+           || Path.same p Predef.path_nativeint
+           || Path.same p Predef.path_int32
+           || Path.same p Predef.path_int64 then
         Paddrarray
       else begin
         try
