@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: alloc.c,v 1.20 2000/11/26 19:29:24 doligez Exp $ */
+/* $Id: alloc.c,v 1.24 2001/12/07 13:39:22 xleroy Exp $ */
 
 /* 1. Allocation functions doing the same work as the macros in the
       case where [Setup_for_gc] and [Restore_after_gc] are no-ops.
@@ -28,14 +29,15 @@
 #define Setup_for_gc
 #define Restore_after_gc
 
-value alloc (mlsize_t wosize, tag_t tag)
+CAMLexport value alloc (mlsize_t wosize, tag_t tag)
 {
   value result;
   mlsize_t i;
 
-  Assert (wosize > 0);
   Assert (tag < 256);
-  if (wosize <= Max_young_wosize){
+  if (wosize == 0){
+    result = Atom (tag);
+  }else if (wosize <= Max_young_wosize){
     Alloc_small (result, wosize, tag);
     if (tag < No_scan_tag){
       for (i = 0; i < wosize; i++) Field (result, i) = 0;
@@ -48,7 +50,7 @@ value alloc (mlsize_t wosize, tag_t tag)
   return result;
 }
 
-value alloc_small (mlsize_t wosize, tag_t tag)
+CAMLexport value alloc_small (mlsize_t wosize, tag_t tag)
 {
   value result;
 
@@ -59,12 +61,12 @@ value alloc_small (mlsize_t wosize, tag_t tag)
   return result;
 }
 
-value alloc_tuple(mlsize_t n)
+CAMLexport value alloc_tuple(mlsize_t n)
 {
   return alloc(n, 0);
 }
 
-value alloc_string (mlsize_t len)
+CAMLexport value alloc_string (mlsize_t len)
 {
   value result;
   mlsize_t offset_index;
@@ -82,13 +84,14 @@ value alloc_string (mlsize_t len)
   return result;
 }
 
-value alloc_final (mlsize_t len, final_fun fun, mlsize_t mem, mlsize_t max)
+CAMLexport value alloc_final (mlsize_t len, final_fun fun,
+                              mlsize_t mem, mlsize_t max)
 {
   return alloc_custom(final_custom_operations(fun),
                       len * sizeof(value), mem, max);
 }
 
-value copy_string(char const *s)
+CAMLexport value copy_string(char const *s)
 {
   int len;
   value res;
@@ -99,7 +102,7 @@ value copy_string(char const *s)
   return res;
 }
 
-value alloc_array(value (*funct)(char const *), char const ** arr)
+CAMLexport value alloc_array(value (*funct)(char const *), char const ** arr)
 {
   CAMLparam0 ();
   mlsize_t nbr, n;
@@ -122,12 +125,12 @@ value alloc_array(value (*funct)(char const *), char const ** arr)
   }
 }
 
-value copy_string_array(char const ** arr)
+CAMLexport value copy_string_array(char const ** arr)
 {
   return alloc_array(copy_string, arr);
 }
 
-int convert_flag_list(value list, int *flags)
+CAMLexport int convert_flag_list(value list, int *flags)
 {
   int res;
   res = 0;
@@ -140,7 +143,7 @@ int convert_flag_list(value list, int *flags)
 
 /* For compiling let rec over values */
 
-value alloc_dummy(value size) /* ML */
+CAMLprim value alloc_dummy(value size)
 {
   mlsize_t wosize = Int_val(size);
 
@@ -148,7 +151,7 @@ value alloc_dummy(value size) /* ML */
   return alloc (wosize, 0);
 }
 
-value update_dummy(value dummy, value newval) /* ML */
+CAMLprim value update_dummy(value dummy, value newval)
 {
   mlsize_t size, i;
   size = Wosize_val(newval);
