@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: callback.c,v 1.12 2001/05/28 15:34:49 xleroy Exp $ */
+/* $Id: callback.c,v 1.14 2001/12/07 13:39:22 xleroy Exp $ */
 
 /* Callbacks from C to Caml */
 
@@ -51,7 +52,7 @@ static void thread_callback(void)
 
 #endif
 
-value callbackN_exn(value closure, int narg, value args[])
+CAMLexport value callbackN_exn(value closure, int narg, value args[])
 {
   int i;
 
@@ -68,14 +69,14 @@ value callbackN_exn(value closure, int narg, value args[])
   return interprete(callback_code, sizeof(callback_code));
 }
 
-value callback_exn(value closure, value arg1)
+CAMLexport value callback_exn(value closure, value arg1)
 {
   value arg[1];
   arg[0] = arg1;
   return callbackN_exn(closure, 1, arg);
 }
 
-value callback2_exn(value closure, value arg1, value arg2)
+CAMLexport value callback2_exn(value closure, value arg1, value arg2)
 {
   value arg[2];
   arg[0] = arg1;
@@ -83,7 +84,8 @@ value callback2_exn(value closure, value arg1, value arg2)
   return callbackN_exn(closure, 2, arg);
 }
 
-value callback3_exn(value closure, value arg1, value arg2, value arg3)
+CAMLexport value callback3_exn(value closure,
+                               value arg1, value arg2, value arg3)
 {
   value arg[3];
   arg[0] = arg1;
@@ -96,7 +98,7 @@ value callback3_exn(value closure, value arg1, value arg2, value arg3)
 
 /* Native-code callbacks.  callback[123]_exn are implemented in asm. */
 
-value callbackN_exn(value closure, int narg, value args[])
+CAMLexport value callbackN_exn(value closure, int narg, value args[])
 {
   CAMLparam1 (closure);
   CAMLxparamN (args, narg);
@@ -131,28 +133,28 @@ value callbackN_exn(value closure, int narg, value args[])
 
 /* Exception-propagating variants of the above */
 
-value callback (value closure, value arg)
+CAMLexport value callback (value closure, value arg)
 {
   value res = callback_exn(closure, arg);
   if (Is_exception_result(res)) mlraise(Extract_exception(res));
   return res;
 }
 
-value callback2 (value closure, value arg1, value arg2)
+CAMLexport value callback2 (value closure, value arg1, value arg2)
 {
   value res = callback2_exn(closure, arg1, arg2);
   if (Is_exception_result(res)) mlraise(Extract_exception(res));
   return res;
 }
 
-value callback3 (value closure, value arg1, value arg2, value arg3)
+CAMLexport value callback3 (value closure, value arg1, value arg2, value arg3)
 {
   value res = callback3_exn(closure, arg1, arg2, arg3);
   if (Is_exception_result(res)) mlraise(Extract_exception(res));
   return res;
 }
 
-value callbackN (value closure, int narg, value args[])
+CAMLexport value callbackN (value closure, int narg, value args[])
 {
   value res = callbackN_exn(closure, narg, args);
   if (Is_exception_result(res)) mlraise(Extract_exception(res));
@@ -178,7 +180,7 @@ static unsigned int hash_value_name(char *name)
   return h % Named_value_size;
 }
 
-value register_named_value(value vname, value val) /* ML */
+CAMLprim value register_named_value(value vname, value val)
 {
   struct named_value * nv;
   char * name = String_val(vname);
@@ -194,7 +196,7 @@ value register_named_value(value vname, value val) /* ML */
   return Val_unit;
 }
 
-value * caml_named_value(char *name)
+CAMLexport value * caml_named_value(char *name)
 {
   struct named_value * nv;
   for (nv = named_value_table[hash_value_name(name)];
