@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: compare.c,v 1.19 2001/02/13 09:29:08 xleroy Exp $ */
+/* $Id: compare.c,v 1.22 2001/12/07 13:39:23 xleroy Exp $ */
 
 #include <string.h>
 #include <stdlib.h>
@@ -25,7 +26,7 @@
 struct compare_item { value * v1, * v2; mlsize_t count; };
 
 #define COMPARE_STACK_INIT_SIZE 256
-#define COMPARE_STACK_MAX_SIZE 1024*1024
+#define COMPARE_STACK_MAX_SIZE (1024*1024)
 
 static struct compare_item compare_stack_init[COMPARE_STACK_INIT_SIZE];
 
@@ -182,7 +183,7 @@ static long compare_val(value v1, value v2)
   }
 }
 
-value compare(value v1, value v2)           /* ML */
+CAMLprim value compare(value v1, value v2)
 {
   long res = compare_val(v1, v2);
   /* Free stack if needed */
@@ -195,42 +196,42 @@ value compare(value v1, value v2)           /* ML */
     return Val_int(0);
 }
 
-value equal(value v1, value v2)            /* ML */
+CAMLprim value equal(value v1, value v2)
 {
   long res = compare_val(v1, v2);
   if (compare_stack != compare_stack_init) compare_free_stack();
   return Val_int(res == 0);
 }
 
-value notequal(value v1, value v2)            /* ML */
+CAMLprim value notequal(value v1, value v2)
 {
   long res = compare_val(v1, v2);
   if (compare_stack != compare_stack_init) compare_free_stack();
   return Val_int(res != 0);
 }
 
-value lessthan(value v1, value v2)            /* ML */
+CAMLprim value lessthan(value v1, value v2)
 {
   long res = compare_val(v1, v2);
   if (compare_stack != compare_stack_init) compare_free_stack();
   return Val_int(res < 0);
 }
 
-value lessequal(value v1, value v2)          /* ML */
+CAMLprim value lessequal(value v1, value v2)
 {
   long res = compare_val(v1, v2);
   if (compare_stack != compare_stack_init) compare_free_stack();
   return Val_int(res <= 0);
 }
 
-value greaterthan(value v1, value v2)        /* ML */
+CAMLprim value greaterthan(value v1, value v2)
 {
   long res = compare_val(v1, v2);
   if (compare_stack != compare_stack_init) compare_free_stack();
   return Val_int(res > 0);
 }
 
-value greaterequal(value v1, value v2)       /* ML */
+CAMLprim value greaterequal(value v1, value v2)
 {
   long res = compare_val(v1, v2);
   if (compare_stack != compare_stack_init) compare_free_stack();
