@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: debugger.c,v 1.20 2000/10/12 18:05:38 doligez Exp $ */
+/* $Id: debugger.c,v 1.23 2001/12/07 13:39:23 xleroy Exp $ */
 
 /* Interface with the debugger */
 
@@ -72,8 +73,8 @@ static void open_connection(void)
   if (dbg_socket == -1 ||
       connect(dbg_socket, &sock_addr.s_gen, sock_addr_len) == -1)
     fatal_error("cannot connect to debugger");
-  dbg_in = open_descriptor(dbg_socket);
-  dbg_out = open_descriptor(dbg_socket);
+  dbg_in = open_descriptor_in(dbg_socket);
+  dbg_out = open_descriptor_out(dbg_socket);
   if (!debugger_in_use) putword(dbg_out, -1); /* first connection */
   putword(dbg_out, getpid());
   flush(dbg_out);
@@ -160,10 +161,10 @@ static void safe_output_value(struct channel *chan, value val)
   external_raise = saved_external_raise;
 }
 
-#define Pc(sp) ((code_t)(sp[0]))
-#define Env(sp) (sp[1])
-#define Extra_args(sp) (Long_val((sp[2])))
-#define Locals(sp) (sp + 3)
+#define Pc(sp) ((code_t)((sp)[0]))
+#define Env(sp) ((sp)[1])
+#define Extra_args(sp) (Long_val(((sp)[2])))
+#define Locals(sp) ((sp) + 3)
 
 void debugger(enum event_kind event)
 {
