@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 2000 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: finalise.c,v 1.4 2000/08/23 17:10:00 doligez Exp $ */
+/* $Id: finalise.c,v 1.7 2001/12/07 13:39:26 xleroy Exp $ */
 
 /* Handling of finalised values. */
 
@@ -77,7 +78,7 @@ void final_do_calls (void)
 }
 
 /* Call a scanning_action [f] on [x]. */
-#define Call_action(f,x) (*f) (x, &(x))
+#define Call_action(f,x) (*(f)) ((x), &(x))
 
 /* Call [*f] on the closures of the finalisable set and
    the closures and values of the finalising set.
@@ -134,7 +135,7 @@ void final_empty_young (void)
 }
 
 /* Put (f,v) in the recent set. */
-value final_register (value f, value v)       /* ML */
+CAMLprim value final_register (value f, value v)
 {
   if (!(Is_block (v) && (Is_in_heap (v) || Is_young (v)))){
     invalid_argument ("Gc.finalise");
