@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: gc_ctrl.c,v 1.28 2001/02/19 10:01:40 xleroy Exp $ */
+/* $Id: gc_ctrl.c,v 1.31 2001/12/07 13:39:28 xleroy Exp $ */
 
 #include "alloc.h"
 #include "compact.h"
@@ -204,16 +205,23 @@ static value heap_stats (int returnstats)
   if (returnstats){
     CAMLlocal1 (res);
 
+    /* get a copy of these before allocating anything... */
+    double minwords = stat_minor_words
+                      + (double) Wsize_bsize (young_end - young_ptr);
+    double prowords = stat_promoted_words;
+    double majwords = stat_major_words + (double) allocated_words;
+    long mincoll = stat_major_collections;
+    long majcoll = stat_minor_collections;
+    long heapsz = stat_heap_size;
+    long cpct = stat_compactions;
+
     res = alloc_tuple (14);
-    Store_field (res, 0,
-                 copy_double (stat_minor_words
-                              + (double) Wsize_bsize (young_end - young_ptr)));
-    Store_field (res, 1, copy_double (stat_promoted_words));
-    Store_field (res, 2,
-                 copy_double (stat_major_words + (double) allocated_words));
-    Store_field (res, 3, Val_long (stat_minor_collections));
-    Store_field (res, 4, Val_long (stat_major_collections));
-    Store_field (res, 5, Val_long (Wsize_bsize (stat_heap_size)));
+    Store_field (res, 0, copy_double (minwords));
+    Store_field (res, 1, copy_double (prowords));
+    Store_field (res, 2, copy_double (majwords));
+    Store_field (res, 3, Val_long (mincoll));
+    Store_field (res, 4, Val_long (majcoll));
+    Store_field (res, 5, Val_long (heapsz));
     Store_field (res, 6, Val_long (heap_chunks));
     Store_field (res, 7, Val_long (live_words));
     Store_field (res, 8, Val_long (live_blocks));
@@ -221,7 +229,7 @@ static value heap_stats (int returnstats)
     Store_field (res, 10, Val_long (free_blocks));
     Store_field (res, 11, Val_long (largest_free));
     Store_field (res, 12, Val_long (fragments));
-    Store_field (res, 13, Val_long (stat_compactions));
+    Store_field (res, 13, Val_long (cpct));
     CAMLreturn (res);
   }else{
     CAMLreturn (Val_unit);
@@ -235,28 +243,31 @@ void heap_check (void)
 }
 #endif
 
-value gc_stat(value v) /* ML */
+CAMLprim value gc_stat(value v)
 {
   Assert (v == Val_unit);
   return heap_stats (1);
 }
 
-value gc_counters(value v) /* ML */
+CAMLprim value gc_counters(value v)
 {
   CAMLparam0 ();   /* v is ignored */
   CAMLlocal1 (res);
 
+  /* get a copy of these before allocating anything... */
+  double minwords = stat_minor_words
+                    + (double) Wsize_bsize (young_end - young_ptr);
+  double prowords = stat_promoted_words;
+  double majwords = stat_major_words + (double) allocated_words;
+
   res = alloc_tuple (3);
-  Store_field (res, 0,
-               copy_double (stat_minor_words
-                            + (double) Wsize_bsize (young_end - young_ptr)));
-  Store_field (res, 1, copy_double (stat_promoted_words));
-  Store_field (res, 2,
-               copy_double (stat_major_words + (double) allocated_words));
+  Store_field (res, 0, copy_double (minwords));
+  Store_field (res, 1, copy_double (prowords));
+  Store_field (res, 2, copy_double (majwords));
   CAMLreturn (res);
 }
 
-value gc_get(value v) /* ML */
+CAMLprim value gc_get(value v)
 {
   CAMLparam0 ();   /* v is ignored */
   CAMLlocal1 (res);
@@ -303,7 +314,7 @@ static long norm_minsize (long int s)
   return s;
 }
 
-value gc_set(value v) /* ML */
+CAMLprim value gc_set(value v)
 {
   unsigned long newpf, newpm;
   asize_t newheapincr;
@@ -344,13 +355,13 @@ value gc_set(value v) /* ML */
   return Val_unit;
 }
 
-value gc_minor(value v) /* ML */
+CAMLprim value gc_minor(value v)
 {                                                    Assert (v == Val_unit);
   minor_collection ();
   return Val_unit;
 }
 
-value gc_major(value v) /* ML */
+CAMLprim value gc_major(value v)
 {                                                    Assert (v == Val_unit);
   empty_minor_heap ();
   finish_major_cycle ();
@@ -358,7 +369,7 @@ value gc_major(value v) /* ML */
   return Val_unit;
 }
 
-value gc_full_major(value v) /* ML */
+CAMLprim value gc_full_major(value v)
 {                                                    Assert (v == Val_unit);
   empty_minor_heap ();
   finish_major_cycle ();
@@ -369,7 +380,7 @@ value gc_full_major(value v) /* ML */
   return Val_unit;
 }
 
-value gc_compaction(value v) /* ML */
+CAMLprim value gc_compaction(value v)
 {                                                    Assert (v == Val_unit);
   empty_minor_heap ();
   finish_major_cycle ();
