@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: hash.c,v 1.16 2000/04/14 09:41:56 doligez Exp $ */
+/* $Id: hash.c,v 1.18 2001/12/07 13:39:28 xleroy Exp $ */
 
 /* The generic hashing primitive */
 
@@ -23,7 +24,7 @@ static long hash_univ_limit, hash_univ_count;
 
 static void hash_aux(value obj);
 
-value hash_univ_param(value count, value limit, value obj) /* ML */
+CAMLprim value hash_univ_param(value count, value limit, value obj)
 {
   hash_univ_limit = Long_val(limit);
   hash_univ_count = Long_val(count);
@@ -134,7 +135,7 @@ static void hash_aux(value obj)
 
 /* Hashing variant tags */
 
-value hash_variant(char * tag)
+CAMLexport value hash_variant(char * tag)
 {
   value accu;
   /* Same hashing algorithm as in ../typing/btype.ml, function hash_variant */
