@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: instrtrace.c,v 1.9 2000/10/02 14:17:48 maranget Exp $ */
+/* $Id: instrtrace.c,v 1.11 2001/12/07 13:39:28 xleroy Exp $ */
 
 /* Trace the instructions executed */
 
@@ -23,7 +24,7 @@
 #include "opnames.h"
 
 extern code_t start_code;
-extern char * names_of_cprim[];
+extern char * names_of_builtin_cprim[];
 
 long icount = 0;
 
@@ -59,9 +60,9 @@ void disasm_instr(pc)
     printf(" %d, %d\n", pc[0], pc[1]); break;
     /* Instructions with a C primitive as operand */
   case C_CALL1: case C_CALL2: case C_CALL3: case C_CALL4: case C_CALL5:
-    printf(" %s\n", names_of_cprim[pc[0]]); break;
+    printf(" %s\n", names_of_builtin_cprim[pc[0]]); break; /* REVISE */
   case C_CALLN:
-    printf(" %d, %s\n", pc[0], names_of_cprim[pc[1]]); break;
+    printf(" %d, %s\n", pc[0], names_of_builtin_cprim[pc[1]]); break;
   default:
     printf("\n");
   }
