@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: intern.c,v 1.37 2000/11/30 17:13:41 xleroy Exp $ */
+/* $Id: intern.c,v 1.44 2001/12/07 13:39:29 xleroy Exp $ */
 
 /* Structured input, compact format */
 
@@ -93,7 +94,7 @@ static long read64s(void)
 #endif
 
 #define readblock(dest,len) \
-  (memmove(dest, intern_src, len), intern_src += len)
+  (memmove((dest), intern_src, (len)), intern_src += (len))
 
 static void intern_cleanup(void)
 {
@@ -102,7 +103,7 @@ static void intern_cleanup(void)
   if (intern_extra_block != NULL) {
     /* free newly allocated heap chunk */
     free_for_heap(intern_extra_block); 
-  } else {
+  } else if (intern_block != 0) {
     /* restore original header for heap block, otherwise GC is confused */
     Hd_val(intern_block) = intern_header;
   }
@@ -285,6 +286,8 @@ static void intern_alloc(mlsize_t whsize, mlsize_t num_objects)
 
   if (whsize == 0) {
     intern_obj_table = NULL;
+    intern_extra_block = NULL;
+    intern_block = 0;
     return;
   }
   wosize = Wosize_whsize(whsize);
@@ -297,7 +300,16 @@ static void intern_alloc(mlsize_t whsize, mlsize_t num_objects)
     intern_color = allocation_color(intern_extra_block);
     intern_dest = intern_extra_block;
   } else {
-    intern_block = alloc(wosize, String_tag);
+    /* this is a specialised version of alloc from alloc.c */
+    if (wosize == 0){
+      intern_block = Atom (String_tag);
+    }else if (wosize <= Max_young_wosize){
+      intern_block = alloc_small (wosize, String_tag);
+    }else{
+      intern_block = alloc_shr (wosize, String_tag);
+      /* do not do the urgent_gc check here because it might darken
+         intern_block into gray and break the Assert 3 lines down */
+    }
     intern_header = Hd_val(intern_block);
     intern_color = Color_hd(intern_header);
     Assert (intern_color == Caml_white || intern_color == Caml_black);
@@ -371,7 +383,7 @@ value input_val(struct channel *chan)
   return res;
 }
 
-value input_value(value vchan)        /* ML */
+CAMLprim value input_value(value vchan)
 {
   CAMLparam1 (vchan);
   struct channel * chan = Channel(vchan);
@@ -383,7 +395,7 @@ value input_value(value vchan)        /* ML */
   CAMLreturn (res);
 }
 
-value input_val_from_string(value str, long int ofs)
+CAMLexport value input_val_from_string(value str, long int ofs)
 {
   CAMLparam1 (str);
   mlsize_t num_objects, size_32, size_64, whsize;
@@ -410,19 +422,16 @@ value input_val_from_string(value str, long int ofs)
   CAMLreturn (obj);
 }
 
-value input_value_from_string(value str, value ofs) /* ML */
+CAMLprim value input_value_from_string(value str, value ofs)
 {
   return input_val_from_string(str, Long_val(ofs));
 }
 
-value input_value_from_malloc(char * data, long ofs)
+static value input_val_from_block(void)
 {
   mlsize_t num_objects, size_32, size_64, whsize;
   value obj;
 
-  intern_input = (unsigned char *) data;
-  intern_src = intern_input + ofs + 2*4;
-  intern_input_malloced = 1;
   num_objects = read32u();
   size_32 = read32u();
   size_64 = read32u();
@@ -437,12 +446,49 @@ value input_value_from_malloc(char * data, long ofs)
   intern_rec(&obj);
   intern_add_to_heap(whsize);
   /* Free everything */
+  if (intern_obj_table != NULL) stat_free(intern_obj_table);
+  return obj;
+}
+
+CAMLexport value input_value_from_malloc(char * data, long ofs)
+{
+  mlsize_t magic, block_len;
+  value obj;
+
+  intern_input = (unsigned char *) data;
+  intern_src = intern_input + ofs;
+  intern_input_malloced = 1;
+  magic = read32u();
+  if (magic != Intext_magic_number) 
+    failwith("input_value_from_malloc: bad object");
+  block_len = read32u();
+  obj = input_val_from_block();
+  /* Free the input */
   stat_free(intern_input);
   if (intern_obj_table != NULL) stat_free(intern_obj_table);
   return obj;
 }
 
-value marshal_data_size(value buff, value ofs) /* ML */
+CAMLexport value input_value_from_block(char * data, long len)
+{
+  mlsize_t magic, block_len;
+  value obj;
+
+  intern_input = (unsigned char *) data;
+  intern_src = intern_input;
+  intern_input_malloced = 0;
+  magic = read32u();
+  if (magic != Intext_magic_number) 
+    failwith("input_value_from_block: bad object");
+  block_len = read32u();
+  if (5*4 + block_len > len)
+    failwith("input_value_from_block: bad block length");
+  obj = input_val_from_block();
+  if (intern_obj_table != NULL) stat_free(intern_obj_table);
+  return obj;
+}
+
+CAMLprim value marshal_data_size(value buff, value ofs)
 {
   uint32 magic;
   mlsize_t block_len;
@@ -491,71 +537,71 @@ unsigned char * code_checksum(void)
 
 /* Functions for writing user-defined marshallers */
 
-int deserialize_uint_1(void)
+CAMLexport int deserialize_uint_1(void)
 {
   return read8u();
 }
 
-int deserialize_sint_1(void)
+CAMLexport int deserialize_sint_1(void)
 {
   return read8s();
 }
 
-int deserialize_uint_2(void)
+CAMLexport int deserialize_uint_2(void)
 {
   return read16u();
 }
 
-int deserialize_sint_2(void)
+CAMLexport int deserialize_sint_2(void)
 {
   return read16s();
 }
 
-uint32 deserialize_uint_4(void)
+CAMLexport uint32 deserialize_uint_4(void)
 {
   return read32u();
 }
 
-int32 deserialize_sint_4(void)
+CAMLexport int32 deserialize_sint_4(void)
 {
   return read32s();
 }
 
-uint64 deserialize_uint_8(void)
+CAMLexport uint64 deserialize_uint_8(void)
 {
   uint64 i;
   deserialize_block_8(&i, 1);
   return i;
 }
 
-int64 deserialize_sint_8(void)
+CAMLexport int64 deserialize_sint_8(void)
 {
   int64 i;
   deserialize_block_8(&i, 1);
   return i;
 }
 
-float deserialize_float_4(void)
+CAMLexport float deserialize_float_4(void)
 {
   float f;
   deserialize_block_4(&f, 1);
   return f;
 }
 
-double deserialize_float_8(void)
+CAMLexport double deserialize_float_8(void)
 {
   double f;
   deserialize_block_8(&f, 1);
   return f;
 }
 
-void deserialize_block_1(void * data, long len)
+CAMLexport void deserialize_block_1(void * data, long len)
 {
   memmove(data, intern_src, len);
   intern_src += len;
 }
 
-void deserialize_block_2(void * data, long len)
+CAMLexport void deserialize_block_2(void * data, long len)
 {
   unsigned char * p, * q;
 #ifndef ARCH_BIG_ENDIAN
@@ -568,7 +614,7 @@ void deserialize_block_2(void * data, long len)
 #endif
 }
 
-void deserialize_block_4(void * data, long len)
+CAMLexport void deserialize_block_4(void * data, long len)
 {
   unsigned char * p, * q;
 #ifndef ARCH_BIG_ENDIAN
@@ -581,7 +627,7 @@ void deserialize_block_4(void * data, long len)
 #endif
 }
 
-void deserialize_block_8(void * data, long len)
+CAMLexport void deserialize_block_8(void * data, long len)
 {
   unsigned char * p, * q;
 #ifndef ARCH_BIG_ENDIAN
@@ -594,7 +640,7 @@ void deserialize_block_8(void * data, long len)
 #endif
 }
 
-void deserialize_error(char * msg)
+CAMLexport void deserialize_error(char * msg)
 {
   intern_cleanup();
   failwith(msg);
