@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: interp.c,v 1.66 2001/05/28 15:14:17 xleroy Exp $ */
+/* $Id: interp.c,v 1.70 2001/12/07 13:39:29 xleroy Exp $ */
 
 /* The bytecode interpreter */
 #include <stdio.h>
@@ -169,6 +170,13 @@ sp is a local copy of the global variable extern_sp. */
 #endif
 #endif
 
+/* Division and modulus madness */
+
+#ifdef NONSTANDARD_DIV_MOD
+static long safe_div(long p, long q);
+static long safe_mod(long p, long q);
+#endif
+
 /* The interpreter itself */
 
 value interprete(code_t prog, asize_t prog_size)
@@ -780,12 +788,8 @@ value interprete(code_t prog, asize_t prog_size)
 
     Instruct(RAISE):
     raise_exception:
-      backtrace_pos = 0;
-      /* fallthrough */
-
-    Instruct(RERAISE):
       if (trapsp >= trap_barrier) debugger(TRAP_BARRIER);
-      if (backtrace_active) stash_backtrace(pc, sp);
+      if (backtrace_active) stash_backtrace(accu, pc, sp);
       sp = trapsp;
       if ((char *) sp >= (char *) stack_high - initial_sp_offset) {
         external_raise = initial_external_raise;
@@ -827,34 +831,34 @@ value interprete(code_t prog, asize_t prog_size)
 
     Instruct(C_CALL1):
       Setup_for_c_call;
-      accu = cprim[*pc](accu);
+      accu = Primitive(*pc)(accu);
       Restore_after_c_call;
       pc++;
       Next;
     Instruct(C_CALL2):
       Setup_for_c_call;
-      accu = cprim[*pc](accu, sp[1]);
+      accu = Primitive(*pc)(accu, sp[1]);
       Restore_after_c_call;
       sp += 1;
       pc++;
       Next;
     Instruct(C_CALL3):
       Setup_for_c_call;
-      accu = cprim[*pc](accu, sp[1], sp[2]);
+      accu = Primitive(*pc)(accu, sp[1], sp[2]);
       Restore_after_c_call;
       sp += 2;
       pc++;
       Next;
     Instruct(C_CALL4):
       Setup_for_c_call;
-      accu = cprim[*pc](accu, sp[1], sp[2], sp[3]);
+      accu = Primitive(*pc)(accu, sp[1], sp[2], sp[3]);
       Restore_after_c_call;
       sp += 3;
       pc++;
       Next;
     Instruct(C_CALL5):
       Setup_for_c_call;
-      accu = cprim[*pc](accu, sp[1], sp[2], sp[3], sp[4]);
+      accu = Primitive(*pc)(accu, sp[1], sp[2], sp[3], sp[4]);
       Restore_after_c_call;
       sp += 4;
       pc++;
@@ -863,7 +867,7 @@ value interprete(code_t prog, asize_t prog_size)
       int nargs = *pc++;
       *--sp = accu;
       Setup_for_c_call;
-      accu = cprim[*pc](sp + 1, nargs);
+      accu = Primitive(*pc)(sp + 1, nargs);
       Restore_after_c_call;
       sp += nargs;
       pc++;
@@ -912,13 +916,21 @@ value interprete(code_t prog, asize_t prog_size)
     Instruct(DIVINT): {
       long divisor = Long_val(*sp++);
       if (divisor == 0) { Setup_for_c_call; raise_zero_divide(); }
+#ifdef NONSTANDARD_DIV_MOD
+      accu = Val_long(safe_div(Long_val(accu), divisor));
+#else
       accu = Val_long(Long_val(accu) / divisor);
+#endif
       Next;
     }
     Instruct(MODINT): {
       long divisor = Long_val(*sp++);
       if (divisor == 0) { Setup_for_c_call; raise_zero_divide(); }
+#ifdef NONSTANDARD_DIV_MOD
+      accu = Val_long(safe_mod(Long_val(accu), divisor));
+#else
       accu = Val_long(Long_val(accu) % divisor);
+#endif
       Next;
     }
     Instruct(ANDINT):
@@ -1022,3 +1034,21 @@ value interprete(code_t prog, asize_t prog_size)
   }
 #endif
 }
+
+#ifdef NONSTANDARD_DIV_MOD
+long safe_div(long p, long q)
+{
+  unsigned long ap = p >= 0 ? p : -p;
+  unsigned long aq = q >= 0 ? q : -q;
+  unsigned long ar = ap / aq;
+  return (p ^ q) >= 0 ? ar : -ar;
+}
+
+long safe_mod(long p, long q)
+{
+  unsigned long ap = p >= 0 ? p : -p;
+  unsigned long aq = q >= 0 ? q : -q;
+  unsigned long ar = ap % aq;
+  return p >= 0 ? ar : -ar;
+}
+#endif
