@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: io.h,v 1.13 2000/02/10 14:04:57 xleroy Exp $ */
+/* $Id: io.h,v 1.18 2001/12/07 13:39:30 xleroy Exp $ */
 
 /* Buffered input/output */
 
@@ -32,6 +33,10 @@ struct channel {
   char * curr;                  /* Current position in the buffer */
   char * max;                   /* Logical end of the buffer (for input) */
   void * mutex;                 /* Placeholder for mutex (for systhreads) */
+  struct channel * next;        /* Linear chaining of channels (flush_all) */
+  int revealed;                 /* For Cash only */
+  int old_revealed;             /* For Cash only */
+  int refcount;                 /* For flush_all and for Cash */
   char buff[IO_BUFFER_SIZE];    /* The buffer itself */
 };
 
@@ -44,29 +49,31 @@ struct channel {
 /* Functions and macros that can be called from C.  Take arguments of
    type struct channel *.  No locking is performed. */
 
-#define putch(channel, ch)                                                  \
-  { if ((channel)->curr >= (channel)->end) flush_partial(channel);          \
-    *((channel)->curr)++ = (ch); }
+#define putch(channel, ch) do{                                            \
+  if ((channel)->curr >= (channel)->end) flush_partial(channel);          \
+  *((channel)->curr)++ = (ch);                                            \
+}while(0)
 
 #define getch(channel)                                                      \
   ((channel)->curr >= (channel)->max                                        \
    ? refill(channel)                                                        \
    : (unsigned char) *((channel))->curr++)
 
-struct channel * open_descriptor (int);
-void close_channel (struct channel *);
-int channel_binary_mode (struct channel *);
+CAMLextern struct channel * open_descriptor_in (int);
+CAMLextern struct channel * open_descriptor_out (int);
+CAMLextern void close_channel (struct channel *);
+CAMLextern int channel_binary_mode (struct channel *);
 
-int flush_partial (struct channel *);
-void flush (struct channel *);
-void putword (struct channel *, uint32);
-int putblock (struct channel *, char *, long);
-void really_putblock (struct channel *, char *, long);
+CAMLextern int flush_partial (struct channel *);
+CAMLextern void flush (struct channel *);
+CAMLextern void putword (struct channel *, uint32);
+CAMLextern int putblock (struct channel *, char *, long);
+CAMLextern void really_putblock (struct channel *, char *, long);
 
-unsigned char refill (struct channel *);
-uint32 getword (struct channel *);
-int getblock (struct channel *, char *, long);
-int really_getblock (struct channel *, char *, long);
+CAMLextern unsigned char refill (struct channel *);
+CAMLextern uint32 getword (struct channel *);
+CAMLextern int getblock (struct channel *, char *, long);
+CAMLextern int really_getblock (struct channel *, char *, long);
 
 /* Extract a struct channel * from the heap object representing it */
 
@@ -74,10 +81,10 @@ int really_getblock (struct channel *, char *, long);
 
 /* The locking machinery */
 
-extern void (*channel_mutex_free) (struct channel *);
-extern void (*channel_mutex_lock) (struct channel *);
-extern void (*channel_mutex_unlock) (struct channel *);
-extern void (*channel_mutex_unlock_exn) (void);
+CAMLextern void (*channel_mutex_free) (struct channel *);
+CAMLextern void (*channel_mutex_lock) (struct channel *);
+CAMLextern void (*channel_mutex_unlock) (struct channel *);
+CAMLextern void (*channel_mutex_unlock_exn) (void);
 
 #define Lock(channel) \
   if (channel_mutex_lock != NULL) (*channel_mutex_lock)(channel)
