@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: major_gc.c,v 1.24 2000/02/11 12:03:15 xleroy Exp $ */
+/* $Id: major_gc.c,v 1.27 2001/12/07 13:39:31 xleroy Exp $ */
 
 #include <limits.h>
 
@@ -203,7 +204,8 @@ static void update_weak_pointers (void)
       sz = Wosize_val (cur);
       for (i = 1; i < sz; i++){
         curfield = Field (cur, i);
-        if (curfield != 0 && Is_block (curfield) && Is_white_val (curfield)){
+        if (curfield != 0 && Is_block (curfield) && Is_in_heap (curfield)
+            && Is_white_val (curfield)){
           Field (cur, i) = 0;
         }
       }
@@ -386,7 +388,7 @@ void init_major_heap (asize_t heap_size)
     page_table [i] = In_heap;
   }
 
-  Hd_hp (heap_start) = Make_header (Wosize_bhsize (stat_heap_size), 0, Caml_blue);
+  Hd_hp (heap_start) = Make_header (Wosize_bhsize(stat_heap_size),0,Caml_blue);
   fl_init_merge ();
   fl_merge_block (Bp_hp (heap_start));
   gc_phase = Phase_idle;
