@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: major_gc.h,v 1.9 2000/01/07 16:51:57 doligez Exp $ */
+/* $Id: major_gc.h,v 1.11 2001/12/07 13:39:31 xleroy Exp $ */
 
 #ifndef _major_gc_
 #define _major_gc_
@@ -46,10 +47,10 @@ typedef int page_table_entry;
 typedef char page_table_entry;
 #endif
 
-extern char *heap_start;
-extern char *heap_end;
+CAMLextern char *heap_start;
+CAMLextern char *heap_end;
 extern unsigned long total_heap_size;
-extern page_table_entry *page_table;
+CAMLextern page_table_entry *page_table;
 extern asize_t page_low, page_high;
 extern char *gc_sweep_hp;
 
