@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1999 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: md5.h,v 1.6 1999/11/17 18:57:12 xleroy Exp $ */
+/* $Id: md5.h,v 1.8 2001/12/07 13:39:32 xleroy Exp $ */
 
 /* MD5 message digest */
 
@@ -21,8 +22,8 @@
 #include "mlvalues.h"
 #include "io.h"
 
-value md5_string (value str, value ofs, value len);
-value md5_chan (value vchan, value len);
+CAMLextern value md5_string (value str, value ofs, value len);
+CAMLextern value md5_chan (value vchan, value len);
 
 struct MD5Context {
         uint32 buf[4];
