@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: memory.c,v 1.22 2001/07/20 14:18:08 doligez Exp $ */
+/* $Id: memory.c,v 1.28 2001/12/07 13:39:32 xleroy Exp $ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -56,10 +57,10 @@ void free_for_heap (header_t *mem)
 
 /* Take a block of memory as argument, which must be the result of a
    call to [alloc_for_heap], and insert it into the heap chaining.
-   The contents of the block must be a sequence of valid objects and
-   fragments: no space between objects and no trailing garbage.  If
-   some objects are blue, they must be added to the free list by the
-   caller.  All other objects must have the color [allocation_color(mem)].
+   The contents of the block must be a sequence of valid blocks and
+   fragments: no space between blocks and no trailing garbage.  If
+   some blocks are blue, they must be added to the free list by the
+   caller.  All other blocks must have the color [allocation_color(mem)].
    The caller must update [allocated_words] if applicable.
    Return value: 0 if no error; -1 in case of error.
 */
@@ -291,12 +292,12 @@ void adjust_gc_speed (mlsize_t mem, mlsize_t max)
    A block value [v] is a shared block if and only if [Is_in_heap (v)]
    is true.
 */
-/* [initialize] never calls the GC, so you may call it while an object is
+/* [initialize] never calls the GC, so you may call it while an block is
    unfinished (i.e. just after a call to [alloc_shr].) */
 void initialize (value *fp, value val)
 {
   *fp = val;
-  if (Is_in_heap (fp) && Is_block (val) && Is_young (val)){
+  if (Is_block (val) && Is_young (val) && Is_in_heap (fp)){
     *ref_table_ptr++ = fp;
     if (ref_table_ptr >= ref_table_limit){
       realloc_ref_table ();
@@ -320,12 +321,7 @@ void * stat_alloc (asize_t sz)
   /* malloc() may return NULL if size is 0 */
   if (result == NULL && sz != 0) raise_out_of_memory ();
 #ifdef DEBUG
-  {
-    value *p;
-    for (p = result; p < (value *) ((char *) result + sz); p++){
-      *p = Debug_uninit_stat;
-    }
-  }
+  memset (result, Debug_uninit_stat, sz);
 #endif
   return result;
 }
