@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: memory.h,v 1.32 2001/07/23 15:34:14 xleroy Exp $ */
+/* $Id: memory.h,v 1.36 2001/12/07 13:39:32 xleroy Exp $ */
 
 /* Allocation macros and functions */
 
@@ -25,14 +26,14 @@
 #include "misc.h"
 #include "mlvalues.h"
 
-value alloc_shr (mlsize_t, tag_t);
+CAMLextern value alloc_shr (mlsize_t, tag_t);
 void adjust_gc_speed (mlsize_t, mlsize_t);
-void modify (value *, value);
-void initialize (value *, value);
-value check_urgent_gc (value);
-void * stat_alloc (asize_t);              /* Size in bytes. */
-void stat_free (void *);
-void * stat_resize (void *, asize_t);     /* Size in bytes. */
+CAMLextern void modify (value *, value);
+CAMLextern void initialize (value *, value);
+CAMLextern value check_urgent_gc (value);
+CAMLextern void * stat_alloc (asize_t);              /* Size in bytes. */
+CAMLextern void stat_free (void *);
+CAMLextern void * stat_resize (void *, asize_t);     /* Size in bytes. */
 header_t *alloc_for_heap (asize_t request);   /* Size in bytes. */
 void free_for_heap (header_t *mem);
 int add_to_heap (header_t *mem);
@@ -41,20 +42,21 @@ color_t allocation_color (void *hp);
 /* void shrink_heap (char *);        Only used in compact.c */
 
 #ifdef DEBUG
-#define DEBUG_clear(result, wosize) { \
-  unsigned long __DEBUG_i; \
-  for (__DEBUG_i = 0; __DEBUG_i < wosize; ++ __DEBUG_i){ \
-    Field (result, __DEBUG_i) = Debug_uninit_minor; \
+#define DEBUG_clear(result, wosize) do{ \
+  unsigned long caml__DEBUG_i; \
+  for (caml__DEBUG_i = 0; caml__DEBUG_i < (wosize); ++ caml__DEBUG_i){ \
+    Field ((result), caml__DEBUG_i) = Debug_uninit_minor; \
   } \
-}
+}while(0)
 #else
 #define DEBUG_clear(result, wosize)
 #endif
 
-#define Alloc_small(result, wosize, tag) {        CAMLassert (wosize >= 1); \
+#define Alloc_small(result, wosize, tag) do{      CAMLassert (wosize >= 1); \
                                             CAMLassert ((tag_t) tag < 256); \
   young_ptr -= Bhsize_wosize (wosize);                                      \
   if (young_ptr < young_limit){                                             \
+    young_ptr += Bhsize_wosize (wosize);                                    \
     Setup_for_gc;                                                           \
     minor_collection ();                                                    \
     Restore_after_gc;                                                       \
@@ -62,15 +64,21 @@ color_t allocation_color (void *hp);
   }                                                                         \
   Hd_hp (young_ptr) = Make_header ((wosize), (tag), Caml_black);            \
   (result) = Val_hp (young_ptr);                                            \
-  DEBUG_clear (result, wosize);                                             \
-}
+  DEBUG_clear ((result), (wosize));                                         \
+}while(0)
 
 /* You must use [Modify] to change a field of an existing shared block,
    unless you are sure the value being overwritten is not a shared block and
    the value being written is not a young block. */
 /* [Modify] never calls the GC. */
+/* [Modify] can also be used to do assignment on data structures that are
+   not in the (major) heap.  In this case, it is a bit slower than
+   simple assignment.
+   In particular, you can use [Modify] when you don't know whether the
+   block being changed is in the minor heap or the major heap.
+*/
 
-#define Modify(fp, val) {                                                   \
+#define Modify(fp, val) do{                                                 \
   value _old_ = *(fp);                                                      \
   *(fp) = (val);                                                            \
   if (Is_in_heap (fp)){                                                     \
@@ -84,8 +92,7 @@ color_t allocation_color (void *hp);
       }                                                                     \
     }                                                                       \
   }                                                                         \
-}                                                                           \
-
+}while(0)
 
 struct caml__roots_block {
   struct caml__roots_block *next;
@@ -94,7 +101,7 @@ struct caml__roots_block {
   value *tables [5];
 };
 
-extern struct caml__roots_block *local_roots;  /* defined in roots.c */
+CAMLextern struct caml__roots_block *local_roots;  /* defined in roots.c */
 
 /* The following macros are used to declare C local variables and
    function parameters of type [value].
@@ -258,7 +265,11 @@ extern struct caml__roots_block *local_roots;  /* defined in roots.c */
 }while(0)
 
 /* convenience macro */
-#define Store_field(block, offset, val) modify (&Field (block, offset), val)
+#define Store_field(block, offset, val) do{ \
+  mlsize_t caml__temp_offset = (offset); \
+  value caml__temp_val = (val); \
+  modify (&Field ((block), caml__temp_offset), caml__temp_val); \
+}while(0)
 
 /*
    NOTE: [Begin_roots] and [End_roots] are superseded by [CAMLparam]*,
@@ -347,12 +358,12 @@ extern struct caml__roots_block *local_roots;  /* defined in roots.c */
    for the duration of the program, or until [remove_global_root] is
    called. */
 
-void register_global_root (value *);
+CAMLextern void register_global_root (value *);
 
 /* [remove_global_root] removes a memory root registered on a global C
    variable with [register_global_root]. */
 
-void remove_global_root (value *);
+CAMLextern void remove_global_root (value *);
 
 
 #endif /* _memory_ */
