@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: minor_gc.h,v 1.11 2000/01/07 16:51:57 doligez Exp $ */
+/* $Id: minor_gc.h,v 1.13 2001/12/07 13:39:32 xleroy Exp $ */
 
 #ifndef _minor_gc_
 #define _minor_gc_
@@ -18,8 +19,8 @@
 
 #include "misc.h"
 
-extern char *young_start, *young_ptr, *young_end, *young_limit;
-extern value **ref_table_ptr, **ref_table_limit;
+CAMLextern char *young_start, *young_ptr, *young_end, *young_limit;
+CAMLextern value **ref_table_ptr, **ref_table_limit;
 extern asize_t minor_heap_size;
 extern int in_minor_collection;
 
@@ -28,8 +29,8 @@ extern int in_minor_collection;
 
 extern void set_minor_heap_size (asize_t);
 extern void empty_minor_heap (void);
-extern void minor_collection (void);
-extern void garbage_collection (void); /* for the native-code system */
+CAMLextern void minor_collection (void);
+CAMLextern void garbage_collection (void); /* for the native-code system */
 extern void realloc_ref_table (void);
 extern void oldify (value, value *);
 
