@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: mlvalues.h,v 1.32 2001/01/05 13:55:13 doligez Exp $ */
+/* $Id: mlvalues.h,v 1.36 2001/12/07 13:39:33 xleroy Exp $ */
 
 #ifndef _mlvalues_
 #define _mlvalues_
@@ -67,7 +68,7 @@ typedef unsigned long mark_t;
 #define Long_val(x)     ((x) >> 1)
 #define Max_long ((1L << (8 * sizeof(value) - 2)) - 1)
 #define Min_long (-(1L << (8 * sizeof(value) - 2)))
-#define Val_int Val_long
+#define Val_int(x) Val_long(x)
 #define Int_val(x) ((int) Long_val(x))
 
 /* Structure of the header:
@@ -173,11 +174,11 @@ typedef opcode_t * code_t;
 
 /* Another special case: objects */
 #define Object_tag 248
-#define Class_val(val) Field(val, 0)
-#define Oid_val(val) Long_val(Field(val, 1))
+#define Class_val(val) Field((val), 0)
+#define Oid_val(val) Long_val(Field((val), 1))
 
 /* Another special case: variants */
-extern value hash_variant(char * tag);
+CAMLextern value hash_variant(char * tag);
 
 /* 2- If tag >= No_scan_tag : a sequence of bytes. */
 
@@ -196,7 +197,7 @@ extern value hash_variant(char * tag);
 /* Strings. */
 #define String_tag 252
 #define String_val(x) ((char *) Bp_val(x))
-mlsize_t string_length (value);
+CAMLextern mlsize_t string_length (value);
 
 /* Floating-point numbers. */
 #define Double_tag 253
@@ -205,15 +206,18 @@ mlsize_t string_length (value);
 #define Double_val(v) (* (double *)(v))
 #define Store_double_val(v,d) (* (double *)(v) = (d))
 #else
-double Double_val (value);
-void Store_double_val (value,double);
+CAMLextern double Double_val (value);
+CAMLextern void Store_double_val (value,double);
 #endif
 
 /* Arrays of floating-point numbers. */
 #define Double_array_tag 254
 #define Double_field(v,i) Double_val((value)((double *)(v) + (i)))
-#define Store_double_field(v,i,d) \
-  Store_double_val((value)((double *)(v) + (i)),d)
+#define Store_double_field(v,i,d) do{ \
+  mlsize_t caml__temp_i = (i); \
+  double caml__temp_d = (d); \
+  Store_double_val((value)((double *) v + caml__temp_i), caml__temp_d); \
+}while(0)
 
 /* Custom blocks.  They contain a pointer to a "method suite"
    of functions (for finalization, comparison, hashing, etc)
@@ -221,7 +225,7 @@ void Store_double_val (value,double);
    the GC; therefore, they must not contain any [value].
    See [custom.h] for operations on method suites. */
 #define Custom_tag 255
-#define Data_custom_val(v) ((void *) &Field(v, 1))
+#define Data_custom_val(v) ((void *) &Field((v), 1))
 struct custom_operations;       /* defined in [custom.h] */
 
 /* Int32.t, Int64.t and Nativeint.t are represented as custom blocks. */
@@ -231,13 +235,13 @@ struct custom_operations;       /* defined in [custom.h] */
 #ifndef ARCH_ALIGN_INT64
 #define Int64_val(v) (*((int64 *) Data_custom_val(v)))
 #else
-extern int64 Int64_val(value v);
+CAMLextern int64 Int64_val(value v);
 #endif
 
 /* 3- Atoms are 0-tuples.  They are statically allocated once and for all. */
 
-extern header_t atom_table[];
-#define Atom(tag) (Val_hp (&(atom_table [tag])))
+CAMLextern header_t atom_table[];
+#define Atom(tag) (Val_hp (&(atom_table [(tag)])))
 
 /* Is_atom tests whether a well-formed block is statically allocated
    outside the heap. For the bytecode system, only zero-sized block (Atoms)
@@ -248,7 +252,7 @@ extern header_t atom_table[];
 #ifndef NATIVE_CODE
 #define Is_atom(v) ((v) >= Atom(0) && (v) <= Atom(255))
 #else
-extern char * static_data_start, * static_data_end;
+CAMLextern char * static_data_start, * static_data_end;
 #define Is_atom(v) \
   ((((char *)(v) >= static_data_start && (char *)(v) < static_data_end) || \
    ((v) >= Atom(0) && (v) <= Atom(255))))
