@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: printexc.c,v 1.9 2001/06/15 14:22:38 xleroy Exp $ */
+/* $Id: printexc.c,v 1.13 2001/12/07 13:39:34 xleroy Exp $ */
 
 /* Print an uncaught exception and abort */
 
@@ -18,6 +19,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include "backtrace.h"
+#include "callback.h"
 #include "debugger.h"
 #include "fail.h"
 #include "misc.h"
@@ -46,7 +48,7 @@ static void add_string(struct stringbuf *buf, char *s)
   buf->ptr += len;
 }
   
-char * format_caml_exception(value exn)
+CAMLexport char * format_caml_exception(value exn)
 {
   mlsize_t start, i;
   value bucket, v;
@@ -96,16 +98,38 @@ char * format_caml_exception(value exn)
 
 void fatal_uncaught_exception(value exn)
 {
-  char * msg = format_caml_exception(exn);
+  char * msg;
+  value * at_exit;
+#ifndef NATIVE_CODE
+  int saved_backtrace_active, saved_backtrace_pos;
+#endif
+  /* Build a string representation of the exception */
+  msg = format_caml_exception(exn);
+  /* Perform "at_exit" processing, ignoring all exceptions that may
+     be triggered by this */
+#ifndef NATIVE_CODE
+  saved_backtrace_active = backtrace_active;
+  saved_backtrace_pos = backtrace_pos;
+  backtrace_active = 0;
+#endif
+  at_exit = caml_named_value("Pervasives.do_at_exit");
+  if (at_exit != NULL) callback_exn(*at_exit, Val_unit);
+#ifndef NATIVE_CODE
+  backtrace_active = saved_backtrace_active;
+  backtrace_pos = saved_backtrace_pos;
+#endif
+  /* Display the uncaught exception */
 #ifdef HAS_UI
-  ui_print_stderr("Fatal error: uncaught exception %s\n", msg);
+  ui_print_stderr("Fatal error: exception %s\n", msg);
 #else
-  fprintf(stderr, "Fatal error: uncaught exception %s\n", msg);
+  fprintf(stderr, "Fatal error: exception %s\n", msg);
 #endif
   free(msg);
+  /* Display the backtrace if available */
 #ifndef NATIVE_CODE
   if (backtrace_active && !debugger_in_use) print_exception_backtrace();
 #endif
+  /* Terminate the process */
 #ifdef HAS_UI
   ui_exit(2);
 #else
