@@ -6,15 +6,17 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: roots.c,v 1.19 2000/02/14 14:30:39 weis Exp $ */
+/* $Id: roots.c,v 1.23 2001/12/07 13:39:34 xleroy Exp $ */
 
 /* To walk the memory roots for garbage collection */
 
 #include "finalise.h"
+#include "globroots.h"
 #include "major_gc.h"
 #include "memory.h"
 #include "minor_gc.h"
@@ -23,46 +25,10 @@
 #include "roots.h"
 #include "stacks.h"
 
-struct caml__roots_block *local_roots = NULL;
-
-/* FIXME turn this into a table and synchronise with asmrun/roots.c */
-struct global_root {
-  value * root;
-  struct global_root * next;
-};
-
-static struct global_root * global_roots = NULL;
+CAMLexport struct caml__roots_block *local_roots = NULL;
 
 void (*scan_roots_hook) (scanning_action f) = NULL;
 
-/* Register a global C root */
-
-void register_global_root(value *r)
-{
-  struct global_root * gr;
-  
-  Assert (((long) r & 3) == 0);  /* compact.c demands this (for now) */
-  gr = (struct global_root *) stat_alloc(sizeof(struct global_root));
-  gr->root = r;
-  gr->next = global_roots;
-  global_roots = gr;
-}
-
-/* Un-register a global C root */
-
-void remove_global_root(value *r)
-{
-  struct global_root ** gp, * gr;
-  for (gp = &global_roots; *gp != NULL; gp = &(*gp)->next) {
-    gr = *gp;
-    if (gr->root == r) {
-      *gp = gr->next;
-      stat_free(gr);
-      return;
-    }
-  }
-}
-
 /* FIXME rename to [oldify_young_roots] and synchronise with asmrun/roots.c */
 /* Call [oldify] on (at least) all the roots that point to the minor heap. */
 void oldify_local_roots (void)
@@ -86,7 +52,7 @@ void oldify_local_roots (void)
     }
   }
   /* Global C roots */
-  for (gr = global_roots; gr != NULL; gr = gr->next) {
+  for (gr = caml_global_roots.forward[0]; gr != NULL; gr = gr->forward[0]) {
     oldify(*(gr->root), gr->root);
   }
   /* Finalised values */
@@ -113,8 +79,8 @@ void do_roots (scanning_action f)
   do_local_roots(f, extern_sp, stack_high, local_roots);
 
   /* Global C roots */
-  for (gr = global_roots; gr != NULL; gr = gr->next) {
-    f (*(gr->root), gr->root);
+  for (gr = caml_global_roots.forward[0]; gr != NULL; gr = gr->forward[0]) {
+    f(*(gr->root), gr->root);
   }
   /* Finalised values */
   final_do_strong_roots (f);
