@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: signals.c,v 1.32 2001/07/02 12:49:56 xleroy Exp $ */
+/* $Id: signals.c,v 1.37 2001/12/07 13:39:36 xleroy Exp $ */
 
 #include <signal.h>
 #include "alloc.h"
@@ -28,6 +29,12 @@
 #include "rotatecursor.h"
 #endif /* macintosh */
 
+#ifdef _WIN32
+typedef void (*sighandler)(int sig);
+extern sighandler win32_signal(int sig, sighandler action);
+#define signal(sig,act) win32_signal(sig,act)
+#endif
+
 int volatile async_signal_mode = 0;
 int volatile pending_signal = 0;
 int volatile something_to_do = 0;
@@ -41,7 +48,7 @@ void process_event(void)
 {
   int signal_number;
   void (*async_action)(void);
-  if (force_major_slice) minor_collection ();
+  if (force_major_slice) minor_collection (); /* FIXME should be check_urgent_gc */
   /* If a signal arrives between the following two instructions,
      it will be lost.  To do: use atomic swap or atomic read-and-clear
      for processors that support it? */
@@ -107,7 +114,7 @@ void urge_major_slice (void)
   something_to_do = 1;
 }
 
-void enter_blocking_section(void)
+CAMLexport void enter_blocking_section(void)
 {
   int temp;
 
@@ -124,8 +131,18 @@ void enter_blocking_section(void)
   if (enter_blocking_section_hook != NULL) enter_blocking_section_hook();
 }
 
-void leave_blocking_section(void)
+CAMLexport void leave_blocking_section(void)
 {
+#ifdef _WIN32
+  /* Under Win32, asynchronous signals such as ctrl-C are not processed
+     immediately (see ctrl_handler in win32.c), but simply set
+     pending_signal and let the system call run to completion.
+     Hence, test pending_signal here and act upon it, before we get
+     a chance to process the result of the system call. */
+  int signal_number = pending_signal;
+  pending_signal = 0;
+  if (signal_number) execute_signal(signal_number, 1);
+#endif
   if (leave_blocking_section_hook != NULL) leave_blocking_section_hook();
   Assert(async_signal_mode);
   async_signal_mode = 0;
@@ -201,7 +218,7 @@ static int posix_signals[] = {
   SIGSTOP, SIGTSTP, SIGTTIN, SIGTTOU, SIGVTALRM, SIGPROF
 };
 
-int convert_signal_number(int signo)
+CAMLexport int convert_signal_number(int signo)
 {
   if (signo < 0 && signo >= -(sizeof(posix_signals) / sizeof(int)))
     return posix_signals[-signo-1];
@@ -221,7 +238,7 @@ static int rev_convert_signal_number(int signo)
 #define NSIG 64
 #endif
 
-value install_signal_handler(value signal_number, value action) /* ML */
+CAMLprim value install_signal_handler(value signal_number, value action)
 {
   CAMLparam2 (signal_number, action);
   int sig;
