@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: stacks.c,v 1.16 2001/05/28 15:14:18 xleroy Exp $ */
+/* $Id: stacks.c,v 1.18 2001/12/07 13:39:36 xleroy Exp $ */
 
 /* To initialize and resize the stacks */
 
@@ -21,12 +22,12 @@
 #include "mlvalues.h"
 #include "stacks.h"
 
-value * stack_low;
-value * stack_high;
-value * stack_threshold;
-value * extern_sp;
-value * trapsp;
-value * trap_barrier;
+CAMLexport value * stack_low;
+CAMLexport value * stack_high;
+CAMLexport value * stack_threshold;
+CAMLexport value * extern_sp;
+CAMLexport value * trapsp;
+CAMLexport value * trap_barrier;
 value global_data;
 
 unsigned long max_stack_size;            /* also used in gc_ctrl.c */
@@ -81,7 +82,7 @@ void realloc_stack(asize_t required_space)
 #undef shift
 }
 
-value ensure_stack_capacity(value required_space) /* ML */
+CAMLprim value ensure_stack_capacity(value required_space)
 {
   asize_t req = Long_val(required_space);
   if (extern_sp - req < stack_low) realloc_stack(req);
