@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: stacks.h,v 1.8 2001/05/28 15:14:18 xleroy Exp $ */
+/* $Id: stacks.h,v 1.10 2001/12/07 13:39:36 xleroy Exp $ */
 
 /* structure of the stacks */
 
@@ -22,12 +23,12 @@
 #include "mlvalues.h"
 #include "memory.h"
 
-extern value * stack_low;
-extern value * stack_high;
-extern value * stack_threshold;
-extern value * extern_sp;
-extern value * trapsp;
-extern value * trap_barrier;
+CAMLextern value * stack_low;
+CAMLextern value * stack_high;
+CAMLextern value * stack_threshold;
+CAMLextern value * extern_sp;
+CAMLextern value * trapsp;
+CAMLextern value * trap_barrier;
 
 #define Trap_pc(tp) (((code_t *)(tp))[0])
 #define Trap_link(tp) (((value **)(tp))[1])
