@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: sys.h,v 1.8 2001/02/19 12:29:00 xleroy Exp $ */
+/* $Id: sys.h,v 1.10 2001/12/07 13:39:37 xleroy Exp $ */
 
 #ifndef _sys_
 #define _sys_
@@ -19,10 +20,9 @@
 
 #define NO_ARG Val_int(0)
 
-extern void sys_error (value);
+CAMLextern void sys_error (value);
 extern void sys_init (char **);
-extern value sys_exit (value);
-extern char * searchpath (char * name);
+CAMLextern value sys_exit (value);
 
 extern char ** caml_main_argv;
 
