@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1997 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: weak.c,v 1.14 2000/09/08 11:48:06 doligez Exp $ */
+/* $Id: weak.c,v 1.17 2001/12/07 13:39:38 xleroy Exp $ */
 
 /* Operations on weak arrays */
 
@@ -23,7 +24,7 @@
 
 value weak_list_head = 0;
 
-value weak_create (value len)        /* ML */
+CAMLprim value weak_create (value len)
 {
   mlsize_t size, i;
   value res;
@@ -37,10 +38,10 @@ value weak_create (value len)        /* ML */
   return res;
 }
 
-#define None_val Val_int(0)
+#define None_val (Val_int(0))
 #define Some_tag 0
 
-value weak_set (value ar, value n, value el)     /* ML */
+CAMLprim value weak_set (value ar, value n, value el)
 {
   mlsize_t offset = Long_val (n) + 1;
                                                    Assert (Is_in_heap (ar));
@@ -55,7 +56,7 @@ value weak_set (value ar, value n, value el)     /* ML */
 #define Setup_for_gc
 #define Restore_after_gc
 
-value weak_get (value ar, value n)        /* ML */
+CAMLprim value weak_get (value ar, value n)
 {
   CAMLparam2 (ar, n);
   mlsize_t offset = Long_val (n) + 1;
@@ -76,7 +77,7 @@ value weak_get (value ar, value n)        /* ML */
 #undef Setup_for_gc
 #undef Restore_after_gc
 
-value weak_get_copy (value ar, value n)        /* ML */
+CAMLprim value weak_get_copy (value ar, value n)
 {
   CAMLparam2 (ar, n);
   mlsize_t offset = Long_val (n) + 1;
@@ -104,7 +105,7 @@ value weak_get_copy (value ar, value n)        /* ML */
   CAMLreturn (res);
 }
 
-value weak_check (value ar, value n)        /* ML */
+CAMLprim value weak_check (value ar, value n)
 {
   mlsize_t offset = Long_val (n) + 1;
                                                    Assert (Is_in_heap (ar));
