@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: s-templ.h,v 1.17 2000/10/29 17:36:44 doligez Exp $ */
+/* $Id: s-templ.h,v 1.19 2001/12/07 13:39:41 xleroy Exp $ */
 
 /* Operating system and standard library dependencies. */
 
@@ -50,6 +51,11 @@
 
 /* Define HAS_STRERROR if you have strerror(). */
 
+#define SUPPORT_DYNAMIC_LINKING
+
+/* Define SUPPORT_DYNAMIC_LINKING if dynamic loading of C stub code
+   via dlopen() is available. */
+
 /* 2. For the Unix library. */
 
 #define HAS_SOCKETS
