@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: main.ml,v 1.9 1999/11/17 18:57:25 xleroy Exp $ *)
+(* $Id: main.ml,v 1.11 2000/03/21 15:16:28 weis Exp $ *)
 
 open Primitives
 open Misc
@@ -24,57 +24,55 @@ open Parameters
 open Program_management
 open Frames
 open Show_information
-
+open Format
 
 let line_buffer = Lexing.from_function read_user_input
 
-let rec loop () =
-  line_loop line_buffer;
-  if !loaded & (not (yes_or_no "The program is running. Quit anyway")) then
-    loop ()
+let rec loop ppf =
+  line_loop ppf line_buffer;
+  if !loaded && (not (yes_or_no "The program is running. Quit anyway")) then
+    loop ppf
 
-let rec protect cont =
+let rec protect ppf cont =
   try
-    cont ()
+    cont ppf
   with
-    End_of_file ->
-      protect (function () ->
+  | End_of_file ->
+      protect ppf (function ppf ->
         forget_process
           !current_checkpoint.c_fd
           !current_checkpoint.c_pid;
-        flush stdout;
+        pp_print_flush ppf ();
         stop_user_input ();
-        loop ())
+        loop ppf)
   | Toplevel ->
-      protect (function () ->
-        flush stdout;
+      protect ppf (function ppf ->
+        pp_print_flush ppf ();
         stop_user_input ();
-        loop ())
+        loop ppf)
   | Sys.Break ->
-      protect (function () ->
-        print_endline "Interrupted.";
+      protect ppf (function ppf ->
+        fprintf ppf "Interrupted.@.";
         Exec.protect (function () ->
-          flush stdout;
           stop_user_input ();
           if !loaded then begin
             try_select_frame 0;
-            show_current_event ()
+            show_current_event ppf;
           end);
-        loop ())
+        loop ppf)
   | Current_checkpoint_lost ->
-      protect (function () ->
-        print_endline "Trying to recover...";
-        flush stdout;
+      protect ppf (function ppf ->
+        fprintf ppf "Trying to recover...@.";
         stop_user_input ();
         recover ();
         try_select_frame 0;
-        show_current_event ();
-        loop ())
+        show_current_event ppf;
+        loop ppf)
   | x ->
       kill_program ();
       raise x
 
-let toplevel_loop () = protect loop
+let toplevel_loop () = protect Format.std_formatter loop
 
 (* Parsing of command-line arguments *)
 
@@ -121,9 +119,7 @@ let main () =
       done
     end;
     current_prompt := debugger_prompt;
-    print_string "\tObjective Caml Debugger version ";
-    print_string Config.version;
-    print_newline(); print_newline();
+    printf "\tObjective Caml Debugger version %s@.@." Config.version;
     Config.load_path := !default_load_path;
     toplevel_loop ();                   (* Toplevel. *)
     kill_program ();
