@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: printval.mli,v 1.9 2000/03/06 22:11:24 weis Exp $ *)
+(* $Id: printval.mli,v 1.10 2001/07/03 11:04:09 xleroy Exp $ *)
 
 open Format
 
@@ -28,5 +28,6 @@ val reset_named_values : unit -> unit
 val find_named_value : int -> Debugcom.Remote_value.t * Types.type_expr
 
 val install_printer :
-  Path.t -> Types.type_expr -> formatter -> (Obj.t -> unit) -> unit
+  Path.t -> Types.type_expr -> formatter -> 
+    (formatter -> Obj.t -> unit) -> unit
 val remove_printer : Path.t -> unit
