@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: compile.ml,v 1.40 2001/07/02 14:25:28 xleroy Exp $ *)
+(* $Id: compile.ml,v 1.43 2001/10/01 08:09:58 ddr Exp $ *)
 
 (* The batch compiler *)
 
@@ -98,6 +98,11 @@ let parse_file inputfile parse_fun ast_magic =
   close_in ic;
   ast
 
+let remove_preprocessed_if_ast inputfile =
+  match !Clflags.preprocessor with
+    None -> ()
+  | Some _ -> if inputfile <> !Location.input_name then remove_file inputfile
+
 (* Compile a .mli file *)
 
 let interface ppf sourcefile =
@@ -105,14 +110,18 @@ let interface ppf sourcefile =
   let prefixname = Filename.chop_extension sourcefile in
   let modulename = String.capitalize(Filename.basename prefixname) in
   let inputfile = preprocess sourcefile in
-  let ast = parse_file inputfile Parse.interface ast_intf_magic_number in
-  if !Clflags.dump_parsetree then fprintf ppf "%a@." Printast.interface ast;
-  let sg = Typemod.transl_signature (initial_env()) ast in
-  if !Clflags.print_types
-    then fprintf std_formatter "%a@." Printtyp.signature sg;
-  Warnings.check_fatal ();
-  Env.save_signature sg modulename (prefixname ^ ".cmi");
-  remove_preprocessed inputfile
+  try
+    let ast = parse_file inputfile Parse.interface ast_intf_magic_number in
+    if !Clflags.dump_parsetree then fprintf ppf "%a@." Printast.interface ast;
+    let sg = Typemod.transl_signature (initial_env()) ast in
+    if !Clflags.print_types
+      then fprintf std_formatter "%a@." Printtyp.signature sg;
+    Warnings.check_fatal ();
+    Env.save_signature sg modulename (prefixname ^ ".cmi");
+    remove_preprocessed inputfile
+  with e ->
+    remove_preprocessed_if_ast inputfile;
+    raise e
 
 (* Compile a .ml file *)
 
@@ -147,6 +156,7 @@ let implementation ppf sourcefile =
   with x ->
     close_out oc;
     remove_file objfile;
+    remove_preprocessed_if_ast inputfile;
     raise x
 
 let c_file name =
