@@ -10,12 +10,14 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: compile.mli,v 1.6 1999/11/17 18:57:31 xleroy Exp $ *)
+(* $Id: compile.mli,v 1.7 2000/03/06 22:11:30 weis Exp $ *)
 
 (* Compile a .ml or .mli file *)
 
-val interface: string -> unit
-val implementation: string -> unit
+open Format
+
+val interface: formatter -> string -> unit
+val implementation: formatter -> string -> unit
 val c_file: string -> unit
 
 val initial_env: unit -> Env.t
