@@ -10,19 +10,22 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: errors.ml,v 1.16 2000/03/08 01:48:22 garrigue Exp $ *)
+(* $Id: errors.ml,v 1.20 2001/06/18 12:26:26 doligez Exp $ *)
 
-(* Error report *)
+(* WARNING: if you change something in this file, you must look at
+   opterrors.ml to see if you need to make the same changes there.
+*)
 
 open Format
-open Location
 
 (* Report an error *)
 
 let report_error ppf exn =
   let report ppf = function
   | Lexer.Error(err, start, stop) ->
-      Location.print ppf {loc_start = start; loc_end = stop; loc_ghost = false};
+      Location.print ppf {Location.loc_start = start;
+                          Location.loc_end = stop;
+                          Location.loc_ghost = false};
       Lexer.report_error ppf err
   | Syntaxerr.Error err ->
       Syntaxerr.report_error ppf err
@@ -55,6 +58,8 @@ let report_error ppf exn =
       Location.print ppf loc; Typeclass.report_error ppf err
   | Translclass.Error(loc, err) ->
       Location.print ppf loc; Translclass.report_error ppf err
+  | Warnings.Errors (n) ->
+      fprintf ppf "@.Error: %d error-enabled warnings occurred." n
   | x -> fprintf ppf "@]"; raise x in
 
   fprintf ppf "@[%a@]@." report exn
