@@ -10,8 +10,9 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: errors.mli,v 1.4 1999/11/17 18:57:31 xleroy Exp $ *)
+(* $Id: errors.mli,v 1.5 2000/03/06 22:11:31 weis Exp $ *)
 
 (* Error report *)
+open Format
 
-val report_error: exn -> unit
+val report_error: formatter -> exn -> unit
