@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: main.ml,v 1.48 2000/12/28 13:03:45 weis Exp $ *)
+(* $Id: main.ml,v 1.53 2001/10/30 09:32:30 xleroy Exp $ *)
 
 open Config
 open Clflags
@@ -36,6 +36,8 @@ let process_file ppf name =
   else if Filename.check_suffix name ext_obj
        || Filename.check_suffix name ext_lib then
     ccobjs := name :: !ccobjs
+  else if Filename.check_suffix name ext_dll then
+    dllibs := name :: !dllibs
   else if Filename.check_suffix name ".c" then begin
     Compile.c_file name;
     match Sys.os_type with
@@ -73,6 +75,8 @@ module Options = Main_args.Make_options (struct
   let _cclib s = ccobjs := s :: !ccobjs
   let _ccopt s = ccopts := s :: !ccopts
   let _custom = set custom_runtime
+  let _dllib s = dllibs := s :: !dllibs
+  let _dllpath s = dllpaths := !dllpaths @ [s]
   let _g = set debug
   let _i = set print_types
   let _I s = include_dirs := s :: !include_dirs
@@ -84,6 +88,7 @@ module Options = Main_args.Make_options (struct
   let _make_runtime () =
     custom_runtime := true; make_runtime := true; link_everything := true
   let _noassert = set noassert
+  let _nolabels = set classic
   let _noautolink = set no_auto_link
   let _o s = exec_name := s; archive_name := s; object_name := s
   let _output_obj () = output_c_object := true; custom_runtime := true
