@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: main_args.ml,v 1.22 2000/11/07 14:41:12 xleroy Exp $ *)
+(* $Id: main_args.ml,v 1.29 2001/10/30 10:11:30 xleroy Exp $ *)
 
 module Make_options (F :
    sig
@@ -20,6 +20,8 @@ module Make_options (F :
      val _cclib : string -> unit
      val _ccopt : string -> unit
      val _custom : unit -> unit
+     val _dllib : string -> unit
+     val _dllpath : string -> unit
      val _g : unit -> unit
      val _i : unit -> unit
      val _I : string -> unit
@@ -31,6 +33,7 @@ module Make_options (F :
      val _make_runtime : unit -> unit
      val _noassert : unit -> unit
      val _noautolink : unit -> unit
+     val _nolabels : unit -> unit
      val _o : string -> unit
      val _output_obj : unit -> unit
      val _pp : string -> unit
@@ -62,6 +65,8 @@ struct
     "-ccopt", Arg.String F._ccopt,
                       "<opt>  Pass option <opt> to the C compiler and linker";
     "-custom", Arg.Unit F._custom, " Link in custom mode";
+    "-dllib", Arg.String F._dllib, "<lib>  Use the dynamically-loaded library <lib>";
+    "-dllpath", Arg.String F._dllpath, "<dir>  Add <dir> to the run-time search path for shared libraries";
     "-g", Arg.Unit F._g, " Save debugging information";
     "-i", Arg.Unit F._i, " Print the types";
     "-I", Arg.String F._I,
@@ -82,6 +87,7 @@ struct
     "-noassert", Arg.Unit F._noassert, " Don't compile assertion checks";
     "-noautolink", Arg.Unit F._noautolink,
            " Don't automatically link C libraries specified in .cma files";
+    "-nolabels", Arg.Unit F._nolabels, " Ignore non-optional labels in types";
     "-o", Arg.String F._o, "<file>  Set output file name to <file>";
     "-output-obj", Arg.Unit F._output_obj,
                           " Output a C object file instead of an executable";
@@ -101,14 +107,16 @@ struct
           "<flags>  Enable or disable warnings according to <flags>:\n\
       \032    A/a enable/disable all warnings\n\
       \032    C/c enable/disable suspicious comment\n\
+      \032    D/d enable/disable deprecated features\n\
       \032    F/f enable/disable partially applied function\n\
+      \032    L/l enable/disable labels omitted in application\n\
       \032    M/m enable/disable overriden method\n\
       \032    P/p enable/disable partial match\n\
       \032    S/s enable/disable non-unit statement\n\
       \032    U/u enable/disable unused match case\n\
       \032    V/v enable/disable hidden instance variable\n\
       \032    X/x enable/disable all other warnings\n\
-      \032    default setting is \"A\" (all warnings enabled)";
+      \032    default setting is \"Al\" (all warnings but labels enabled)";
     "-warn-error" , Arg.String F._warn_error,
       "<flags>  Treat the warnings enabled by <flags> as errors.\n\
         \032    See option -w for the list of flags.\n\
