@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: main_args.mli,v 1.15 2000/11/07 14:41:12 xleroy Exp $ *)
+(* $Id: main_args.mli,v 1.18 2001/10/30 09:32:30 xleroy Exp $ *)
 
 module Make_options (F :
     sig
@@ -20,6 +20,8 @@ module Make_options (F :
       val _cclib : string -> unit
       val _ccopt : string -> unit
       val _custom : unit -> unit
+      val _dllib : string -> unit
+      val _dllpath : string -> unit
       val _g : unit -> unit
       val _i : unit -> unit
       val _I : string -> unit
@@ -31,6 +33,7 @@ module Make_options (F :
       val _make_runtime : unit -> unit
       val _noassert : unit -> unit
       val _noautolink : unit -> unit
+      val _nolabels : unit -> unit
       val _o : string -> unit
       val _output_obj : unit -> unit
       val _pp : string -> unit
