@@ -10,12 +10,14 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: optcompile.mli,v 1.6 1999/11/17 18:57:32 xleroy Exp $ *)
+(* $Id: optcompile.mli,v 1.7 2000/03/07 05:02:32 garrigue Exp $ *)
 
 (* Compile a .ml or .mli file *)
 
-val interface: string -> unit
-val implementation: string -> unit
+open Format
+
+val interface: formatter -> string -> unit
+val implementation: formatter -> string -> unit
 val c_file: string -> unit
 
 val initial_env: unit -> Env.t
