@@ -10,8 +10,8 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: opterrors.mli,v 1.4 1999/11/17 18:57:32 xleroy Exp $ *)
+(* $Id: opterrors.mli,v 1.5 2000/03/07 05:02:32 garrigue Exp $ *)
 
 (* Error report *)
 
-val report_error: exn -> unit
+val report_error: Format.formatter -> exn -> unit
