@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: optmain.ml,v 1.56 2001/02/19 20:27:46 maranget Exp $ *)
+(* $Id: optmain.ml,v 1.61 2001/09/27 07:30:15 garrigue Exp $ *)
 
 open Config
 open Clflags
@@ -73,6 +73,8 @@ let main () =
              "<opt>  Pass option <opt> to the C compiler and linker";
        "-compact", Arg.Clear optimize_for_speed,
              " Optimize code size rather than speed";
+       "-dllpath", Arg.String (fun s -> dllpaths := !dllpaths @ [s]),
+             "<dir>  Add <dir> to the run-time search path for shared libraries";
        "-i", Arg.Set print_types, " Print the types";
        "-I", Arg.String(fun dir -> include_dirs := dir :: !include_dirs),
              "<dir>  Add <dir> to the list of include directories";
@@ -89,10 +91,10 @@ let main () =
        "-labels", Arg.Clear classic, " Use commuting label mode";
        "-linkall", Arg.Set link_everything,
              " Link all modules, even unused ones";
-       "-modern", Arg.Clear classic, " (deprecated) same as -labels";
        "-noassert", Arg.Set noassert, " Don't compile assertion checks";
        "-noautolink", Arg.Set no_auto_link,
              " Don't automatically link C libraries specified in .cma files";
+       "-nolabels", Arg.Set classic, " Ignore non-optional labels in types";
        "-o", Arg.String(fun s -> exec_name := s;
                                  archive_name := s;
                                  object_name := s),
@@ -116,14 +118,16 @@ let main () =
              "<flags>  Enable or disable warnings according to <flags>:\n\
          \032    A/a enable/disable all warnings\n\
          \032    C/c enable/disable suspicious comment\n\
+         \032    D/d enable/disable deprecated features\n\
          \032    F/f enable/disable partially applied function\n\
+         \032    L/l enable/disable labels omitted in application\n\
          \032    M/m enable/disable overriden methods\n\
          \032    P/p enable/disable partial match\n\
          \032    S/s enable/disable non-unit statement\n\
          \032    U/u enable/disable unused match case\n\
          \032    V/v enable/disable hidden instance variables\n\
          \032    X/x enable/disable all other warnings\n\
-         \032    default setting is A (all warnings enabled)";
+         \032    default setting is \"Al\" (all warnings but labels enabled)";
        "-warn-error" , Arg.String (Warnings.parse_options true),
          "<flags>  Enable or disable fatal warnings according to <flags>\n\
            \032    (see option -w for the list of flags)\n\
