@@ -16,8 +16,8 @@
          nil
          'string)
 ;labels
-   '("\\(\\([~?]\\|\\<\\)[a-z][a-z0-9_']*:\\)[^:=]" 1 brown)
-   '("[~?][ (]*[a-z][a-z0-9_']*" nil brown)
+   '("\\(\\([~?]\\|\\<\\)[a-z][a-zA-Z0-9_']*:\\)[^:=]" 1 brown)
+   '("[~?][ (]*[a-z][a-zA-Z0-9_']*" nil brown)
 ;modules
    '("\\<\\(assert\\|open\\|include\\)\\>" nil brown)
    '("`?\\<[A-Z][A-Za-z0-9_\']*\\>" nil MidnightBlue)
@@ -33,7 +33,7 @@
    '("\\<\\(object\\|struct\\|sig\\|begin\\|end\\)\\>" 2 include)
 ;control
    (list (concat
-          "\\<\\(do\\(ne\\|wnto\\)?\\|else\\|for\\|if"
+          "\\<\\(do\\(ne\\|wnto\\)?\\|else\\|for\\|i\\(f\\|gnore\\)"
           "\\|lazy\\|match\\|new\\|or\\|t\\(hen\\|o\\|ry\\)"
           "\\|w\\(h\\(en\\|ile\\)\\|ith\\)\\)\\>"
           "\\|\|\\|->\\|&\\|#")
