@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: lexgen.ml,v 1.9 1999/11/30 16:06:16 garrigue Exp $ *)
+(* $Id: lexgen.ml,v 1.10 2000/12/28 13:03:52 weis Exp $ *)
 
 (* Compiling a lexer definition *)
 
@@ -113,8 +113,8 @@ let rec nullable = function
     Empty      -> true
   | Chars _    -> false
   | Action _   -> false
-  | Seq(r1,r2) -> nullable r1 & nullable r2
-  | Alt(r1,r2) -> nullable r1 or nullable r2
+  | Seq(r1,r2) -> nullable r1 && nullable r2
+  | Alt(r1,r2) -> nullable r1 || nullable r2
   | Star r     -> true
 
 let rec firstpos = function
