@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: output.ml,v 1.14 1999/11/17 18:57:34 xleroy Exp $ *)
+(* $Id: output.ml,v 1.15 2000/08/10 09:58:07 xleroy Exp $ *)
 
 (* Output the DFA tables and its entry points *)
 
@@ -39,7 +39,7 @@ let copy_chars_win32 ic oc start stop =
 
 let copy_chars =
   match Sys.os_type with
-    "Win32" -> copy_chars_win32
+    "Win32" | "Cygwin" -> copy_chars_win32
   | _       -> copy_chars_unix
 
 let copy_chunk sourcefile ic oc loc =
