@@ -19,6 +19,9 @@ ocamlopt \- The Objective Caml native-code compiler
 .B \-compact
 ]
 [
+.B \-labels
+]
+[
 .B \-unsafe
 ]
 [
@@ -216,6 +219,12 @@ is saved in the file
 .B \-v
 Print the version number of the compiler.
 
+.TP
+.B \-labels
+Switch to ``commuting label mode''. Arguments must be
+explicitly labeled by labels appearing in types. Arguments with different
+labels may commute freely.
+
 .TP
 .B \-unsafe
 Turn bound checking off on array and string accesses (the v.(i) and
