@@ -6,11 +6,12 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../../LICENSE.  *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: dynlink.ml,v 1.23 2001/07/02 12:55:06 xleroy Exp $ *)
+(* $Id: dynlink.ml,v 1.26 2001/12/07 13:39:52 xleroy Exp $ *)
 
 (* Dynamic loading of .cmo files *)
 
@@ -29,6 +30,7 @@ type error =
   | Linking_error of string * linking_error
   | Corrupted_interface of string
   | File_not_found of string
+  | Cannot_open_dll of string
 
 exception Error of error
 
@@ -178,8 +180,13 @@ let loadfile file_name =
     if buffer = Config.cma_magic_number then begin
       let toc_pos = input_binary_int ic in  (* Go to table of contents *)
       seek_in ic toc_pos;
-      let toc = (input_value ic : library) in
-      List.iter (load_compunit ic file_name) toc.lib_units
+      let lib = (input_value ic : library) in
+      begin try 
+        Dll.open_dlls lib.lib_dllibs
+      with Failure reason ->
+        raise(Error(Cannot_open_dll reason))
+      end;
+      List.iter (load_compunit ic file_name) lib.lib_units
     end else
       raise(Error(Not_a_bytecode_file file_name));
     close_in ic
@@ -219,3 +226,5 @@ let error_message = function
       "corrupted interface file " ^ name
   | File_not_found name ->
       "cannot find file " ^ name ^ " in search path"
+  | Cannot_open_dll reason ->
+      "error loading shared library: " ^ reason
