@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: make_img.c,v 1.10 2000/04/05 18:30:17 doligez Exp $ */
+/* $Id: make_img.c,v 1.12 2001/12/07 13:39:54 xleroy Exp $ */
 
 #include "libgraph.h"
 #include "image.h"
@@ -42,21 +43,6 @@ value gr_make_image(value m)
                  ZPixmap, 0, NULL, width, height,
                  BitmapPad(grdisplay), 0);
 
-  /* To optimize RGB => color id calculation */
-  if( !direct_rgb ){
-    /* they are declared in color.c */
-    byte_order = idata->byte_order;
-    bitmap_unit = idata->bitmap_unit;
-    bits_per_pixel = idata->bits_per_pixel;
-#ifdef DIRECT_RGB_DEBUG
-    fprintf(stderr, "Byte_order: %d = %s\n", byte_order,
-            byte_order ? "LSBFirst" : "MSBFirst");
-    fprintf(stderr, "Bitmp_unit: %d\n", bitmap_unit);
-    fprintf(stderr, "Bits per pixel: %d\n", idata->bits_per_pixel);
-#endif
-    direct_rgb = True;
-  }
-
   bdata = (char *) stat_alloc(height * idata->bytes_per_line);
   idata->data = bdata;
   has_transp = False;
