@@ -7,12 +7,14 @@
 (*   Copyright 1999 Institut National de Recherche en Informatique et    *)
 (*   en Automatique and Kyoto University.  All rights reserved.          *)
 (*   This file is distributed under the terms of the GNU Library         *)
-(*   General Public License.                                             *)
+(*   General Public License, with the special exception on linking       *)
+(*   described in file ../../../LICENSE.                                 *)
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: editor.ml,v 1.27 2000/12/28 13:03:58 weis Exp $ *)
+(* $Id: editor.ml,v 1.32 2001/12/07 13:39:56 xleroy Exp $ *)
 
+open StdLabels
 open Tk
 open Parsetree
 open Location
@@ -33,25 +35,34 @@ let compiler_preferences () =
     (fun () ->
       ref := Textvariable.get variable = (if invert then "0" else "1"))
   in
+  let use_pp = ref (!Clflags.preprocessor <> None) in
   let chkbuttons, setflags = List.split
       (List.map
          ~f:(fun (text, ref, invert) -> mk_chkbutton ~text ~ref ~invert)
          [ "No pervasives", Clflags.nopervasives, false;
            "No warnings", Typecheck.nowarnings, false;
-           "Labels commute", Clflags.classic, true;
+           "No labels", Clflags.classic, false;
            "Recursive types", Clflags.recursive_types, false;
            "Lex on load", lex_on_load, false;
-           "Type on load", type_on_load, false ])
+           "Type on load", type_on_load, false;
+           "Preprocessor", use_pp, false ])
   in
+  let pp_command = Entry.create tl (* ~state:(if !use_pp then `Normal else`Disabled) *) in
+  begin match !Clflags.preprocessor with None -> ()
+  | Some pp -> Entry.insert pp_command ~index:(`Num 0) ~text:pp
+  end;
   let buttons = Frame.create tl in
   let ok = Button.create buttons ~text:"Ok" ~padx:20 ~command:
       begin fun () ->
         List.iter ~f:(fun f -> f ()) setflags;
+        Clflags.preprocessor :=
+          if !use_pp then Some (Entry.get pp_command) else None;
         destroy tl
       end
   and cancel = Jg_button.create_destroyer tl ~parent:buttons ~text:"Cancel"
   in
   pack chkbuttons ~side:`Top ~anchor:`W;
+  pack [pp_command] ~side:`Top ~anchor:`E;
   pack [ok;cancel] ~side:`Left ~fill:`X ~expand:true;
   pack [buttons] ~side:`Bottom ~fill:`X
 
@@ -99,7 +110,7 @@ let goto_line tw =
 
 let select_shell txt =
   let shells = Shell.get_all () in
-  let shells = Sort.list shells ~order:(fun (x,_) (y,_) -> x <= y) in
+  let shells = List.sort shells ~cmp:compare in
   let tl = Jg_toplevel.titled "Select Shell" in
   Jg_bind.escape_destroy tl;
   Wm.transient_set tl ~master:(Winfo.toplevel txt.tw);
@@ -145,7 +156,7 @@ let send_phrase txt =
         let i1,i2 = Text.tag_nextrange txt.tw ~tag:"sel" ~start:tstart in
         let phrase = Text.get txt.tw ~start:(i1,[]) ~stop:(i2,[]) in
         sh#send phrase;
-        if Str.string_match phrase ~pat:(Str.regexp ";;") ~pos:0
+        if Str.string_match (Str.regexp ";;") phrase 0
         then sh#send "\n" else sh#send ";;\n"
       with Not_found | Protocol.TkError _ ->
         let text = Text.get txt.tw ~start:tstart ~stop:tend in
@@ -204,16 +215,16 @@ let search_pos_window txt ~x ~y =
   let `Linechar (l, c) = Text.index txt.tw ~index:(`Atxy(x,y), []) in
   let text = Jg_text.get_all txt.tw in
   let pos = Searchpos.lines_to_chars l ~text + c in
-  try if txt.structure <> [] then
-    try Searchpos.search_pos_structure txt.structure ~pos
-    with Searchpos.Found_str (kind, env) ->
-      Searchpos.view_type kind ~env
-  else
-    try Searchpos.search_pos_signature
-        txt.psignature ~pos ~env:!Searchid.start_env;
-      ()
-    with Searchpos.Found_sig (kind, lid, env) ->
+  try if txt.structure <> [] then begin match
+    Searchpos.search_pos_structure txt.structure ~pos
+  with [] -> ()
+  | (kind, env, loc) :: _ -> Searchpos.view_type kind ~env
+  end else begin match
+    Searchpos.search_pos_signature txt.psignature ~pos ~env:!Searchid.start_env
+  with [] -> ()
+  | ((kind, lid), env, loc) :: _ ->
       Searchpos.view_decl lid ~kind ~env
+  end
   with Not_found -> ()
 
 let search_pos_menu txt ~x ~y =
@@ -221,20 +232,21 @@ let search_pos_menu txt ~x ~y =
   let `Linechar (l, c) = Text.index txt.tw ~index:(`Atxy(x,y), []) in
   let text = Jg_text.get_all txt.tw in
   let pos = Searchpos.lines_to_chars l ~text + c in
-  try if txt.structure <> [] then
-    try Searchpos.search_pos_structure txt.structure ~pos
-    with Searchpos.Found_str (kind, env) ->
+  try if txt.structure <> [] then begin match
+    Searchpos.search_pos_structure txt.structure ~pos
+  with [] -> ()
+  | (kind, env, loc) :: _ ->
       let menu = Searchpos.view_type_menu kind ~env ~parent:txt.tw in
       let x = x + Winfo.rootx txt.tw and y = y + Winfo.rooty txt.tw - 10 in
       Menu.popup menu ~x ~y
-  else
-    try Searchpos.search_pos_signature
-        txt.psignature ~pos ~env:!Searchid.start_env;
-      ()
-    with Searchpos.Found_sig (kind, lid, env) ->
+  end else begin match
+    Searchpos.search_pos_signature txt.psignature ~pos ~env:!Searchid.start_env
+  with [] -> ()
+  | ((kind, lid), env, loc) :: _ ->
       let menu = Searchpos.view_decl_menu lid ~kind ~env ~parent:txt.tw in
       let x = x + Winfo.rootx txt.tw and y = y + Winfo.rooty txt.tw - 10 in
       Menu.popup menu ~x ~y
+  end
   with Not_found -> ()
 
 let string_width s =
@@ -250,7 +262,7 @@ let indent_line =
   fun tw ->
     let `Linechar(l,c) = Text.index tw ~index:(ins,[])
     and line = Text.get tw ~start:(ins,[`Linestart]) ~stop:(ins,[`Lineend]) in
-    ignore (Str.string_match ~pat:reg line ~pos:0);
+    ignore (Str.string_match reg line 0);
     let len = Str.match_end () in
     if len < c then Text.insert tw ~index:(ins,[]) ~text:"\t" else
     let width = string_width (Str.matched_string line) in
@@ -260,7 +272,7 @@ let indent_line =
       let previous =
         Text.get tw ~start:(ins,[`Line(-1);`Linestart])
           ~stop:(ins,[`Line(-1);`Lineend]) in
-      ignore (Str.string_match ~pat:reg previous ~pos:0);
+      ignore (Str.string_match reg previous 0);
       let previous = Str.matched_string previous in
       let width_previous = string_width previous in
       if  width_previous <= width then 2 else width_previous - width
@@ -288,8 +300,9 @@ class editor ~top ~menus = object (self)
   method reset_window_menu =
     Menu.delete window_menu#menu ~first:(`Num 0) ~last:`End;
     List.iter
-      (Sort.list windows ~order:
-         (fun w1 w2 -> Filename.basename w1.name < Filename.basename w2.name))
+      (List.sort windows ~cmp:
+         (fun w1 w2 ->
+           compare (Filename.basename w1.name) (Filename.basename w2.name)))
       ~f:
       begin fun txt ->
         Menu.add_radiobutton window_menu#menu
@@ -340,7 +353,7 @@ class editor ~top ~menus = object (self)
       ~action:(fun _ ->
         let text =
           Text.get tw ~start:(`Mark"insert",[]) ~stop:(`Mark"insert",[`Lineend])
-        in ignore (Str.string_match ~pat:(Str.regexp "[ \t]*") text ~pos:0);
+        in ignore (Str.string_match (Str.regexp "[ \t]*") text 0);
         if Str.match_end () <> String.length text then begin
           Clipboard.clear ();
           Clipboard.append ~data:text ()
@@ -390,7 +403,7 @@ class editor ~top ~menus = object (self)
     try
       if Sys.file_exists name then
         if txt.name = name then
-          Sys.rename ~src:name ~dst:(name ^ "~")
+          Sys.rename name (name ^ "~")
         else begin match
           Jg_message.ask ~master:top ~title:"Save"
             ("File `" ^ name ^ "' exists. Overwrite it?")
@@ -432,7 +445,7 @@ class editor ~top ~menus = object (self)
       and buf = String.create 4096 in
       Text.delete tw ~start:tstart ~stop:tend;
       while
-        len := input file ~buf ~pos:0 ~len:4096;
+        len := input file buf 0 4096;
         !len > 0
       do
         Jg_text.output tw ~buf ~pos:0 ~len:!len
@@ -492,10 +505,23 @@ class editor ~top ~menus = object (self)
     if not (Winfo.ismapped top) then Wm.deiconify top;
     match file with None -> ()
     | Some file ->
-        self#load_text [file]; 
+        self#load_text [file];
         Text.mark_set current_tw ~mark:"insert" ~index:(tpos pos);
-        Text.yview_index current_tw
-          ~index:(`Linechar(1,0),[`Char pos; `Line (-2)])
+        try
+          let index =
+            Text.search current_tw ~switches:[`Backwards] ~pattern:"*)"
+              ~start:(tpos pos) ~stop:(tpos pos ~modi:[`Line(-1)]) in
+          let index =
+            Text.search current_tw ~switches:[`Backwards] ~pattern:"(*"
+              ~start:(index,[]) ~stop:(tpos pos ~modi:[`Line(-20)]) in
+          let s = Text.get current_tw ~start:(index,[`Line(-1);`Linestart])
+              ~stop:(index,[`Line(-1);`Lineend]) in
+          for i = 0 to String.length s - 1 do
+            match s.[i] with '\t'|' ' -> () | _ -> raise Not_found
+          done;
+          Text.yview_index current_tw ~index:(index,[`Line(-1)])
+        with _ ->
+          Text.yview_index current_tw ~index:(tpos pos ~modi:[`Line(-2)])
 
   initializer
     (* Create a first window *)
