@@ -7,11 +7,12 @@
 (*   Copyright 1999 Institut National de Recherche en Informatique et    *)
 (*   en Automatique and Kyoto University.  All rights reserved.          *)
 (*   This file is distributed under the terms of the GNU Library         *)
-(*   General Public License.                                             *)
+(*   General Public License, with the special exception on linking       *)
+(*   described in file ../../../LICENSE.                                 *)
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: jg_message.mli,v 1.6 2000/04/20 02:45:44 garrigue Exp $ *)
+(* $Id: jg_message.mli,v 1.8 2001/12/07 13:39:59 xleroy Exp $ *)
 
 open Widget
 
@@ -26,4 +27,4 @@ val formatted :
 
 val ask :
     title:string -> ?master:toplevel widget ->
-    ?cancel:bool -> string -> [`cancel|`no|`yes]
+    ?no:bool -> ?cancel:bool -> string -> [`cancel|`no|`yes]
