@@ -7,12 +7,15 @@
 (*   Copyright 1999 Institut National de Recherche en Informatique et    *)
 (*   en Automatique and Kyoto University.  All rights reserved.          *)
 (*   This file is distributed under the terms of the GNU Library         *)
-(*   General Public License.                                             *)
+(*   General Public License, with the special exception on linking       *)
+(*   described in file ../../../LICENSE.                                 *)
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: searchpos.ml,v 1.20 2001/04/13 09:27:04 garrigue Exp $ *)
+(* $Id: searchpos.ml,v 1.26 2001/12/07 13:40:01 xleroy Exp $ *)
 
+open StdLabels
+open Support
 open Tk
 open Jg_tk
 open Parsetree
@@ -36,7 +39,23 @@ let lines_to_chars n ~text:s =
   in ltc n ~pos:0
 
 let in_loc loc ~pos =
-  pos >= loc.loc_start && pos < loc.loc_end
+  loc.loc_ghost || pos >= loc.loc_start && pos < loc.loc_end
+
+let le_loc loc1 loc2 =
+  loc1.loc_start <= loc2.loc_start
+  && loc1.loc_end >= loc2.loc_end
+
+let add_found ~found sol ~env ~loc =
+  if loc.loc_ghost then () else
+  if List.exists !found ~f:(fun (_,_,loc') -> le_loc loc loc') then ()
+  else found := (sol, env, loc) ::
+    List.filter !found ~f:(fun (_,_,loc') -> not (le_loc loc' loc))
+
+let observe ~ref ?init f x =
+  let old = !ref in
+  begin match init with None -> () | Some x -> ref := x end;
+  try (f x : unit); let v = !ref in ref := old; v
+  with exn -> ref := old; raise exn
 
 let rec string_of_longident = function
     Lident s -> s
@@ -69,7 +88,7 @@ let rec list_of_path = function
 
 class buffer ~size = object
   val buffer = Buffer.create size
-  method out ~buf = Buffer.add_substring buffer buf
+  method out buf = Buffer.add_substring buffer buf
   method get = Buffer.contents buffer
 end
 
@@ -77,16 +96,20 @@ end
 
 type skind = [`Type|`Class|`Module|`Modtype]
 
-exception Found_sig of skind * Longident.t * Env.t
+let found_sig = ref ([] : ((skind * Longident.t) * Env.t * Location.t) list)
+let add_found_sig = add_found ~found:found_sig
 
 let rec search_pos_type t ~pos ~env =
   if in_loc ~pos t.ptyp_loc then
-  begin (match t.ptyp_desc with
+  begin match t.ptyp_desc with
     Ptyp_any
   | Ptyp_var _ -> ()
   | Ptyp_variant(tl, _, _) ->
-      List.iter tl
-        ~f:(fun (_,_,tl) -> List.iter tl ~f:(search_pos_type ~pos ~env))
+      List.iter tl ~f:
+        begin function
+            Rtag (_,_,tl) -> List.iter tl ~f:(search_pos_type ~pos ~env)
+          | Rinherit st -> search_pos_type ~pos ~env st
+        end
   | Ptyp_arrow (_, t1, t2) ->
       search_pos_type t1 ~pos ~env;
       search_pos_type t2 ~pos ~env
@@ -94,7 +117,7 @@ let rec search_pos_type t ~pos ~env =
       List.iter tl ~f:(search_pos_type ~pos ~env)
   | Ptyp_constr (lid, tl) ->
       List.iter tl ~f:(search_pos_type ~pos ~env);
-      raise (Found_sig (`Type, lid, env))
+      add_found_sig (`Type, lid) ~env ~loc:t.ptyp_loc
   | Ptyp_object fl ->
       List.iter fl ~f:
         begin function
@@ -103,16 +126,15 @@ let rec search_pos_type t ~pos ~env =
         end
   | Ptyp_class (lid, tl, _) ->
       List.iter tl ~f:(search_pos_type ~pos ~env);
-      raise (Found_sig (`Type, lid, env))
-  | Ptyp_alias (t, _) -> search_pos_type ~pos ~env t);
-  raise Not_found
+      add_found_sig (`Type, lid) ~env ~loc:t.ptyp_loc
+  | Ptyp_alias (t, _) -> search_pos_type ~pos ~env t
   end
 
 let rec search_pos_class_type cl ~pos ~env =
-  if in_loc cl.pcty_loc ~pos then begin
+  if in_loc cl.pcty_loc ~pos then
     begin match cl.pcty_desc with
       Pcty_constr (lid, _) ->
-        raise (Found_sig (`Class, lid, env))
+        add_found_sig (`Class, lid) ~env ~loc:cl.pcty_loc
     | Pcty_signature (_, cfl) ->
         List.iter cfl ~f:
           begin function
@@ -133,9 +155,7 @@ let rec search_pos_class_type cl ~pos ~env =
     | Pcty_fun (_, ty, cty) ->
         search_pos_type ty ~pos ~env;
         search_pos_class_type cty ~pos ~env
-    end;
-    raise Not_found
-  end
+    end
 
 let search_pos_type_decl td ~pos ~env =
   if in_loc ~pos td.ptype_loc then begin
@@ -150,8 +170,7 @@ let search_pos_type_decl td ~pos ~env =
           ~f:(fun (_, tl) -> List.iter tl ~f:(search_pos_type ~pos ~env))
     | Ptype_record dl ->
         List.iter dl ~f:(fun (_, _, t) -> search_pos_type t ~pos ~env)
-    end;
-    raise Not_found
+    end
   end
   
 let rec search_pos_signature l ~pos ~env =
@@ -170,14 +189,14 @@ let rec search_pos_signature l ~pos ~env =
         with Typemod.Error _ | Typeclass.Error _
         | Typetexp.Error _  | Typedecl.Error _ -> env
     in
-    if in_loc ~pos pt.psig_loc then begin
+    if in_loc ~pos pt.psig_loc then
       begin match pt.psig_desc with
         Psig_value (_, desc) -> search_pos_type desc.pval_type ~pos ~env
       | Psig_type l ->
           List.iter l ~f:(fun (_,desc) -> search_pos_type_decl ~pos desc ~env)
       | Psig_exception (_, l) ->
           List.iter l ~f:(search_pos_type ~pos ~env);
-          raise (Found_sig (`Type, Lident "exn", env))
+          add_found_sig (`Type, Lident "exn") ~env ~loc:pt.psig_loc
       | Psig_module (_, t) -> 
           search_pos_module t ~pos ~env
       | Psig_modtype (_, Pmodtype_manifest t) ->
@@ -190,18 +209,16 @@ let rec search_pos_signature l ~pos ~env =
           List.iter l
             ~f:(fun ci -> search_pos_class_type ci.pci_expr ~pos ~env)
       (* The last cases should not happen in generated interfaces *) 
-      | Psig_open lid -> raise (Found_sig (`Module, lid, env))
+      | Psig_open lid -> add_found_sig (`Module, lid) ~env ~loc:pt.psig_loc
       | Psig_include t -> search_pos_module t ~pos ~env
       end;
-      raise Not_found
-    end;
     env
   end)
 
 and search_pos_module m ~pos ~env =
   if in_loc m.pmty_loc ~pos then begin
     begin match m.pmty_desc with
-      Pmty_ident lid -> raise (Found_sig (`Modtype, lid, env))
+      Pmty_ident lid -> add_found_sig (`Modtype, lid) ~env ~loc:m.pmty_loc
     | Pmty_signature sg -> search_pos_signature sg ~pos ~env
     | Pmty_functor (_ , m1, m2) ->
         search_pos_module m1 ~pos ~env;
@@ -213,14 +230,17 @@ and search_pos_module m ~pos ~env =
               _, Pwith_type t -> search_pos_type_decl t ~pos ~env 
             | _ -> ()
           end
-    end;
-    raise Not_found
+    end
   end
 
+let search_pos_signature l ~pos ~env =
+  observe ~ref:found_sig (search_pos_signature ~pos ~env) l
+
 (* the module display machinery *)
 
 type module_widgets =
     { mw_frame: Widget.frame Widget.widget;
+      mw_title: Widget.label Widget.widget option;
       mw_detach: Widget.button Widget.widget;
       mw_edit: Widget.button Widget.widget;
       mw_intf: Widget.button Widget.widget }
@@ -228,13 +248,13 @@ type module_widgets =
 let shown_modules = Hashtbl.create 17
 let default_frame = ref None
 let filter_modules () =
-  Hashtbl.iter shown_modules ~f:
-    begin fun ~key ~data ->
+  Hashtbl.iter
+    (fun key data ->
       if not (Winfo.exists data.mw_frame) then
-        Hashtbl.remove shown_modules key
-    end
+        Hashtbl.remove shown_modules key)
+    shown_modules
 let add_shown_module path ~widgets =
-  Hashtbl.add shown_modules ~key:path ~data:widgets
+  Hashtbl'.add shown_modules ~key:path ~data:widgets
 let find_shown_module path =
   try
     filter_modules ();
@@ -297,12 +317,14 @@ let rec view_signature ?title ?path ?(env = !start_env) ?(detach=false) sign =
   in
   let tl, tw, finish =
     try match path, !default_frame with
-      None, Some mw when not detach ->
+      None, Some ({mw_title=Some label} as mw) when not detach ->
         Button.configure mw.mw_detach
           ~command:(fun () -> view_signature sign ~title ~env);
         pack [mw.mw_detach] ~side:`Left;
         Pack.forget [mw.mw_edit; mw.mw_intf];
         List.iter ~f:destroy (Winfo.children mw.mw_frame);
+        Label.configure label ~text:title;
+        pack [label] ~fill:`X ~side:`Bottom;
         Jg_message.formatted ~title ~on:mw.mw_frame ~maxheight:15 ()
     | None, _ -> raise Not_found
     | Some path, _ ->
@@ -312,10 +334,15 @@ let rec view_signature ?title ?path ?(env = !start_env) ?(detach=false) sign =
             view_module path ~env;
             find_shown_module path
         in
+        begin match mw.mw_title with None -> ()
+        | Some label ->
+            Label.configure label ~text:title;
+            pack [label] ~fill:`X ~side:`Bottom
+        end;
         Button.configure mw.mw_detach
           ~command:(fun () -> view_signature sign ~title ~env ~detach:true);
         pack [mw.mw_detach] ~side:`Left;
-        Pack.forget [mw.mw_edit; mw.mw_intf];
+        let repack = ref false in
         List.iter2 [mw.mw_edit; mw.mw_intf] [".ml"; ".mli"] ~f:
           begin fun button ext ->
             try
@@ -325,12 +352,14 @@ let rec view_signature ?title ?path ?(env = !start_env) ?(detach=false) sign =
                   (String.uncapitalize (Ident.name id) ^ ext) in
               Button.configure button
                 ~command:(fun () -> edit_source ~file ~path ~sign);
+              if !repack then Pack.forget [button] else
+              if not (Winfo.viewable button) then repack := true;
               pack [button] ~side:`Left
-            with Not_found -> ()
+            with Not_found ->
+              Pack.forget [button]
           end;
         let top = Winfo.toplevel mw.mw_frame in
         if not (Winfo.ismapped top) then Wm.deiconify top;
-        Focus.set top;
         List.iter ~f:destroy (Winfo.children mw.mw_frame);
         Jg_message.formatted ~title ~on:mw.mw_frame ~maxheight:15 ()
     with Not_found ->
@@ -366,23 +395,24 @@ let rec view_signature ?title ?path ?(env = !start_env) ?(detach=false) sign =
     ~action:(fun ev ->
       let `Linechar (l, c) =
         Text.index tw ~index:(`Atxy(ev.ev_MouseX,ev.ev_MouseY), []) in
-      try try
-        search_pos_signature pt ~pos:(lines_to_chars l ~text + c) ~env;
-        break ()
-      with Found_sig (kind, lid, env) -> view_decl lid ~kind ~env
+      try
+        match search_pos_signature pt ~pos:(lines_to_chars l ~text + c) ~env
+        with [] -> break ()
+        | ((kind, lid), env, loc) :: _ -> view_decl lid ~kind ~env
       with Not_found | Env.Error _ -> ());
-  bind tw ~events:[`ButtonPressDetail 3] ~fields:[`MouseX;`MouseY] ~breakable:true
+  bind tw ~events:[`ButtonPressDetail 3] ~breakable:true
+    ~fields:[`MouseX;`MouseY]
     ~action:(fun ev ->
       let x = ev.ev_MouseX and y = ev.ev_MouseY in
       let `Linechar (l, c) =
         Text.index tw ~index:(`Atxy(x,y), []) in
-      try try
-        search_pos_signature pt ~pos:(lines_to_chars l ~text + c) ~env;
-        break ()
-      with Found_sig (kind, lid, env) ->
-        let menu = view_decl_menu lid ~kind ~env ~parent:tw in
-        let x = x + Winfo.rootx tw and y = y + Winfo.rooty tw - 10 in
-        Menu.popup menu ~x ~y
+      try
+        match search_pos_signature pt ~pos:(lines_to_chars l ~text + c) ~env
+        with [] -> break ()
+        | ((kind, lid), env, loc) :: _ ->
+            let menu = view_decl_menu lid ~kind ~env ~parent:tw in
+            let x = x + Winfo.rootx tw and y = y + Winfo.rooty tw - 10 in
+            Menu.popup menu ~x ~y
       with Not_found -> ())
 
 and view_signature_item sign ~path ~env =
@@ -474,7 +504,7 @@ and view_decl_menu lid ~kind ~env ~parent =
     let buf = new buffer ~size:60 in
     let (fo,ff) = Format.get_formatter_output_functions ()
     and margin = Format.get_margin () in
-    Format.set_formatter_output_functions ~out:buf#out ~flush:(fun () -> ());
+    Format.set_formatter_output_functions buf#out (fun () -> ());
     Format.set_margin 60;
     Format.open_hbox ();
     if kind = `Type then
@@ -488,9 +518,9 @@ and view_decl_menu lid ~kind ~env ~parent =
         Format.std_formatter
         (find_modtype path env);
     Format.close_box (); Format.print_flush ();
-    Format.set_formatter_output_functions ~out:fo ~flush:ff;
+    Format.set_formatter_output_functions fo ff;
     Format.set_margin margin;
-    let l = Str.split ~sep:~!"\n" buf#get in
+    let l = Str.split ~!"\n" buf#get in
     let font =
       let font =
         Option.get Widget.default_toplevel ~name:"font" ~clas:"Font" in
@@ -511,7 +541,6 @@ type fkind = [
   | `Class of Path.t * Types.class_type
   | `Module of Path.t * Types.module_type
 ]
-exception Found_str of fkind * Env.t
 
 let view_type kind ~env =
   match kind with
@@ -573,16 +602,16 @@ let view_type_menu kind ~env ~parent =
       let buf = new buffer ~size:60 in
       let (fo,ff) = Format.get_formatter_output_functions ()
       and margin = Format.get_margin () in
-      Format.set_formatter_output_functions ~out:buf#out ~flush:(fun () -> ());
+      Format.set_formatter_output_functions buf#out ignore;
       Format.set_margin 60;
       Format.open_hbox ();
       Printtyp.reset ();
       Printtyp.mark_loops ty;
       Printtyp.type_expr Format.std_formatter ty;
       Format.close_box (); Format.print_flush ();
-      Format.set_formatter_output_functions ~out:fo ~flush:ff;
+      Format.set_formatter_output_functions fo ff;
       Format.set_margin margin;
-      let l = Str.split ~sep:~!"\n" buf#get in
+      let l = Str.split ~!"\n" buf#get in
       let font =
         let font =
           Option.get Widget.default_toplevel ~name:"font" ~clas:"Font" in
@@ -603,6 +632,9 @@ let view_type_menu kind ~env ~parent =
   end;
   menu
 
+let found_str = ref ([] : (fkind * Env.t * Location.t) list)
+let add_found_str = add_found ~found:found_str
+
 let rec search_pos_structure ~pos str =
   List.iter str ~f:
   begin function
@@ -632,7 +664,8 @@ and search_pos_class_expr ~pos cl =
   if in_loc cl.cl_loc ~pos then begin
     begin match cl.cl_desc with
       Tclass_ident path ->
-        raise (Found_str (`Class (path, cl.cl_type), !start_env))
+        add_found_str (`Class (path, cl.cl_type))
+          ~env:!start_env ~loc:cl.cl_loc
     | Tclass_structure cls ->
         List.iter cls.cl_field ~f:
           begin function
@@ -667,17 +700,19 @@ and search_pos_class_expr ~pos cl =
     | Tclass_constraint (cl, _, _, _) ->
         search_pos_class_expr cl ~pos
     end;
-    raise (Found_str
-             (`Class (Pident (Ident.create "c"), cl.cl_type), !start_env))
+    add_found_str (`Class (Pident (Ident.create "c"), cl.cl_type))
+      ~env:!start_env ~loc:cl.cl_loc
   end
 
 and search_pos_expr ~pos exp =
   if in_loc exp.exp_loc ~pos then begin
   begin match exp.exp_desc with
     Texp_ident (path, _) ->
-      raise (Found_str (`Exp(`Val path, exp.exp_type), exp.exp_env))
+      add_found_str (`Exp(`Val path, exp.exp_type))
+        ~env:exp.exp_env ~loc:exp.exp_loc
   | Texp_constant v ->
-      raise (Found_str (`Exp(`Const, exp.exp_type), exp.exp_env))
+      add_found_str (`Exp(`Const, exp.exp_type))
+        ~env:exp.exp_env ~loc:exp.exp_loc
   | Texp_let (_, expl, exp) ->
       List.iter expl ~f:
       begin fun (pat, exp') ->
@@ -734,12 +769,15 @@ and search_pos_expr ~pos exp =
       search_pos_expr a ~pos; search_pos_expr b ~pos
   | Texp_send (exp, _) -> search_pos_expr exp ~pos
   | Texp_new (path, _) ->
-      raise (Found_str (`Exp(`New path, exp.exp_type), exp.exp_env))
+      add_found_str (`Exp(`New path, exp.exp_type))
+        ~env:exp.exp_env ~loc:exp.exp_loc
   | Texp_instvar (_,path) ->
-      raise (Found_str (`Exp(`Var path, exp.exp_type), exp.exp_env))
+      add_found_str (`Exp(`Var path, exp.exp_type))
+        ~env:exp.exp_env ~loc:exp.exp_loc
   | Texp_setinstvar (_, path, exp) ->
       search_pos_expr exp ~pos;
-      raise (Found_str (`Exp(`Var path, exp.exp_type), exp.exp_env))
+      add_found_str (`Exp(`Var path, exp.exp_type))
+        ~env:exp.exp_env ~loc:exp.exp_loc
   | Texp_override (_, l) ->
       List.iter l ~f:(fun (_, exp) -> search_pos_expr exp ~pos)
   | Texp_letmodule (id, modexp, exp) ->
@@ -749,7 +787,7 @@ and search_pos_expr ~pos exp =
   | Texp_assert exp ->
       search_pos_expr exp ~pos
   end;
-  raise (Found_str (`Exp(`Expr, exp.exp_type), exp.exp_env))
+  add_found_str (`Exp(`Expr, exp.exp_type)) ~env:exp.exp_env ~loc:exp.exp_loc
   end
 
 and search_pos_pat ~pos ~env pat =
@@ -757,10 +795,11 @@ and search_pos_pat ~pos ~env pat =
   begin match pat.pat_desc with
     Tpat_any -> ()
   | Tpat_var id ->
-      raise (Found_str (`Exp(`Val (Pident id), pat.pat_type), env))
+      add_found_str (`Exp(`Val (Pident id), pat.pat_type))
+        ~env ~loc:pat.pat_loc
   | Tpat_alias (pat, _) -> search_pos_pat pat ~pos ~env
   | Tpat_constant _ ->
-      raise (Found_str (`Exp(`Const, pat.pat_type), env))
+      add_found_str (`Exp(`Const, pat.pat_type)) ~env ~loc:pat.pat_loc
   | Tpat_tuple l ->
       List.iter l ~f:(search_pos_pat ~pos ~env)
   | Tpat_construct (_, l) ->
@@ -771,24 +810,29 @@ and search_pos_pat ~pos ~env pat =
       List.iter l ~f:(fun (_, pat) -> search_pos_pat pat ~pos ~env)
   | Tpat_array l ->
       List.iter l ~f:(search_pos_pat ~pos ~env)
-  | Tpat_or (a, b) ->
+  | Tpat_or (a, b, None) ->
       search_pos_pat a ~pos ~env; search_pos_pat b ~pos ~env
+  | Tpat_or (_, _, Some _) ->
+      ()
   end;
-  raise (Found_str (`Exp(`Pat, pat.pat_type), env))
+  add_found_str (`Exp(`Pat, pat.pat_type)) ~env ~loc:pat.pat_loc
   end
 
 and search_pos_module_expr ~pos m =
   if in_loc m.mod_loc ~pos then begin
     begin match m.mod_desc with
       Tmod_ident path ->
-        raise
-          (Found_str (`Module (path, m.mod_type), m.mod_env))
+        add_found_str (`Module (path, m.mod_type))
+          ~env:m.mod_env ~loc:m.mod_loc
     | Tmod_structure str -> search_pos_structure str ~pos
     | Tmod_functor (_, _, m) -> search_pos_module_expr m ~pos
     | Tmod_apply (a, b, _) ->
         search_pos_module_expr a ~pos; search_pos_module_expr b ~pos
     | Tmod_constraint (m, _, _) -> search_pos_module_expr m ~pos
     end;
-    raise (Found_str (`Module (Pident (Ident.create "M"), m.mod_type),
-                      m.mod_env))
+    add_found_str (`Module (Pident (Ident.create "M"), m.mod_type))
+      ~env:m.mod_env ~loc:m.mod_loc
   end
+
+let search_pos_structure ~pos str =
+  observe ~ref:found_str (search_pos_structure ~pos) str
