@@ -7,12 +7,15 @@
 (*   Copyright 1999 Institut National de Recherche en Informatique et    *)
 (*   en Automatique and Kyoto University.  All rights reserved.          *)
 (*   This file is distributed under the terms of the GNU Library         *)
-(*   General Public License.                                             *)
+(*   General Public License, with the special exception on linking       *)
+(*   described in file ../../../LICENSE.                                 *)
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: shell.ml,v 1.25 2000/12/28 13:04:26 weis Exp $ *)
+(* $Id: shell.ml,v 1.31 2001/12/07 13:40:02 xleroy Exp $ *)
 
+open StdLabels
+module Unix = UnixLabels
 open Tk
 open Jg_tk
 open Dummy
@@ -38,18 +41,11 @@ class ['a] history () = object
     List.nth history ((l + count - 1) mod l)
 end
 
-let dump_mem ?(pos = 0) ?len obj =
-  if not (Obj.is_block obj) then invalid_arg "Shell.dump_mem";
-  let len =
-    match len with
-    | None -> Obj.size obj * Sys.word_size / 8 - pos
-    | Some x -> x in
-  let buf = Buffer.create 256 in
-  for i = pos to len - 1 do
-    let c = String.unsafe_get (Obj.obj obj) i in
-    Buffer.add_string buf (Printf.sprintf "%02x" (Char.code c))
-  done;
-  Buffer.contents buf
+let dump_handle (h : Unix.file_descr) =
+  let obj = Obj.repr h in
+  if Obj.is_int obj || Obj.tag obj <> Obj.final_tag then
+    invalid_arg "Shell.dump_handle";
+  Nativeint.format "%x" (Obj.obj obj)
 
 (* The shell class. Now encapsulated *)
 
@@ -64,7 +60,7 @@ object (self)
   val pid =
     let env =
       if Sys.os_type = "Win32" then
-        let sigdef = "CAMLSIGPIPE=" ^ dump_mem (Obj.repr sig2) in
+        let sigdef = "CAMLSIGPIPE=" ^ dump_handle sig2 in
         Array.append env [|sigdef|]
       else env
     in
@@ -194,9 +190,9 @@ object (self)
       let fileinput_thread fd =
         let buf = String.create 1024 in
         let len = ref 0 in
-        try while len := ThreadUnix.read fd ~buf ~pos:0 ~len:1024; !len > 0 do
+        try while len := Unix.read fd ~buf ~pos:0 ~len:1024; !len > 0 do
           Mutex.lock imutex;
-          Buffer.add_substring ibuffer buf ~pos:0 ~len:!len;
+          Buffer.add_substring ibuffer buf 0 !len;
           Mutex.unlock imutex
         done with Unix.Unix_error _ -> ()
       in
@@ -204,7 +200,7 @@ object (self)
       let rec read_buffer () =
         Mutex.lock imutex;
         if Buffer.length ibuffer > 0 then begin
-          self#insert (Str.global_replace ~pat:~!"\r\n" ~templ:"\n"
+          self#insert (Str.global_replace ~!"\r\n" "\n"
                          (Buffer.contents ibuffer));
           Buffer.reset ibuffer;
           Text.mark_set textw ~mark:"input" ~index:(`Mark"insert",[`Char(-1)])
@@ -250,21 +246,29 @@ let may_exec =
 
 let path_sep = if Sys.os_type = "Win32" then ";" else ":"
 
-let warnings = ref "A"
+let warnings = ref "Al"
+
+let program_not_found prog =
+  ignore begin
+    Jg_message.ask ~cancel:false ~no:false ~title:"Error"
+      ("Program \"" ^ String.escaped prog ^ "\"\nwas not found in path")
+  end
 
 let f ~prog ~title =
   let progargs =
-    List.filter ~f:((<>) "") (Str.split ~sep:~!" " prog) in
+    List.filter ~f:((<>) "") (Str.split ~!" " prog) in
   if progargs = [] then () else
   let prog = List.hd progargs in
   let path =
     try Sys.getenv "PATH" with Not_found -> "/bin" ^ path_sep ^ "/usr/bin" in
-  let exec_path = Str.split ~sep:~!path_sep path in
+  let exec_path = Str.split ~!path_sep path in
+  let exec_path =
+    if Sys.os_type = "Win32" then "."::exec_path else exec_path in
   let exists =
     if not (Filename.is_implicit prog) then may_exec prog else
     List.exists exec_path
       ~f:(fun dir -> may_exec (Filename.concat dir prog)) in
-  if not exists then () else
+  if not exists then program_not_found prog else
   let tl = Jg_toplevel.titled title in
   let menus = Frame.create tl ~name:"menubar" in
   let file_menu = new Jg_menu.c "File" ~parent:menus
@@ -280,14 +284,14 @@ let f ~prog ~title =
   pack [frame] ~fill:`Both ~expand:true;
   let env = Array.map (Unix.environment ()) ~f:
       begin fun s ->
-        if Str.string_match ~pat:~!"TERM=" s ~pos:0 then "TERM=dumb" else s
+        if Str.string_match ~!"TERM=" s 0 then "TERM=dumb" else s
       end in
   let load_path =
     List2.flat_map !Config.load_path ~f:(fun dir -> ["-I"; dir]) in
-  let labels = if !Clflags.classic then [] else ["-labels"] in
+  let labels = if !Clflags.classic then ["-nolabels"] else [] in
   let rectypes = if !Clflags.recursive_types then ["-rectypes"] else [] in
   let warnings =
-    if List.mem "-w" progargs || !warnings = "A" then []
+    if List.mem "-w" progargs || !warnings = "Al" then []
     else ["-w"; !warnings]
   in
   let args =
