@@ -7,13 +7,15 @@
 (*   Copyright 1999 Institut National de Recherche en Informatique et    *)
 (*   en Automatique and Kyoto University.  All rights reserved.          *)
 (*   This file is distributed under the terms of the GNU Library         *)
-(*   General Public License.                                             *)
+(*   General Public License, with the special exception on linking       *)
+(*   described in file ../../../LICENSE.                                 *)
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: useunix.ml,v 1.10 2000/04/12 03:43:19 garrigue Exp $ *)
+(* $Id: useunix.ml,v 1.12 2001/12/07 13:40:02 xleroy Exp $ *)
 
-open Unix
+open StdLabels
+open UnixLabels
 
 let get_files_in_directory dir =
   match
@@ -30,7 +32,7 @@ let get_files_in_directory dir =
         | None ->
             closedir dirh; l 
       in
-      Sort.list ~order:(<=) (get_them [])
+      List.sort ~cmp:compare (get_them [])
 
 let is_directory name =
   try
