@@ -7,12 +7,14 @@
 (*   Copyright 1999 Institut National de Recherche en Informatique et    *)
 (*   en Automatique and Kyoto University.  All rights reserved.          *)
 (*   This file is distributed under the terms of the GNU Library         *)
-(*   General Public License.                                             *)
+(*   General Public License, with the special exception on linking       *)
+(*   described in file ../../../LICENSE.                                 *)
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: viewer.ml,v 1.19 2001/04/15 21:56:24 garrigue Exp $ *)
+(* $Id: viewer.ml,v 1.26 2001/12/07 13:40:03 xleroy Exp $ *)
 
+open StdLabels
 open Tk
 open Jg_tk
 open Mytypes
@@ -42,7 +44,7 @@ let list_modules ~path =
 
 let reset_modules box =
   Listbox.delete box ~first:(`Num 0) ~last:`End;
-  module_list := Sort.list ~order:(Jg_completion.lt_string ~nocase:true)
+  module_list := Sort.list (Jg_completion.lt_string ~nocase:true)
       (list_modules ~path:!Config.load_path);
   Listbox.insert box ~index:`End ~texts:!module_list;
   Jg_box.recenter box ~index:(`Num 0)
@@ -99,10 +101,7 @@ let choose_symbol ~title ~env ?signature ?path l =
   and detach = Button.create buttons ~text:"Detach"
   and edit = Button.create buttons ~text:"Impl"
   and intf = Button.create buttons ~text:"Intf" in
-  let l = Sort.list l ~order:
-      (fun (li1, _) (li2,_) ->
-        string_of_longident li1 < string_of_longident li2)
-  in
+  let l = List.sort l ~cmp:(fun (li1, _) (li2,_) -> compare li1 li2) in
   let nl = List.map l ~f:
     begin fun (li, k) ->
       string_of_longident li ^ " (" ^ string_of_kind k ^ ")"
@@ -143,7 +142,7 @@ let choose_symbol ~title ~env ?signature ?path l =
       let frame = Frame.create tl in
       pack [frame] ~side:`Bottom ~fill:`X;
       add_shown_module path
-        ~widgets:{ mw_frame = frame; mw_detach = detach;
+        ~widgets:{ mw_frame = frame; mw_title = None; mw_detach = detach;
                    mw_edit = edit; mw_intf = intf }
   end
 
@@ -160,46 +159,49 @@ let guess_search_mode s : [`Type | `Long | `Pattern] =
   done;
   if !is_type then `Type else if !is_long then `Long else `Pattern
 
-let search_which = ref "itself"
+
+let search_string ?(mode="symbol") ew =
+  let text = Entry.get ew in
+  try
+    if text = "" then () else
+    let l = match mode with
+      "Name" ->
+        begin match guess_search_mode text with
+          `Long -> search_string_symbol text
+        | `Pattern -> search_pattern_symbol text
+        | `Type -> search_string_type text ~mode:`included
+        end
+    | "Type" -> search_string_type text ~mode:`included
+    | "Exact" -> search_string_type text ~mode:`exact
+    | _ -> assert false
+    in
+    match l with [] -> ()
+    | [lid,kind] -> view_symbol lid ~kind ~env:!start_env
+    | l          -> choose_symbol ~title:"Choose symbol" ~env:!start_env l
+  with Searchid.Error (s,e) ->
+    Entry.icursor ew ~index:(`Num s)
+
+let search_which = ref "Name"
 
 let search_symbol () =
   if !module_list = [] then
-  module_list := Sort.list ~order:(<) (list_modules ~path:!Config.load_path);
+  module_list := List.sort ~cmp:compare (list_modules ~path:!Config.load_path);
   let tl = Jg_toplevel.titled "Search symbol" in
   Jg_bind.escape_destroy tl;
   let ew = Entry.create tl ~width:30 in
   let choice = Frame.create tl
   and which = Textvariable.create ~on:tl () in
   let itself = Radiobutton.create choice ~text:"Itself"
-        ~variable:which ~value:"itself"
+        ~variable:which ~value:"Name"
   and extype = Radiobutton.create choice ~text:"Exact type"
-        ~variable:which ~value:"exact"
+        ~variable:which ~value:"Exact"
   and iotype = Radiobutton.create choice ~text:"Included type"
-        ~variable:which ~value:"iotype"
+        ~variable:which ~value:"Type"
   and buttons = Frame.create tl in
   let search = Button.create buttons ~text:"Search" ~command:
     begin fun () ->
       search_which := Textvariable.get which;
-      let text = Entry.get ew in
-      try if text = "" then () else      
-        let l =
-          match !search_which with
-            "itself" ->
-              begin match guess_search_mode text with
-                `Long -> search_string_symbol text
-              | `Pattern -> search_pattern_symbol text
-              | `Type -> search_string_type text ~mode:`included
-              end
-          | "iotype" -> search_string_type text ~mode:`included
-          | "exact" -> search_string_type text ~mode:`exact
-          | _ -> assert false
-        in
-        if l <> [] then
-        choose_symbol ~title:"Choose symbol" ~env:!start_env l
-      with Searchid.Error (s,e) ->
-        Entry.selection_clear ew;
-        Entry.selection_range ew ~start:(`Num s) ~stop:(`Num e);
-        Entry.xview_index ew ~index:(`Num s)
+      search_string ew ~mode:!search_which
     end
   and ok = Jg_button.create_destroyer tl ~parent:buttons ~text:"Cancel" in
 
@@ -303,6 +305,8 @@ let show_help () =
   let fw, tw, sb = Jg_text.create_with_scrollbar tl in
   let ok = Jg_button.create_destroyer ~parent:tl ~text:"Ok" tl in
   Text.insert tw ~index:tend ~text:Help.text;
+  Text.configure tw ~state:`Disabled;
+  Jg_bind.enter_focus tw;
   pack [tw] ~side:`Left ~fill:`Both ~expand:true;
   pack [sb] ~side:`Right ~fill:`Y;
   pack [fw] ~side:`Top ~expand:true ~fill:`Both;
@@ -336,24 +340,8 @@ let f ?(dir=Unix.getcwd()) ?on () =
 
   let ew = Entry.create tl in
   let buttons = Frame.create tl in
-  let search = Button.create buttons ~text:"Search" ~pady:1 ~command:
-    begin fun () ->
-      let s = Entry.get ew in
-      let mode = guess_search_mode s in
-      let l =
-        match mode with
-        | `Long -> search_string_symbol s
-        | `Pattern ->  search_pattern_symbol s
-        | `Type ->
-            try  search_string_type ~mode:`included s
-            with Searchid.Error (start,stop) ->
-              Entry.icursor ew ~index:(`Num start); []
-      in
-      match l with [] -> ()
-      | [lid,kind] when mode = `Long ->
-          view_symbol lid ~kind ~env:!start_env
-      | _ -> choose_symbol ~title:"Choose symbol" ~env:!start_env l
-    end
+  let search = Button.create buttons ~text:"Search" ~pady:1
+      ~command:(fun () -> search_string ew)
   and close =
     Button.create buttons ~text:"Close all" ~pady:1 ~command:close_all_views
   in
@@ -404,8 +392,11 @@ class st_viewer ?(dir=Unix.getcwd()) ?on () =
   let menus = Frame.create tl ~name:"menubar" in
   let filemenu = new Jg_menu.c "File" ~parent:menus
   and modmenu = new Jg_menu.c "Modules" ~parent:menus
+  and viewmenu = new Jg_menu.c "View" ~parent:menus
   and helpmenu = new Jg_menu.c "Help" ~parent:menus in
+  let search_frame = Frame.create tl in
   let boxes_frame = Frame.create tl ~name:"boxes" in
+  let label = Label.create tl ~anchor:`W ~padx:5 in
   let view = Frame.create tl in
   let buttons = Frame.create tl in
   let all = Button.create buttons ~text:"Show all" ~padx:20
@@ -415,15 +406,39 @@ class st_viewer ?(dir=Unix.getcwd()) ?on () =
   and intf = Button.create buttons ~text:"Intf" in
 object (self)
   val mutable boxes = []
+  val mutable show_all = fun () -> ()
 
   method create_box =
     let fmbox, mbox, sb = Jg_box.create_with_scrollbar boxes_frame in
+    bind mbox ~events:[`Modified([`Double], `ButtonPressDetail 1)]
+      ~action:(fun _ -> show_all ());
+    bind mbox ~events:[`Modified([`Double], `KeyPressDetail "Return")]
+      ~action:(fun _ -> show_all ());
     boxes <- boxes @ [fmbox, mbox];
     pack [sb] ~side:`Right ~fill:`Y;
     pack [mbox] ~side:`Left ~fill:`Both ~expand:true;
     pack [fmbox] ~side:`Left ~fill:`Both ~expand:true;
     fmbox, mbox
 
+  initializer
+    (* Search *)
+    let ew = Entry.create search_frame
+    and searchtype = Textvariable.create ~on:tl () in
+    bind ew ~events:[`KeyPressDetail "Return"] ~action:
+      (fun _ -> search_string ew ~mode:(Textvariable.get searchtype));
+    Jg_bind.enter_focus ew;
+    let search_button ?value text =
+      Radiobutton.create search_frame
+        ~text ~variable:searchtype ~value:text in
+    let symbol = search_button "Name"
+    and atype = search_button "Type" in
+    Radiobutton.select symbol;
+    pack [Label.create search_frame ~text:"Search"] ~side:`Left ~ipadx:5;
+    pack [ew] ~fill:`X ~expand:true ~side:`Left;
+    pack [Label.create search_frame ~text:"by"] ~side:`Left ~ipadx:5;
+    pack [symbol; atype] ~side:`Left;
+    pack [Label.create search_frame] ~side:`Right
+
   initializer
     (* Boxes *)
     let fmbox, mbox = self#create_box in
@@ -434,7 +449,8 @@ object (self)
     Setpath.add_update_hook (fun () -> reset_modules mbox; self#hide_after 1);
     List.iter [1;2] ~f:(fun _ -> ignore self#create_box);
     Searchpos.default_frame := Some
-      { mw_frame = view; mw_detach = detach; mw_edit = edit; mw_intf = intf };
+        { mw_frame = view; mw_title = Some label;
+          mw_detach = detach; mw_edit = edit; mw_intf = intf };
 
     (* Buttons *)
     pack [close] ~side:`Right ~fill:`X ~expand:true;
@@ -448,6 +464,20 @@ object (self)
     filemenu#add_command "Shell..." ~command:start_shell;
     filemenu#add_command "Quit" ~command:(fun () -> destroy tl);
 
+    (* View menu *)
+    viewmenu#add_command "Show all defs" ~command:(fun () -> show_all ());
+    let show_search = Textvariable.create ~on:tl () in
+    Textvariable.set show_search "1";
+    Menu.add_checkbutton viewmenu#menu ~label:"Search Entry"
+      ~variable:show_search ~indicatoron:true ~state:`Active
+      ~command:
+      begin fun () ->
+        let v = Textvariable.get show_search in
+        if v = "1" then begin
+          pack [search_frame] ~after:menus ~fill:`X
+        end else Pack.forget [search_frame]
+      end;
+
     (* modules menu *)
     modmenu#add_command "Path editor..."
       ~command:(fun () -> Setpath.set ~dir);
@@ -458,13 +488,14 @@ object (self)
     (* Help menu *)
     helpmenu#add_command "Manual..." ~command:show_help;
 
-    pack [filemenu#button; modmenu#button] ~side:`Left ~ipadx:5 ~anchor:`W;
+    pack [filemenu#button; viewmenu#button; modmenu#button]
+      ~side:`Left ~ipadx:5 ~anchor:`W;
     pack [helpmenu#button] ~side:`Right ~anchor:`E ~ipadx:5;
-    pack [menus] ~side:`Top ~fill:`X;      
-    (* pack [close; search] ~fill:`X ~side:`Right ~expand:true; *)
+    pack [menus] ~fill:`X;      
+    pack [search_frame] ~fill:`X;      
     pack [boxes_frame] ~fill:`Both ~expand:true;
-    pack [view] ~fill:`X ~expand:false;
-    pack [buttons] ~fill:`X ~side:`Bottom ~expand:false;
+    pack [buttons] ~fill:`X ~side:`Bottom;
+    pack [view] ~fill:`Both ~side:`Bottom ~expand:true;
     reset_modules mbox
 
   val mutable shown_paths = []
@@ -505,11 +536,7 @@ object (self)
       match path with None -> 1
       | Some path -> self#get_box ~path
     in
-
-    let l = Sort.list l ~order:
-        (fun (li1, _) (li2,_) ->
-          string_of_longident li1 < string_of_longident li2)
-    in
+    let l = List.sort l ~cmp:(fun (li1, _) (li2,_) -> compare li1 li2) in
     let nl = List.map l ~f:
         begin fun (li, k) ->
           string_of_longident li ^ " (" ^ string_of_kind k ^ ")"
@@ -517,10 +544,14 @@ object (self)
     let _, box = List.nth boxes n in
     Listbox.delete box ~first:(`Num 0) ~last:`End;
     Listbox.insert box ~index:`End ~texts:nl;
-    Jg_box.add_completion box ~double:false ~action:
-      begin fun index ->
-        let `Num pos = Listbox.index box ~index in
+
+    let current = ref None in
+    let display index =
+      let `Num pos = Listbox.index box ~index in
+      try
         let li, k = List.nth l pos in
+        self#hide_after (n+1);
+        if !current = Some (li,k) then () else
         let path =
           match path, li with
             None, Ldot (lip, _) ->
@@ -530,16 +561,22 @@ object (self)
               end
           | _ -> path
         in
-        view_symbol li ~kind:k ~env ?path;
-      end;
+        current := Some (li,k);
+        view_symbol li ~kind:k ~env ?path
+      with Failure "nth" -> ()
+    in
+    Jg_box.add_completion box ~double:false ~action:display;
+    bind box ~events:[`KeyRelease] ~fields:[`Char]
+      ~action:(fun ev -> display `Active);
+
     begin match signature with
       None -> ()
     | Some signature ->
-        Button.configure all ~command:
+        show_all <-
           begin fun () ->
+            current := None;
             view_signature signature ~title ~env ?path
-          end;
-        pack [all] ~side:`Right ~fill:`X ~expand:true
+          end
     end
 end
 
