@@ -9,12 +9,14 @@
 (*   Copyright 1999 Institut National de Recherche en Informatique et    *)
 (*   en Automatique and Kyoto University.  All rights reserved.          *)
 (*   This file is distributed under the terms of the GNU Library         *)
-(*   General Public License.                                             *)
+(*   General Public License, with the special exception on linking       *)
+(*   described in file ../../../LICENSE.                                 *)
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: compile.ml,v 1.22 2000/12/28 13:04:35 weis Exp $ *)
+(* $Id: compile.ml,v 1.26 2001/12/07 13:40:03 xleroy Exp $ *)
 
+open StdLabels
 open Tables
 
 (* CONFIGURE *)
@@ -152,7 +154,7 @@ let ppMLtype ?(any=false) ?(return=false) ?(def=false) ?(counter=ref 0) =
           with
             Not_found -> prerr_endline ("ppMLtype " ^ s ^ " ?"); s
           else if not def && List.length typdef.constructors > 1 then
-            "#" ^ s
+            "[< " ^ s ^ "]"
           else s
         else s
       with Not_found -> s
@@ -447,8 +449,9 @@ let rec converterCAMLtoTK ~context_widget argname ty =
        name ^ args
  |  Subtype (s, s') ->
        let name = "cCAMLtoTK" ^ s' ^ "_" ^ s ^ " " in
-       let args = if safetype then "(" ^ argname ^ " : #" ^ s' ^ "_" ^ s ^ ")"
-                  else argname
+       let args =
+         if safetype then "(" ^ argname ^ " : [< " ^ s' ^ "_" ^ s ^ "])"
+                     else argname
        in
        let args = 
            if requires_widget_context s then context_widget ^ " " ^ args
@@ -579,8 +582,8 @@ let write_CAMLtoTK ~w ~def:typdef ?safetype:(st = true) name =
         "dummy" in
     if st then begin
       w " : ";
-      if typdef.variant then w "#";
-      w name; w " -> tkArgs "
+      if typdef.variant then w ("[< " ^ name ^ "]") else w name;
+      w " -> tkArgs "
     end;
     w (" = function");
     List.iter constrs
