@@ -9,11 +9,15 @@
 (*   Copyright 1999 Institut National de Recherche en Informatique et    *)
 (*   en Automatique and Kyoto University.  All rights reserved.          *)
 (*   This file is distributed under the terms of the GNU Library         *)
-(*   General Public License.                                             *)
+(*   General Public License, with the special exception on linking       *)
+(*   described in file ../../../LICENSE.                                 *)
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: tables.ml,v 1.16 2000/12/28 13:04:47 weis Exp $ *)
+(* $Id: tables.ml,v 1.19 2001/12/07 13:40:04 xleroy Exp $ *)
+
+open StdLabels
+open Support
 
 (* Internal compiler errors *)
 
@@ -60,7 +64,7 @@ type fullcomponent = {
   }
 
 let sort_components =
-  Sort.list ~order:(fun c1 c2 ->  c1.ml_name < c2.ml_name)
+  List.sort ~cmp:(fun c1 c2 ->  compare c1.ml_name c2.ml_name)
 
 
 (* components are given either in full or abbreviated *)
@@ -153,7 +157,7 @@ let new_type typname arity =
                 subtypes = []; 
                 requires_widget_context = false;
                 variant = false} in
-    Hashtbl.add types_table ~key:typname ~data:typdef;
+    Hashtbl'.add types_table ~key:typname ~data:typdef;
     typdef
 
 
@@ -344,8 +348,8 @@ let enter_subtype typ arity subtyp constructors =
       in
        (* TODO: duplicate def in subtype are not checked *)
        typdef.subtypes <-
-          (subtyp , Sort.list real_constructors
-                      ~order:(fun c1 c2 -> c1.var_name <= c2.var_name)) ::
+          (subtyp , List.sort real_constructors
+             ~cmp:(fun c1 c2 -> compare c1.var_name c2.var_name)) ::
           typdef.subtypes
     end
 
@@ -391,7 +395,7 @@ let enter_widget name components =
       try List.assoc External sorted_components
       with Not_found -> []
   in
-  Hashtbl.add module_table ~key:name 
+  Hashtbl'.add module_table ~key:name 
     ~data:{module_type = Widget; commands = commands; externals = externals}
   
 (******************** Functions ********************)
@@ -418,6 +422,5 @@ let enter_module name components =
       try List.assoc External sorted_components
       with Not_found -> []
   in
-    Hashtbl.add module_table ~key:name 
+    Hashtbl'.add module_table ~key:name 
       ~data:{module_type = Family; commands = commands; externals = externals}
-
