@@ -8,15 +8,17 @@
 (*   Copyright 1999 Institut National de Recherche en Informatique et    *)
 (*   en Automatique and Kyoto University.  All rights reserved.          *)
 (*   This file is distributed under the terms of the GNU Library         *)
-(*   General Public License.                                             *)
+(*   General Public License, with the special exception on linking       *)
+(*   described in file ../../../LICENSE.                                 *)
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: tetris.ml,v 1.11 2000/12/28 13:04:52 weis Exp $ *)
+(* $Id: tetris.ml,v 1.13 2001/12/07 13:40:07 xleroy Exp $ *)
 
 (* A Tetris game for LablTk *)
 (* written by Jun P. Furuse *)
 
+open StdLabels
 open Tk
 
 exception Done
@@ -643,21 +645,21 @@ let _ =
         | "h" ->
             let m = copy_block current in
             m.x <- m.x - 1;
-            try_to_move m; ()
+            ignore (try_to_move m)
         | "j" ->
             let m = copy_block current in
             m.d <- m.d + 1;
             if m.d = List.length m.pattern then m.d <- 0;
-            try_to_move m; ()  
+            ignore (try_to_move m)
         | "k" ->
             let m = copy_block current in
             m.d <- m.d - 1;
             if m.d < 0 then m.d <- List.length m.pattern - 1;
-            try_to_move m; ()
+            ignore (try_to_move m)
         | "l" ->
             let m = copy_block current in
             m.x <- m.x + 1;
-            try_to_move m; ()
+            ignore (try_to_move m)
         | "m" ->
             remove_timer ();
             loop ()
