@@ -9,11 +9,12 @@
 /*   Copyright 1999 Institut National de Recherche en Informatique et    */
 /*   en Automatique and Kyoto University.  All rights reserved.          */
 /*   This file is distributed under the terms of the GNU Library         */
-/*   General Public License.                                             */
+/*   General Public License, with the special exception on linking       */
+/*   described in file ../../../LICENSE.                                 */
 /*                                                                       */
 /*************************************************************************/
 
-/* $Id: cltkEval.c,v 1.7 2001/07/02 12:24:12 xleroy Exp $ */
+/* $Id: cltkEval.c,v 1.9 2001/12/07 13:40:08 xleroy Exp $ */
 
 #include <stdlib.h>
 
@@ -55,7 +56,7 @@ value copy_string_list(int argc, char **argv)
  *   this version works on an arbitrary Tcl command,
  *   and does parsing and substitution
  */
-value camltk_tcl_eval(value str) /* ML */
+CAMLprim value camltk_tcl_eval(value str)
 {
   int code;
   char *cmd = NULL;
@@ -167,7 +168,7 @@ int fill_args (char **argv, int where, value v)
 }
 
 /* v is an array of TkArg */
-value camltk_tcl_direct_eval(value v) /* ML */
+CAMLprim value camltk_tcl_direct_eval(value v)
 {
   int i;
   int size;                     /* size of argv */
