@@ -9,11 +9,12 @@
 /*   Copyright 1999 Institut National de Recherche en Informatique et    */
 /*   en Automatique and Kyoto University.  All rights reserved.          */
 /*   This file is distributed under the terms of the GNU Library         */
-/*   General Public License.                                             */
+/*   General Public License, with the special exception on linking       */
+/*   described in file ../../../LICENSE.                                 */
 /*                                                                       */
 /*************************************************************************/
 
-/* $Id: cltkMain.c,v 1.7 2000/08/25 08:22:01 garrigue Exp $ */
+/* $Id: cltkMain.c,v 1.9 2001/12/07 13:40:09 xleroy Exp $ */
 
 #include <string.h>
 #include <tcl.h>
@@ -64,7 +65,7 @@ Tk_Window cltk_mainWindow;
 int cltk_slave_mode = 0;
 
 /* Initialisation, based on tkMain.c */
-value camltk_opentk(value display, value name) /* ML */
+CAMLprim value camltk_opentk(value display, value name)
 {
   if (!cltk_slave_mode) {
     /* Create an interpreter, dies if error */
