@@ -9,11 +9,12 @@
 /*   Copyright 1999 Institut National de Recherche en Informatique et    */
 /*   en Automatique and Kyoto University.  All rights reserved.          */
 /*   This file is distributed under the terms of the GNU Library         */
-/*   General Public License.                                             */
+/*   General Public License, with the special exception on linking       */
+/*   described in file ../../../LICENSE.                                 */
 /*                                                                       */
 /*************************************************************************/
 
-/* $Id: cltkMisc.c,v 1.6 2000/11/23 13:44:58 doligez Exp $ */
+/* $Id: cltkMisc.c,v 1.8 2001/12/07 13:40:09 xleroy Exp $ */
 
 #include <string.h>
 #include <tcl.h>
@@ -23,7 +24,7 @@
 #include "camltk.h"
 
 /* Parsing results */
-value camltk_splitlist (value v) /* ML */
+CAMLprim value camltk_splitlist (value v)
 {
   int argc;
   char **argv;
