@@ -9,13 +9,15 @@
 (*   Copyright 1999 Institut National de Recherche en Informatique et    *)
 (*   en Automatique and Kyoto University.  All rights reserved.          *)
 (*   This file is distributed under the terms of the GNU Library         *)
-(*   General Public License.                                             *)
+(*   General Public License, with the special exception on linking       *)
+(*   described in file ../../../LICENSE.                                 *)
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: fileevent.ml,v 1.9 2000/04/12 03:43:23 garrigue Exp $ *)
+(* $Id: fileevent.ml,v 1.11 2001/12/07 13:40:09 xleroy Exp $ *)
 
 open Unix
+open Support
 open Protocol
 
 external add_file_input : file_descr -> cbid -> unit
@@ -33,8 +35,8 @@ let fd_table = Hashtbl.create 37 (* Avoid space leak in callback table *)
 
 let add_fileinput ~fd ~callback:f =
   let id = new_function_id () in
-  Hashtbl.add callback_naming_table ~key:id ~data:(fun _ -> f());
-  Hashtbl.add fd_table ~key:(fd, 'r') ~data:id;
+  Hashtbl'.add callback_naming_table ~key:id ~data:(fun _ -> f());
+  Hashtbl'.add fd_table ~key:(fd, 'r') ~data:id;
   if !Protocol.debug then begin
     Protocol.prerr_cbid id; prerr_endline " for fileinput"
   end;
@@ -56,8 +58,8 @@ let remove_fileinput ~fd =
 
 let add_fileoutput ~fd ~callback:f =
   let id = new_function_id () in
-  Hashtbl.add callback_naming_table ~key:id ~data:(fun _ -> f());
-  Hashtbl.add fd_table ~key:(fd, 'w') ~data:id;
+  Hashtbl'.add callback_naming_table ~key:id ~data:(fun _ -> f());
+  Hashtbl'.add fd_table ~key:(fd, 'w') ~data:id;
   if !Protocol.debug then begin
     Protocol.prerr_cbid id; prerr_endline " for fileoutput"
   end;
