@@ -9,12 +9,15 @@
 (*   Copyright 1999 Institut National de Recherche en Informatique et    *)
 (*   en Automatique and Kyoto University.  All rights reserved.          *)
 (*   This file is distributed under the terms of the GNU Library         *)
-(*   General Public License.                                             *)
+(*   General Public License, with the special exception on linking       *)
+(*   described in file ../../../LICENSE.                                 *)
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: support.mli,v 1.5 1999/12/16 12:25:08 garrigue Exp $ *)
+(* $Id: support.mli,v 1.7 2001/12/07 13:40:10 xleroy Exp $ *)
 
 val split_str : pred:(char -> bool) -> string -> string list
 val may : ('a -> 'b) -> 'a option -> 'b option
 val maycons : ('a -> 'b) -> 'a option -> 'b list -> 'b list
+module Hashtbl' :
+  sig val add : ('a, 'b) Hashtbl.t -> key:'a -> data:'b -> unit end
