@@ -6,11 +6,12 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../../LICENSE.  *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: nat.mli,v 1.4 1999/11/17 18:57:47 xleroy Exp $ *)
+(* $Id: nat.mli,v 1.6 2001/12/07 13:40:16 xleroy Exp $ *)
 
 (* Module [Nat]: operations on natural numbers *)
 
@@ -26,7 +27,7 @@ external blit_nat: nat -> int -> nat -> int -> int -> unit = "blit_nat"
 val copy_nat: nat -> int -> int -> nat
 external set_digit_nat: nat -> int -> int -> unit = "set_digit_nat"
 external nth_digit_nat: nat -> int -> int = "nth_digit_nat"
-external length_nat: nat -> int = "%obj_size"
+val length_nat: nat -> int
 val length_nat : nat -> int 
 external num_digits_nat: nat -> int -> int -> int = "num_digits_nat"
 external num_leading_zero_bits_in_digit: nat -> int -> int = "num_leading_zero_bits_in_digit"
