@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1995 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: win32.c,v 1.30 2001/07/23 15:35:31 xleroy Exp $ */
+/* $Id: win32.c,v 1.33 2001/12/07 13:40:20 xleroy Exp $ */
 
 /* Thread interface for Win32 threads */
 
@@ -67,7 +68,7 @@ struct caml_thread_descr {
 
 struct caml_thread_struct {
   HANDLE wthread;               /* The Windows thread handle */
-  value descr;                  /* The heap-allocated descriptor */
+  value descr;                  /* The heap-allocated descriptor (root) */
   struct caml_thread_struct * next;  /* Double linking of running threads */
   struct caml_thread_struct * prev;
 #ifdef NATIVE_CODE
@@ -86,6 +87,7 @@ struct caml_thread_struct {
   struct longjmp_buffer * external_raise; /* Saved external_raise */
   int backtrace_pos;            /* Saved backtrace_pos */
   code_t * backtrace_buffer;    /* Saved backtrace_buffer */
+  value backtrace_last_exn;     /* Saved backtrace_last_exn (root) */
 #endif
 };
 
@@ -93,15 +95,18 @@ typedef struct caml_thread_struct * caml_thread_t;
 
 /* The descriptor for the currently executing thread (thread-specific) */
 
-static __declspec( thread ) caml_thread_t curr_thread = NULL;
+static caml_thread_t curr_thread = NULL;
 
 /* The global mutex used to ensure that at most one thread is running
    Caml code */
 static HANDLE caml_mutex;
 
-/* The thread-specific variable holding last locked I/O channel */
+/* The key used for storing the thread descriptor in the specific data
+   of the corresponding Posix thread. */
+static DWORD thread_descriptor_key;
 
-static __declspec( thread ) struct channel * last_channel_locked = NULL;
+/* The key used for unlocking I/O channels on exceptions */
+static DWORD last_channel_locked_key;
 
 /* Identifier for next thread creation */
 static long thread_next_ident = 0;
@@ -121,6 +126,9 @@ static void caml_thread_scan_roots(scanning_action action)
   th = curr_thread;
   do {
     (*action)(th->descr, &th->descr);
+#ifndef NATIVE_CODE
+    (*action)(th->backtrace_last_exn, &th->backtrace_last_exn);
+#endif
     /* Don't rescan the stack of the current thread, it was done already */
     if (th != curr_thread) {
 #ifdef NATIVE_CODE
@@ -164,6 +172,7 @@ static void caml_thread_enter_blocking_section(void)
   curr_thread->external_raise = external_raise;
   curr_thread->backtrace_pos = backtrace_pos;
   curr_thread->backtrace_buffer = backtrace_buffer;
+  curr_thread->backtrace_last_exn = backtrace_last_exn;
 #endif
   /* Release the global mutex */
   ReleaseMutex(caml_mutex);
@@ -173,6 +182,9 @@ static void caml_thread_leave_blocking_section(void)
 {
   /* Re-acquire the global mutex */
   WaitForSingleObject(caml_mutex, INFINITE);
+  /* Update curr_thread to point to the thread descriptor corresponding
+     to the thread currently executing */
+  curr_thread = TlsGetValue(thread_descriptor_key);
   /* Restore the stack-related global variables */
 #ifdef NATIVE_CODE
   caml_bottom_of_stack= curr_thread->bottom_of_stack;
@@ -190,6 +202,7 @@ static void caml_thread_leave_blocking_section(void)
   external_raise = curr_thread->external_raise;
   backtrace_pos = curr_thread->backtrace_pos;
   backtrace_buffer = curr_thread->backtrace_buffer;
+  backtrace_last_exn = curr_thread->backtrace_last_exn;
 #endif
   if (prev_leave_blocking_section_hook != NULL)
     (*prev_leave_blocking_section_hook)();
@@ -219,19 +232,20 @@ static void caml_io_mutex_lock(struct channel * chan)
      unlock the mutex.  The alternative (doing the setspecific
      before locking the mutex is also incorrect, since we could
      then unlock a mutex that is unlocked or locked by someone else. */
-  last_channel_locked = chan;
+  TlsSetValue(last_channel_locked_key, (void *) chan);
   leave_blocking_section();
 }
 
 static void caml_io_mutex_unlock(struct channel * chan)
 {
   ReleaseMutex((HANDLE) chan->mutex);
-  last_channel_locked = NULL;
+  TlsSetValue(last_channel_locked_key, NULL);
 }
 
 static void caml_io_mutex_unlock_exn(void)
 {
-  if (last_channel_locked != NULL) caml_io_mutex_unlock(last_channel_locked);
+  struct channel * chan = TlsGetValue(last_channel_locked_key);
+  if (chan != NULL) caml_io_mutex_unlock(chan);
 }
 
 /* The "tick" thread fakes a signal at regular intervals. */
@@ -256,7 +270,7 @@ static void caml_thread_finalize(value vthread)
 
 /* Initialize the thread machinery */
 
-value caml_thread_initialize(value unit)   /* ML */
+CAMLprim caml_thread_initialize(value unit)
 {
   value vthread = Val_unit;
   value descr;
@@ -267,6 +281,9 @@ value caml_thread_initialize(value unit)   /* ML */
     /* Initialize the main mutex and acquire it */
     caml_mutex = CreateMutex(NULL, TRUE, NULL);
     if (caml_mutex == NULL) caml_wthread_error("Thread.init");
+    /* Initialize the TLS keys */
+    thread_descriptor_key = TlsAlloc();
+    last_channel_locked_key = TlsAlloc();
     /* Create a finalized value to hold thread handle */
     vthread = alloc_final(sizeof(struct caml_thread_handle) / sizeof(value),
                           caml_thread_finalize, 1, 1000);
@@ -290,6 +307,8 @@ value caml_thread_initialize(value unit)   /* ML */
     curr_thread->prev = curr_thread;
     /* The stack-related fields will be filled in at the next
        enter_blocking_section */
+    /* Associate the thread descriptor with the thread */
+    TlsSetValue(thread_descriptor_key, (void *) curr_thread);
     /* Set up the hooks */
     prev_scan_roots_hook = scan_roots_hook;
     scan_roots_hook = caml_thread_scan_roots;
@@ -321,14 +340,14 @@ static void caml_thread_start(caml_thread_t th)
 {
   value clos;
 
-  /* Initialize the per-thread variables */
-  curr_thread = th;
-  last_channel_locked = NULL;
+  /* Associate the thread descriptor with the thread */
+  TlsSetValue(thread_descriptor_key, (void *) th);
+  TlsSetValue(last_channel_locked_key, NULL);
   /* Acquire the global mutex and set up the stack variables */
   leave_blocking_section();
   /* Callback the closure */
   clos = Start_closure(th->descr);
-  Modify(&(Start_closure(th->descr)), Val_unit);
+  modify(&(Start_closure(th->descr)), Val_unit);
   callback_exn(clos, Val_unit);
   /* Remove th from the doubly-linked list of threads */
   th->next->prev = th->prev;
@@ -346,7 +365,7 @@ static void caml_thread_start(caml_thread_t th)
   /* The thread now stops running */
 }
 
-value caml_thread_new(value clos)          /* ML */
+CAMLprim caml_thread_new(value clos)
 {
   caml_thread_t th;
   value vthread = Val_unit;
@@ -382,6 +401,7 @@ value caml_thread_new(value clos)          /* ML */
     th->external_raise = NULL;
     th->backtrace_pos = 0;
     th->backtrace_buffer = NULL;
+    th->backtrace_last_exn = Val_unit;
 #endif
     /* Add thread info block to the list of threads */
     th->next = curr_thread->next;
@@ -413,7 +433,7 @@ value caml_thread_new(value clos)          /* ML */
 
 /* Return the current thread */
 
-value caml_thread_self(value unit)         /* ML */
+CAMLprim caml_thread_self(value unit)
 {
   if (curr_thread == NULL) invalid_argument("Thread.self: not initialized");
   return curr_thread->descr;
@@ -421,14 +441,14 @@ value caml_thread_self(value unit)         /* ML */
 
 /* Return the identifier of a thread */
 
-value caml_thread_id(value th)          /* ML */
+CAMLprim caml_thread_id(value th)
 {
   return Ident(th);
 }
 
 /* Print uncaught exception and backtrace */
 
-value caml_thread_uncaught_exception(value exn)  /* ML */
+CAMLprim caml_thread_uncaught_exception(value exn)
 {
   char * msg = format_caml_exception(exn);
   fprintf(stderr, "Thread %d killed on uncaught exception %s\n",
@@ -443,7 +463,7 @@ value caml_thread_uncaught_exception(value exn)  /* ML */
 
 /* Allow re-scheduling */
 
-value caml_thread_yield(value unit)        /* ML */
+CAMLprim caml_thread_yield(value unit)
 {
   enter_blocking_section();
   Sleep(0);
@@ -453,7 +473,7 @@ value caml_thread_yield(value unit)        /* ML */
 
 /* Suspend the current thread until another thread terminates */
 
-value caml_thread_join(value th)          /* ML */
+CAMLprim caml_thread_join(value th)
 {
   HANDLE h;
   Begin_root(th)                /* prevent deallocation of handle */
@@ -491,7 +511,7 @@ static struct custom_operations caml_mutex_ops = {
   custom_deserialize_default
 };
 
-value caml_mutex_new(value unit)        /* ML */
+CAMLprim caml_mutex_new(value unit)
 {
   value mut;
   mut = alloc_custom(&caml_mutex_ops, sizeof(HANDLE), 1, Max_mutex_number);
@@ -500,7 +520,7 @@ value caml_mutex_new(value unit)        /* ML */
   return mut;
 }
 
-value caml_mutex_lock(value mut)           /* ML */
+CAMLprim caml_mutex_lock(value mut)
 {
   int retcode;
   Begin_root(mut)               /* prevent deallocation of mutex */
@@ -512,7 +532,7 @@ value caml_mutex_lock(value mut)           /* ML */
   return Val_unit;
 }
 
-value caml_mutex_unlock(value mut)           /* ML */
+CAMLprim caml_mutex_unlock(value mut)
 {
   BOOL retcode;
   Begin_root(mut)               /* prevent deallocation of mutex */
@@ -524,7 +544,7 @@ value caml_mutex_unlock(value mut)           /* ML */
   return Val_unit;
 }
 
-value caml_mutex_try_lock(value mut)           /* ML */
+CAMLprim caml_mutex_try_lock(value mut)
 {
   int retcode;
   retcode = WaitForSingleObject(Mutex_val(mut), 0);
@@ -535,7 +555,7 @@ value caml_mutex_try_lock(value mut)           /* ML */
 
 /* Delay */
 
-value caml_thread_delay(value val)        /* ML */
+CAMLprim caml_thread_delay(value val)
 {
   enter_blocking_section();
   Sleep((DWORD)(Double_val(val)*1000)); /* milliseconds */
@@ -574,7 +594,7 @@ static struct custom_operations caml_condition_ops = {
   custom_deserialize_default
 };
 
-value caml_condition_new(value unit)        /* ML */
+CAMLprim caml_condition_new(value unit)
 {
   value cond;
   cond = alloc_custom(&caml_condition_ops, sizeof(struct caml_condvar),
@@ -586,7 +606,7 @@ value caml_condition_new(value unit)        /* ML */
   return cond;
 }
 
-value caml_condition_wait(value cond, value mut)           /* ML */
+CAMLprim caml_condition_wait(value cond, value mut)
 {
   int retcode;
   HANDLE m = Mutex_val(mut);
@@ -609,7 +629,7 @@ value caml_condition_wait(value cond, value mut)           /* ML */
   return Val_unit;
 }
 
-value caml_condition_signal(value cond)           /* ML */
+CAMLprim caml_condition_signal(value cond)
 {
   HANDLE s = Condition_val(cond)->sem;
 
@@ -625,7 +645,7 @@ value caml_condition_signal(value cond)           /* ML */
   return Val_unit;
 }
 
-value caml_condition_broadcast(value cond)           /* ML */
+CAMLprim caml_condition_broadcast(value cond)
 {
   HANDLE s = Condition_val(cond)->sem;
   unsigned long c = Condition_val(cond)->count;
@@ -655,7 +675,7 @@ static void caml_wait_signal_handler(int signo)
 
 typedef void (*sighandler_type)(int);
 
-value caml_wait_signal(value sigs)
+CAMLprim value caml_wait_signal(value sigs)
 {
   HANDLE event;
   int res, s, retcode;
