@@ -6,11 +6,12 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../../LICENSE.  *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pervasives.ml,v 1.28 2001/02/09 09:40:12 xleroy Exp $ *)
+(* $Id: pervasives.ml,v 1.34 2001/12/07 13:40:22 xleroy Exp $ *)
 
 (* Same as ../../stdlib/pervasives.ml, except that I/O functions have
    been redefined to not block the whole process, but only the calling
@@ -121,6 +122,15 @@ let neg_infinity =
 let nan =
   float_of_bytes "\127\240\000\000\000\000\000\001"
               (* 0x7F F0 00 00 00 00 00 01 *)
+let max_float =
+  float_of_bytes "\127\239\255\255\255\255\255\255"
+              (* 0x7f ef ff ff ff ff ff ff *)
+let min_float =
+  float_of_bytes "\000\016\000\000\000\000\000\000"
+              (* 0x00 10 00 00 00 00 00 00 *)
+let epsilon_float =
+  float_of_bytes "\060\176\000\000\000\000\000\000"
+              (* 0x3c b0 00 00 00 00 00 00 *)
 type fpclass =
     FP_normal
   | FP_subnormal
@@ -178,7 +188,7 @@ let string_of_bool b =
 let bool_of_string = function
   | "true" -> true
   | "false" -> false
-  | _ -> invalid_arg "string_of_bool"
+  | _ -> invalid_arg "bool_of_string"
 
 let string_of_int n =
   format_int "%d" n
@@ -202,8 +212,8 @@ let rec (@) l1 l2 =
 type in_channel
 type out_channel
 
-external open_descriptor_out: int -> out_channel = "caml_open_descriptor"
-external open_descriptor_in: int -> in_channel = "caml_open_descriptor"
+external open_descriptor_out: int -> out_channel = "caml_open_descriptor_out"
+external open_descriptor_in: int -> in_channel = "caml_open_descriptor_in"
 
 let stdin = open_descriptor_in 0
 let stdout = open_descriptor_out 1
@@ -256,6 +266,21 @@ let rec flush oc =
       wait_outchan oc (-1); false in
   if success then () else flush oc
 
+external out_channels_list : unit -> out_channel list 
+                           = "caml_out_channels_list"
+
+let flush_all () = 
+  let rec iter = function
+      [] -> ()
+    | a::l ->
+        begin try
+            flush a
+        with Sys_error _ ->
+          () (* ignore channels closed during a preceding flush. *)
+        end;
+        iter l
+  in iter (out_channels_list ())
+
 external unsafe_output_partial : out_channel -> string -> int -> int -> int
                         = "caml_output_partial"
 
@@ -287,6 +312,8 @@ let output oc s ofs len =
   then invalid_arg "output"
   else unsafe_output oc s ofs len
 
+let output' oc ~buf ~pos ~len = output oc buf pos len
+
 let rec output_byte oc b =
   try
     output_byte_blocking oc b
@@ -350,6 +377,8 @@ let input ic s ofs len =
   then invalid_arg "input"
   else unsafe_input ic s ofs len
 
+let input' ic ~buf ~pos ~len = input ic buf pos len
+
 let rec unsafe_really_input ic s ofs len =
   if len <= 0 then () else begin
     let r = unsafe_input ic s ofs len in
@@ -447,7 +476,7 @@ let read_float () = float_of_string(read_line())
 
 external sys_exit : int -> 'a = "sys_exit"
 
-let exit_function = ref (fun () -> flush stdout; flush stderr)
+let exit_function = ref flush_all
 
 let at_exit f =
   let g = !exit_function in
@@ -458,3 +487,7 @@ let do_at_exit () = (!exit_function) ()
 let exit retcode =
   do_at_exit ();
   sys_exit retcode
+
+external register_named_value: string -> 'a -> unit = "register_named_value"
+
+let _ = register_named_value "Pervasives.do_at_exit" do_at_exit
