@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: accept.c,v 1.10 2000/03/07 09:07:38 xleroy Exp $ */
+/* $Id: accept.c,v 1.12 2001/12/07 13:40:24 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
@@ -22,7 +23,7 @@
 
 #include "socketaddr.h"
 
-value unix_accept(value sock)          /* ML */
+CAMLprim value unix_accept(value sock)
 {
   int retcode;
   value res;
@@ -46,6 +47,6 @@ value unix_accept(value sock)          /* ML */
 
 #else
 
-value unix_accept(value sock) { invalid_argument("accept not implemented"); }
+CAMLprim value unix_accept(value sock) { invalid_argument("accept not implemented"); }
   
 #endif
