@@ -6,16 +6,17 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: chroot.c,v 1.6 1999/11/17 18:57:57 xleroy Exp $ */
+/* $Id: chroot.c,v 1.8 2001/12/07 13:40:26 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 
-value unix_chroot(value path)           /* ML */
+CAMLprim value unix_chroot(value path)
 {
   int ret;
   ret = chroot(String_val(path));
