@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: connect.c,v 1.9 2000/03/07 09:07:38 xleroy Exp $ */
+/* $Id: connect.c,v 1.11 2001/12/07 13:40:26 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <signals.h>
@@ -20,7 +21,7 @@
 
 #include "socketaddr.h"
 
-value unix_connect(value socket, value address)   /* ML */
+CAMLprim value unix_connect(value socket, value address)
 {
   int retcode;
   union sock_addr_union addr;
@@ -36,7 +37,7 @@ value unix_connect(value socket, value address)   /* ML */
 
 #else
 
-value unix_connect(value socket, value address)
+CAMLprim value unix_connect(value socket, value address)
 { invalid_argument("connect not implemented"); }
   
 #endif
