@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: errmsg.c,v 1.9 2000/04/05 18:30:19 doligez Exp $ */
+/* $Id: errmsg.c,v 1.11 2001/12/07 13:40:27 xleroy Exp $ */
 
 #include <errno.h>
 #include <mlvalues.h>
@@ -22,7 +23,7 @@ extern int error_table[];
 
 extern char * strerror(int);
 
-value unix_error_message(value err)   /* ML */
+CAMLprim value unix_error_message(value err)
 {
   int errnum;
   errnum = Is_block(err) ? Int_val(Field(err, 0)) : error_table[Int_val(err)];
@@ -34,7 +35,7 @@ value unix_error_message(value err)   /* ML */
 extern int sys_nerr;
 extern char *sys_errlist[];
 
-value unix_error_message(value err)
+CAMLprim value unix_error_message(value err)
 {
   int errnum;
   errnum = Is_block(err) ? Int_val(Field(err, 0)) : error_table[Int_val(err)];
