@@ -6,18 +6,19 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: ftruncate.c,v 1.6 1999/11/17 18:58:00 xleroy Exp $ */
+/* $Id: ftruncate.c,v 1.8 2001/12/07 13:40:28 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 
 #ifdef HAS_TRUNCATE
 
-value unix_ftruncate(value fd, value len)    /* ML */
+CAMLprim value unix_ftruncate(value fd, value len)
 {
   if (ftruncate(Int_val(fd), Long_val(len)) == -1)
     uerror("ftruncate", Nothing);
@@ -26,7 +27,7 @@ value unix_ftruncate(value fd, value len)    /* ML */
 
 #else
 
-value unix_ftruncate(value fd, value len)
+CAMLprim value unix_ftruncate(value fd, value len)
 { invalid_argument("ftruncate not implemented"); }
 
 #endif
