@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: getgr.c,v 1.10 2001/07/02 12:24:13 xleroy Exp $ */
+/* $Id: getgr.c,v 1.12 2001/12/07 13:40:29 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <fail.h>
@@ -38,7 +39,7 @@ static value alloc_group_entry(struct group *entry)
   return res;
 }
 
-value unix_getgrnam(value name)        /* ML */
+CAMLprim value unix_getgrnam(value name)
 {
   struct group * entry;
   entry = getgrnam(String_val(name));
@@ -46,7 +47,7 @@ value unix_getgrnam(value name)        /* ML */
   return alloc_group_entry(entry);
 }
 
-value unix_getgrgid(value gid)         /* ML */
+CAMLprim value unix_getgrgid(value gid)
 {
   struct group * entry;
   entry = getgrgid(Int_val(gid));
