@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: gethost.c,v 1.18 2001/07/02 12:24:13 xleroy Exp $ */
+/* $Id: gethost.c,v 1.20 2001/12/07 13:40:29 xleroy Exp $ */
 
 #include <string.h>
 #include <mlvalues.h>
@@ -64,7 +65,7 @@ static value alloc_host_entry(struct hostent *entry)
   return res;
 }
 
-value unix_gethostbyaddr(value a)   /* ML */
+CAMLprim value unix_gethostbyaddr(value a)
 {
   uint32 adr;
   struct hostent * entry;
@@ -76,7 +77,7 @@ value unix_gethostbyaddr(value a)   /* ML */
   return alloc_host_entry(entry);
 }
 
-value unix_gethostbyname(value name)   /* ML */
+CAMLprim value unix_gethostbyname(value name)
 {
   char hostname[256];
   struct hostent * entry;
@@ -91,10 +92,10 @@ value unix_gethostbyname(value name)   /* ML */
 
 #else
 
-value unix_gethostbyaddr(value name)
+CAMLprim value unix_gethostbyaddr(value name)
 { invalid_argument("gethostbyaddr not implemented"); }
   
-value unix_gethostbyname(value name)
+CAMLprim value unix_gethostbyname(value name)
 { invalid_argument("gethostbyname not implemented"); }
  
 #endif
