@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: gethostname.c,v 1.8 2000/04/11 13:05:01 xleroy Exp $ */
+/* $Id: gethostname.c,v 1.10 2001/12/07 13:40:29 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
@@ -27,7 +28,7 @@
 #define MAXHOSTNAMELEN 256
 #endif
 
-value unix_gethostname(value unit)         /* ML */
+CAMLprim value unix_gethostname(value unit)
 {
   char name[MAXHOSTNAMELEN];
   gethostname(name, MAXHOSTNAMELEN);
@@ -40,7 +41,7 @@ value unix_gethostname(value unit)         /* ML */
 
 #include <sys/utsname.h>
 
-value unix_gethostname(value unit)
+CAMLprim value unix_gethostname(value unit)
 {
   struct utsname un;
   uname(&un);
@@ -49,7 +50,7 @@ value unix_gethostname(value unit)
 
 #else
 
-value unix_gethostname(value unit)
+CAMLprim value unix_gethostname(value unit)
 { invalid_argument("gethostname not implemented"); }
 
 #endif
