@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: getlogin.c,v 1.6 1999/11/17 18:58:01 xleroy Exp $ */
+/* $Id: getlogin.c,v 1.8 2001/12/07 13:40:29 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
@@ -19,7 +20,7 @@
 
 extern char * getlogin(void);
 
-value unix_getlogin(void)            /* ML */
+CAMLprim value unix_getlogin(void)
 {
   char * name;
   name = getlogin();
