@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: getpeername.c,v 1.7 2000/03/07 09:07:38 xleroy Exp $ */
+/* $Id: getpeername.c,v 1.9 2001/12/07 13:40:29 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
@@ -19,7 +20,7 @@
 
 #include "socketaddr.h"
 
-value unix_getpeername(value sock)          /* ML */
+CAMLprim value unix_getpeername(value sock)
 {
   int retcode;
   union sock_addr_union addr;
@@ -33,7 +34,7 @@ value unix_getpeername(value sock)          /* ML */
 
 #else
 
-value unix_getpeername(value sock)
+CAMLprim value unix_getpeername(value sock)
 { invalid_argument("getpeername not implemented"); }
   
 #endif
