@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: gmtime.c,v 1.14 2001/02/22 08:59:19 xleroy Exp $ */
+/* $Id: gmtime.c,v 1.16 2001/12/07 13:40:31 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
@@ -35,7 +36,7 @@ static value alloc_tm(struct tm *tm)
   return res;
 }
 
-value unix_gmtime(value t)             /* ML */
+CAMLprim value unix_gmtime(value t)
 {
   time_t clock;
   struct tm * tm;
@@ -45,7 +46,7 @@ value unix_gmtime(value t)             /* ML */
   return alloc_tm(tm);
 }
 
-value unix_localtime(value t)          /* ML */
+CAMLprim value unix_localtime(value t)
 {
   time_t clock;
   struct tm * tm;
@@ -57,7 +58,7 @@ value unix_localtime(value t)          /* ML */
 
 #ifdef HAS_MKTIME
 
-value unix_mktime(value t)            /* ML */
+CAMLprim value unix_mktime(value t)
 {
   struct tm tm;
   time_t clock;
@@ -87,6 +88,6 @@ value unix_mktime(value t)            /* ML */
 
 #else
 
-value unix_mktime(value t) { invalid_argument("mktime not implemented"); }
+CAMLprim value unix_mktime(value t) { invalid_argument("mktime not implemented"); }
 
 #endif
