@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: lseek.c,v 1.7 1999/11/17 18:58:03 xleroy Exp $ */
+/* $Id: lseek.c,v 1.9 2001/12/07 13:40:31 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
@@ -27,7 +28,7 @@ static int seek_command_table[] = {
   SEEK_SET, SEEK_CUR, SEEK_END
 };
 
-value unix_lseek(value fd, value ofs, value cmd)   /* ML */
+CAMLprim value unix_lseek(value fd, value ofs, value cmd)
 {
   long ret;
   ret = lseek(Int_val(fd), Long_val(ofs),
