@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: open.c,v 1.7 1999/11/17 18:58:04 xleroy Exp $ */
+/* $Id: open.c,v 1.9 2001/12/07 13:40:32 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
@@ -25,7 +26,7 @@ static int open_flag_table[] = {
   O_RDONLY, O_WRONLY, O_RDWR, O_NONBLOCK, O_APPEND, O_CREAT, O_TRUNC, O_EXCL
 };
 
-value unix_open(value path, value flags, value perm) /* ML */
+CAMLprim value unix_open(value path, value flags, value perm)
 {
   int ret;
 
