@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: opendir.c,v 1.6 1999/11/17 18:58:04 xleroy Exp $ */
+/* $Id: opendir.c,v 1.8 2001/12/07 13:40:32 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
@@ -21,7 +22,7 @@
 #include <sys/dir.h>
 #endif
 
-value unix_opendir(value path)         /* ML */
+CAMLprim value unix_opendir(value path)
 {
   DIR * d;
   d = opendir(String_val(path));
