@@ -6,17 +6,18 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: pipe.c,v 1.7 1999/11/17 18:58:04 xleroy Exp $ */
+/* $Id: pipe.c,v 1.9 2001/12/07 13:40:32 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
 #include "unixsupport.h"
 
-value unix_pipe(void)                /* ML */
+CAMLprim value unix_pipe(void)
 {
   int fd[2];
   value res;
