@@ -6,11 +6,16 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: readdir.c,v 1.7 1999/11/17 18:58:04 xleroy Exp $ */
+/* $Id: readdir.c,v 1.10 2001/12/07 13:40:32 xleroy Exp $ */
+
+/* This avoids "overflow" errors when reading from very large directories
+   (PR#609) */
+#define _FILE_OFFSET_BITS 64
 
 #include <mlvalues.h>
 #include <fail.h>
@@ -25,7 +30,7 @@ typedef struct dirent directory_entry;
 typedef struct direct directory_entry;
 #endif
 
-value unix_readdir(value d)            /* ML */
+CAMLprim value unix_readdir(value d)
 {
   directory_entry * e;
 
