@@ -6,16 +6,17 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: rmdir.c,v 1.7 1999/11/17 18:58:05 xleroy Exp $ */
+/* $Id: rmdir.c,v 1.9 2001/12/07 13:40:33 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 
-value unix_rmdir(value path)           /* ML */
+CAMLprim value unix_rmdir(value path)
 {
   if (rmdir(String_val(path)) == -1) uerror("rmdir", path);
   return Val_unit;
