@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: select.c,v 1.15 1999/12/17 12:42:44 xleroy Exp $ */
+/* $Id: select.c,v 1.17 2001/12/07 13:40:33 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
@@ -59,7 +60,7 @@ static value fdset_to_fdlist(file_descr_set *fdset)
   return res;
 }
 
-value unix_select(value readfds, value writefds, value exceptfds, value timeout) /* ML */
+CAMLprim value unix_select(value readfds, value writefds, value exceptfds, value timeout)
 {
   file_descr_set read, write, except;
   double tm;
@@ -98,7 +99,7 @@ value unix_select(value readfds, value writefds, value exceptfds, value timeout)
 
 #else
 
-value unix_select(value readfds, value writefds, value exceptfds, value timeout)
+CAMLprim value unix_select(value readfds, value writefds, value exceptfds, value timeout)
 { invalid_argument("select not implemented"); }
 
 #endif
