@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1997 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: setsid.c,v 1.3 1999/11/17 18:58:06 xleroy Exp $ */
+/* $Id: setsid.c,v 1.5 2001/12/07 13:40:33 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
@@ -18,7 +19,7 @@
 #include <unistd.h>
 #endif
 
-value unix_setsid(value unit)              /* ML */
+CAMLprim value unix_setsid(value unit)
 {
 #ifdef HAS_SETSID
   return Val_int(setsid());
