@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: shutdown.c,v 1.8 1999/11/17 18:58:06 xleroy Exp $ */
+/* $Id: shutdown.c,v 1.10 2001/12/07 13:40:35 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
@@ -23,7 +24,7 @@ static int shutdown_command_table[] = {
   0, 1, 2
 };
 
-value unix_shutdown(value sock, value cmd)   /* ML */
+CAMLprim value unix_shutdown(value sock, value cmd)
 {
   if (shutdown(Int_val(sock), shutdown_command_table[Int_val(cmd)]) == -1)
     uerror("shutdown", Nothing);
@@ -32,7 +33,7 @@ value unix_shutdown(value sock, value cmd)   /* ML */
 
 #else
 
-value unix_shutdown(value sock, value cmd)
+CAMLprim value unix_shutdown(value sock, value cmd)
 { invalid_argument("shutdown not implemented"); }
 
 #endif
