@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: stat.c,v 1.9 1999/11/17 18:58:08 xleroy Exp $ */
+/* $Id: stat.c,v 1.11 2001/12/07 13:40:36 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <memory.h>
@@ -64,7 +65,7 @@ static value stat_aux(struct stat *buf)
   return v;
 }
 
-value unix_stat(value path)             /* ML */
+CAMLprim value unix_stat(value path)
 {
   int ret;
   struct stat buf;
@@ -73,7 +74,7 @@ value unix_stat(value path)             /* ML */
   return stat_aux(&buf);
 }
 
-value unix_lstat(value path)             /* ML */
+CAMLprim value unix_lstat(value path)
 {
   int ret;
   struct stat buf;
@@ -86,7 +87,7 @@ value unix_lstat(value path)             /* ML */
   return stat_aux(&buf);
 }
 
-value unix_fstat(value fd)             /* ML */
+CAMLprim value unix_fstat(value fd)
 {
   int ret;
   struct stat buf;
