@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: strofaddr.c,v 1.6 1999/11/17 18:58:08 xleroy Exp $ */
+/* $Id: strofaddr.c,v 1.8 2001/12/07 13:40:36 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
@@ -20,7 +21,7 @@
 
 #include "socketaddr.h"
 
-value unix_string_of_inet_addr(value a) /* ML */
+CAMLprim value unix_string_of_inet_addr(value a)
 {
   struct in_addr address;
   address.s_addr = GET_INET_ADDR(a);
@@ -29,7 +30,7 @@ value unix_string_of_inet_addr(value a) /* ML */
 
 #else
 
-value unix_string_of_inet_addr(value a)
+CAMLprim value unix_string_of_inet_addr(value a)
 { invalid_argument("string_of_inet_addr not implemented"); }
 
 #endif
