@@ -6,18 +6,19 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: symlink.c,v 1.6 1999/11/17 18:58:08 xleroy Exp $ */
+/* $Id: symlink.c,v 1.8 2001/12/07 13:40:36 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 
 #ifdef HAS_SYMLINK
 
-value unix_symlink(value path1, value path2) /* ML */
+CAMLprim value unix_symlink(value path1, value path2)
 {
   if (symlink(String_val(path1), String_val(path2)) == -1)
     uerror("symlink", path2);
@@ -26,7 +27,7 @@ value unix_symlink(value path1, value path2) /* ML */
 
 #else
 
-value unix_symlink(value path1, value path2)
+CAMLprim value unix_symlink(value path1, value path2)
 { invalid_argument("symlink not implemented"); }
 
 #endif
