@@ -6,18 +6,19 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: truncate.c,v 1.6 1999/11/17 18:58:09 xleroy Exp $ */
+/* $Id: truncate.c,v 1.8 2001/12/07 13:40:37 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 
 #ifdef HAS_TRUNCATE
 
-value unix_truncate(value path, value len)   /* ML */
+CAMLprim value unix_truncate(value path, value len)
 {
   if (truncate(String_val(path), Long_val(len)) == -1)
     uerror("truncate", path);
@@ -26,7 +27,7 @@ value unix_truncate(value path, value len)   /* ML */
 
 #else
 
-value unix_truncate(value path, value len)
+CAMLprim value unix_truncate(value path, value len)
 { invalid_argument("truncate not implemented"); }
 
 #endif
