@@ -6,11 +6,12 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../../LICENSE.  *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: unix.ml,v 1.49 2001/07/26 14:10:09 xleroy Exp $ *)
+(* $Id: unix.ml,v 1.51 2001/12/07 13:40:37 xleroy Exp $ *)
 
 type error =
     E2BIG
@@ -166,9 +167,9 @@ let write fd buf ofs len =
   else unsafe_write fd buf ofs len
 
 external in_channel_of_descr : file_descr -> in_channel
-                             = "caml_open_descriptor"
+                             = "caml_open_descriptor_in"
 external out_channel_of_descr : file_descr -> out_channel
-                              = "caml_open_descriptor"
+                              = "caml_open_descriptor_out"
 external descr_of_in_channel : in_channel -> file_descr = "channel_descriptor"
 external descr_of_out_channel : out_channel -> file_descr
                               = "channel_descriptor"
