@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: accept.c,v 1.12 2000/03/16 13:35:19 xleroy Exp $ */
+/* $Id: accept.c,v 1.15 2001/12/07 13:40:43 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
@@ -18,7 +19,7 @@
 #include "unixsupport.h"
 #include "socketaddr.h"
 
-value unix_accept(sock)          /* ML */
+CAMLprim value unix_accept(sock)
      value sock;
 {
   SOCKET sconn = (SOCKET) Handle_val(sock);
@@ -46,8 +47,10 @@ value unix_accept(sock)          /* ML */
     setsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE, 
                (char *) &oldvalue, oldvaluelen);
   }
-  if (snew == INVALID_SOCKET)
-    unix_error(WSAGetLastError(), "accept", Nothing);
+  if (snew == INVALID_SOCKET) {
+    win32_maperr(WSAGetLastError());
+    uerror("accept", Nothing);
+  }
   Begin_roots2 (fd, adr)
     fd = win_alloc_handle((HANDLE) snew);
     adr = alloc_sockaddr(&addr, addr_len);
