@@ -6,17 +6,18 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: bind.c,v 1.6 2000/03/16 13:35:19 xleroy Exp $ */
+/* $Id: bind.c,v 1.9 2001/12/07 13:40:43 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 #include "socketaddr.h"
   
-value unix_bind(socket, address)      /* ML */
+CAMLprim value unix_bind(socket, address)
      value socket, address;
 {
   int ret;
@@ -25,6 +26,9 @@ value unix_bind(socket, address)      /* ML */
 
   get_sockaddr(address, &addr, &addr_len);
   ret = bind((SOCKET) Handle_val(socket), &addr.s_gen, addr_len);
-  if (ret == -1) unix_error(WSAGetLastError(), "bind", Nothing);
+  if (ret == -1) {
+    win32_maperr(WSAGetLastError());
+    uerror("bind", Nothing);
+  }
   return Val_unit;
 }
