@@ -6,19 +6,20 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: close.c,v 1.2 1999/11/17 18:58:13 xleroy Exp $ */
+/* $Id: close.c,v 1.4 2001/12/07 13:40:43 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 
-value unix_close(value fd)             /* ML */
+CAMLprim value unix_close(value fd)
 {
   if (! CloseHandle(Handle_val(fd))) {
-    _dosmaperr(GetLastError());
+    win32_maperr(GetLastError());
     uerror("close", Nothing);
   }
   return Val_unit;
