@@ -6,17 +6,18 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: connect.c,v 1.6 2000/03/16 13:35:19 xleroy Exp $ */
+/* $Id: connect.c,v 1.9 2001/12/07 13:40:43 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 #include "socketaddr.h"
 
-value unix_connect(socket, address)   /* ML */
+CAMLprim value unix_connect(socket, address)
      value socket, address;
 {
   SOCKET s = (SOCKET) Handle_val(socket);
@@ -28,6 +29,9 @@ value unix_connect(socket, address)   /* ML */
   enter_blocking_section();
   retcode = connect(s, &addr.s_gen, addr_len);
   leave_blocking_section();
-  if (retcode == -1) unix_error(WSAGetLastError(), "connect", Nothing);
+  if (retcode == -1) {
+    win32_maperr(WSAGetLastError());
+    uerror("connect", Nothing);
+  }
   return Val_unit;
 }
