@@ -6,22 +6,23 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: dup.c,v 1.3 1999/11/17 18:58:13 xleroy Exp $ */
+/* $Id: dup.c,v 1.5 2001/12/07 13:40:44 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 
-value unix_dup(value fd)               /* ML */
+CAMLprim value unix_dup(value fd)
 {
   HANDLE newh;
   if (! DuplicateHandle(GetCurrentProcess(), Handle_val(fd),
                         GetCurrentProcess(), &newh,
                         0L, TRUE, DUPLICATE_SAME_ACCESS)) {
-    _dosmaperr(GetLastError());
+    win32_maperr(GetLastError());
     return -1;
   }
   return win_alloc_handle(newh);
