@@ -6,16 +6,17 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: dup2.c,v 1.3 1999/11/17 18:58:13 xleroy Exp $ */
+/* $Id: dup2.c,v 1.5 2001/12/07 13:40:44 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 
-value unix_dup2(value fd1, value fd2)        /* ML */
+CAMLprim value unix_dup2(value fd1, value fd2)
 {
   HANDLE oldh, newh;
 
@@ -23,7 +24,7 @@ value unix_dup2(value fd1, value fd2)        /* ML */
   if (! DuplicateHandle(GetCurrentProcess(), Handle_val(fd1),
                         GetCurrentProcess(), &newh,
                         0L, TRUE, DUPLICATE_SAME_ACCESS)) {
-    _dosmaperr(GetLastError());
+    win32_maperr(GetLastError());
     return -1;
   }
   Handle_val(fd2) = newh;
