@@ -6,17 +6,18 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: getpeername.c,v 1.5 2000/03/16 13:35:19 xleroy Exp $ */
+/* $Id: getpeername.c,v 1.8 2001/12/07 13:40:44 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 #include "socketaddr.h"
 
-value unix_getpeername(sock)          /* ML */
+CAMLprim value unix_getpeername(sock)
      value sock;
 {
   int retcode;
@@ -26,6 +27,9 @@ value unix_getpeername(sock)          /* ML */
   addr_len = sizeof(sock_addr);
   retcode = getpeername((SOCKET) Handle_val(sock),
                         &addr.s_gen, &addr_len);
-  if (retcode == -1) unix_error(WSAGetLastError(), "getpeername", Nothing);
+  if (retcode == -1) {
+    win32_maperr(WSAGetLastError());
+    uerror("getpeername", Nothing);
+  }
   return alloc_sockaddr(&addr, addr_len);
 }
