@@ -6,17 +6,18 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: getsockname.c,v 1.4 2000/03/16 13:35:19 xleroy Exp $ */
+/* $Id: getsockname.c,v 1.6 2001/12/07 13:40:44 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 #include "socketaddr.h"
 
-value unix_getsockname(sock)          /* ML */
+CAMLprim value unix_getsockname(sock)
      value sock;
 {
   int retcode;
