@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: gettimeofday.c,v 1.4 1999/11/29 19:03:55 doligez Exp $ */
+/* $Id: gettimeofday.c,v 1.6 2001/12/07 13:40:44 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
@@ -21,7 +22,7 @@
 static time_t initial_time = 0; /* 0 means uninitialized */
 static DWORD initial_tickcount;
 
-value unix_gettimeofday(value unit)                /* ML */
+CAMLprim value unix_gettimeofday(value unit)
 {
   if (initial_time == 0) {
     initial_tickcount = GetTickCount();
