@@ -6,20 +6,23 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: listen.c,v 1.5 1999/11/17 18:58:14 xleroy Exp $ */
+/* $Id: listen.c,v 1.8 2001/12/07 13:40:45 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 #include <winsock.h>
 
-value unix_listen(sock, backlog) /* ML */
+CAMLprim value unix_listen(sock, backlog)
      value sock, backlog;
 {
-  if (listen((SOCKET) Handle_val(sock), Int_val(backlog)) == -1)
-    unix_error(WSAGetLastError(), "listen", Nothing);
+  if (listen((SOCKET) Handle_val(sock), Int_val(backlog)) == -1) {
+    win32_maperr(WSAGetLastError());
+    uerror("listen", Nothing);
+  }
   return Val_unit;
 }
