@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: lseek.c,v 1.2 1999/11/17 18:58:14 xleroy Exp $ */
+/* $Id: lseek.c,v 1.4 2001/12/07 13:40:45 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
@@ -27,13 +28,13 @@ static int seek_command_table[] = {
   FILE_BEGIN, FILE_CURRENT, FILE_END
 };
 
-value unix_lseek(value fd, value ofs, value cmd)   /* ML */
+CAMLprim value unix_lseek(value fd, value ofs, value cmd)
 {
   long ret;
   ret = SetFilePointer(Handle_val(fd), Long_val(ofs), NULL,
                        seek_command_table[Int_val(cmd)]);
   if (ret == -1) {
-    _dosmaperr(GetLastError());
+    win32_maperr(GetLastError());
     uerror("lseek", Nothing);
   }
   return Val_long(ret);
