@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: read.c,v 1.3 2000/11/23 13:45:03 doligez Exp $ */
+/* $Id: read.c,v 1.5 2001/12/07 13:40:45 xleroy Exp $ */
 
 #include <string.h>
 #include <mlvalues.h>
@@ -18,7 +19,7 @@
 #include <signals.h>
 #include "unixsupport.h"
 
-value unix_read(value fd, value buf, value ofs, value len) /* ML */
+CAMLprim value unix_read(value fd, value buf, value ofs, value len)
 {
   DWORD numbytes, numread;
   BOOL ret;
@@ -32,7 +33,7 @@ value unix_read(value fd, value buf, value ofs, value len) /* ML */
     ret = ReadFile(h, iobuf, numbytes, &numread, NULL);
     leave_blocking_section();
     if (! ret) {
-      _dosmaperr(GetLastError());
+      win32_maperr(GetLastError());
       uerror("read", Nothing);
     }
     memmove (&Byte(buf, Long_val(ofs)), iobuf, numread);
