@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: sendrecv.c,v 1.11 2000/11/23 13:45:03 doligez Exp $ */
+/* $Id: sendrecv.c,v 1.14 2001/12/07 13:40:46 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
@@ -22,7 +23,7 @@ static int msg_flag_table[] = {
   MSG_OOB, MSG_DONTROUTE, MSG_PEEK
 };
 
-value unix_recv(value sock, value buff, value ofs, value len, value flags)
+CAMLprim value unix_recv(value sock, value buff, value ofs, value len, value flags)
 {
   int ret;
   long numbytes;
@@ -35,13 +36,16 @@ value unix_recv(value sock, value buff, value ofs, value len, value flags)
     ret = recv((SOCKET) Handle_val(sock), iobuf, (int) numbytes,
                convert_flag_list(flags, msg_flag_table));
     leave_blocking_section();
-    if (ret == -1) unix_error(WSAGetLastError(), "recv", Nothing);
+    if (ret == -1) {
+      win32_maperr(WSAGetLastError());
+      uerror("recv", Nothing);
+    }
     memmove (&Byte(buff, Long_val(ofs)), iobuf, ret);
   End_roots();
   return Val_int(ret);
 }
 
-value unix_recvfrom(value sock, value buff, value ofs, value len, value flags) /* ML */
+CAMLprim value unix_recvfrom(value sock, value buff, value ofs, value len, value flags)
 {
   int ret;
   long numbytes;
@@ -61,7 +65,10 @@ value unix_recvfrom(value sock, value buff, value ofs, value len, value flags) /
                    convert_flag_list(flags, msg_flag_table),
                    &addr.s_gen, &addr_len);
     leave_blocking_section();
-    if (ret == -1) unix_error(WSAGetLastError(), "recvfrom", Nothing);
+    if (ret == -1) {
+      win32_maperr(WSAGetLastError());
+      uerror("recvfrom", Nothing);
+    }
     memmove (&Byte(buff, Long_val(ofs)), iobuf, ret);
     adr = alloc_sockaddr(&addr, addr_len);
     res = alloc_small(2, 0);
@@ -71,7 +78,7 @@ value unix_recvfrom(value sock, value buff, value ofs, value len, value flags) /
   return res;
 }
 
-value unix_send(value sock, value buff, value ofs, value len, value flags) /* ML */
+CAMLprim value unix_send(value sock, value buff, value ofs, value len, value flags)
 {
   int ret;
   long numbytes;
@@ -84,7 +91,10 @@ value unix_send(value sock, value buff, value ofs, value len, value flags) /* ML
   ret = send((SOCKET) Handle_val(sock), iobuf, (int) numbytes,
              convert_flag_list(flags, msg_flag_table));
   leave_blocking_section();
-  if (ret == -1) unix_error(WSAGetLastError(), "send", Nothing);
+  if (ret == -1) {
+    win32_maperr(WSAGetLastError());
+    uerror("send", Nothing);
+  }
   return Val_int(ret);
 }
 
@@ -106,11 +116,14 @@ value unix_sendto_native(value sock, value buff, value ofs, value len, value fla
                convert_flag_list(flags, msg_flag_table),
                &addr.s_gen, addr_len);
   leave_blocking_section();
-  if (ret == -1) unix_error(WSAGetLastError(), "sendto", Nothing);
+  if (ret == -1) {
+    win32_maperr(WSAGetLastError());
+    uerror("sendto", Nothing);
+  }
   return Val_int(ret);
 }
 
-value unix_sendto(argv, argc)    /* ML */
+CAMLprim value unix_sendto(argv, argc)
      value * argv;
      int argc;
 {
