@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: shutdown.c,v 1.5 1999/11/17 18:58:15 xleroy Exp $ */
+/* $Id: shutdown.c,v 1.8 2001/12/07 13:40:46 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
@@ -20,11 +21,13 @@ static int shutdown_command_table[] = {
   0, 1, 2
 };
 
-value unix_shutdown(sock, cmd)   /* ML */
+CAMLprim value unix_shutdown(sock, cmd)
      value sock, cmd;
 {
   if (shutdown((SOCKET) Handle_val(sock),
-               shutdown_command_table[Int_val(cmd)]) == -1)
-    unix_error(WSAGetLastError(), "shutdown", Nothing);
+               shutdown_command_table[Int_val(cmd)]) == -1) {
+    win32_maperr(WSAGetLastError());
+    uerror("shutdown", Nothing);
+  }
   return Val_unit;
 }
