@@ -6,17 +6,18 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: sleep.c,v 1.2 1999/11/17 18:58:15 xleroy Exp $ */
+/* $Id: sleep.c,v 1.4 2001/12/07 13:40:46 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 #include <windows.h>
 
-value unix_sleep(t)              /* ML */
+CAMLprim value unix_sleep(t)
      value t;
 {
   enter_blocking_section();
