@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: system.c,v 1.5 1999/11/17 18:58:15 xleroy Exp $ */
+/* $Id: system.c,v 1.7 2001/12/07 13:40:46 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <memory.h>
@@ -19,7 +20,7 @@
 #include <process.h>
 #include <stdio.h>
 
-value win_system(cmd)           /* ML */
+CAMLprim value win_system(cmd)
      value cmd;
 {
   int ret;
