@@ -6,11 +6,12 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../../LICENSE.  *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: unix.ml,v 1.29 2001/07/27 12:52:05 xleroy Exp $ *)
+(* $Id: unix.ml,v 1.31 2001/12/07 13:40:47 xleroy Exp $ *)
 
 (* Initialization *)
 
@@ -188,8 +189,8 @@ let write fd buf ofs len =
 
 (* Interfacing with the standard input/output library *)
 
-external open_read_descriptor : int -> in_channel = "caml_open_descriptor"
-external open_write_descriptor : int -> out_channel = "caml_open_descriptor"
+external open_read_descriptor : int -> in_channel = "caml_open_descriptor_in"
+external open_write_descriptor : int -> out_channel = "caml_open_descriptor_out"
 external fd_of_in_channel : in_channel -> int = "channel_descriptor"
 external fd_of_out_channel : out_channel -> int = "channel_descriptor"
 
