@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: windir.c,v 1.8 1999/11/17 18:58:16 xleroy Exp $ */
+/* $Id: windir.c,v 1.10 2001/12/07 13:40:47 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <memory.h>
@@ -18,7 +19,7 @@
 #include <alloc.h>
 #include "unixsupport.h"
 
-value win_findfirst(name)             /* ML */
+CAMLprim value win_findfirst(name)
      value name;
 {
   int h;
@@ -42,7 +43,7 @@ value win_findfirst(name)             /* ML */
   return v;
 }
 
-value win_findnext(valh)        /* ML */
+CAMLprim value win_findnext(valh)
      value valh;
 {
   int retcode;
@@ -53,7 +54,7 @@ value win_findnext(valh)        /* ML */
   return copy_string(fileinfo.name);
 }
 
-value win_findclose(valh)       /* ML */
+CAMLprim value win_findclose(valh)
      value valh;
 {
   if (_findclose(Int_val(valh)) != 0) uerror("closedir", Nothing);
