@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: winwait.c,v 1.11 2001/06/25 13:07:15 xleroy Exp $ */
+/* $Id: winwait.c,v 1.13 2001/12/07 13:40:47 xleroy Exp $ */
 
 #include <windows.h>
 #include <mlvalues.h>
@@ -37,7 +38,7 @@ enum { CAML_WNOHANG = 1, CAML_WUNTRACED = 2 };
 
 static int wait_flag_table[] = { CAML_WNOHANG, CAML_WUNTRACED };
 
-value win_waitpid(value vflags, value vpid_req) /* ML */
+CAMLprim value win_waitpid(value vflags, value vpid_req)
 {
   int status, flags;
   HANDLE pid_req = (HANDLE) Long_val(vpid_req);
@@ -45,12 +46,12 @@ value win_waitpid(value vflags, value vpid_req) /* ML */
   flags = convert_flag_list(vflags, wait_flag_table);
   if ((flags & CAML_WNOHANG) == 0) {
     if (WaitForSingleObject(pid_req, INFINITE) == WAIT_FAILED) {
-      _dosmaperr(GetLastError());
+      win32_maperr(GetLastError());
       uerror("waitpid", Nothing);
     }
   }
   if (! GetExitCodeProcess(pid_req, &status)) {
-    _dosmaperr(GetLastError());
+    win32_maperr(GetLastError());
     uerror("waitpid", Nothing);
   }
   if (status == STILL_ACTIVE)
