@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: asttypes.mli,v 1.6 1999/11/17 18:58:17 xleroy Exp $ *)
+(* $Id: asttypes.mli,v 1.8 1999/12/06 17:05:19 garrigue Exp $ *)
 
 (* Auxiliary a.s.t. types used by parsetree and typedtree. *)
 
@@ -20,7 +20,7 @@ type constant =
   | Const_string of string
   | Const_float of string
 
-type rec_flag = Nonrecursive | Recursive
+type rec_flag = Nonrecursive | Recursive | Default
 
 type direction_flag = Upto | Downto
 
@@ -29,3 +29,5 @@ type private_flag = Private | Public
 type mutable_flag = Immutable | Mutable
 
 type virtual_flag = Virtual | Concrete
+
+type label = string
