@@ -10,14 +10,16 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: lexer.mli,v 1.10 2001/02/14 10:47:22 doligez Exp $ *)
+(* $Id: lexer.mli,v 1.12 2001/11/08 12:59:37 furuse Exp $ *)
 
 (* The lexical analyzer *)
 
 val token: Lexing.lexbuf -> Parser.token
+val skip_sharp_bang: Lexing.lexbuf -> unit
 
 type error =
   | Illegal_character of char
+  | Illegal_escape of string
   | Unterminated_comment
   | Unterminated_string
   | Unterminated_string_in_comment
