@@ -10,9 +10,10 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: location.mli,v 1.10 1999/11/17 18:58:18 xleroy Exp $ *)
+(* $Id: location.mli,v 1.11 2000/03/06 22:11:38 weis Exp $ *)
 
 (* Source code locations, used in parsetree *)
+open Format
 
 type t =
   { loc_start: int; loc_end: int; loc_ghost: bool }
@@ -25,8 +26,9 @@ val rhs_loc: int -> t
 val input_name: string ref
 val input_lexbuf: Lexing.lexbuf option ref
 
-val print: t -> unit
-val print_warning: t -> Warnings.t -> unit
+val print: formatter -> t -> unit
+val print_warning: t -> formatter -> Warnings.t -> unit
+val prerr_warning: t -> Warnings.t -> unit
 val echo_eof: unit -> unit
 val reset: unit -> unit
 
