@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: parse.ml,v 1.13 1999/12/30 17:09:37 doligez Exp $ *)
+(* $Id: parse.ml,v 1.14 2000/12/28 13:05:57 weis Exp $ *)
 
 (* Entry points in the parser *)
 
@@ -32,7 +32,7 @@ let rec skip_phrase lexbuf =
 
 let maybe_skip_phrase lexbuf =
   if Parsing.is_current_lookahead Parser.SEMISEMI
-  or Parsing.is_current_lookahead Parser.EOF
+  || Parsing.is_current_lookahead Parser.EOF
   then ()
   else skip_phrase lexbuf
 
