@@ -6,11 +6,12 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../LICENSE.     *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: arg.ml,v 1.13 2001/02/05 14:59:23 doligez Exp $ *)
+(* $Id: arg.ml,v 1.16 2001/12/07 13:40:48 xleroy Exp $ *)
 
 type spec =
   | Unit of (unit -> unit)     (* Call the function with unit argument *)
@@ -44,6 +45,8 @@ let usage speclist errmsg =
   List.iter (function (key, _, doc) -> eprintf "  %s %s\n" key doc) speclist;
   try ignore (assoc3 "-help" speclist)
   with Not_found -> eprintf "  -help  display this list of options\n";
+  try ignore (assoc3 "--help" speclist)
+  with Not_found -> eprintf "  --help display this list of options\n";
 ;;
 
 let current = ref 0;;
@@ -55,6 +58,7 @@ let parse speclist anonfun errmsg =
       if initpos < Array.length Sys.argv then Sys.argv.(initpos) else "(?)" in
     begin match error with
       | Unknown "-help" -> ()
+      | Unknown "--help" -> ()
       | Unknown s ->
           eprintf "%s: unknown option `%s'.\n" progname s
       | Missing s ->
@@ -66,7 +70,9 @@ let parse speclist anonfun errmsg =
           eprintf "%s: %s.\n" progname s
     end;
     usage speclist errmsg;
-    exit (if error = (Unknown "-help") then 0 else 2);
+    if error = Unknown "-help" || error = Unknown "--help"
+    then exit 0
+    else exit 2
   in
   let l = Array.length Sys.argv in
   incr current;
