@@ -6,11 +6,12 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../LICENSE.     *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: gc.ml,v 1.12 2001/02/05 14:59:23 doligez Exp $ *)
+(* $Id: gc.ml,v 1.14 2001/12/07 13:40:51 xleroy Exp $ *)
 
 type stat = {
   minor_words : float;
@@ -75,19 +76,22 @@ let allocated_bytes () =
 external finalise : ('a -> unit) -> 'a -> unit = "final_register";;
 
 
-type alarm = {mutable active : bool; f : unit -> unit};;
+type alarm_rec = {active : alarm; f : unit -> unit}
+and alarm = bool ref
+;;
 
-let rec call_alarm a =
-  if a.active then begin
-    finalise call_alarm a;
-    a.f ();
+let rec call_alarm arec =
+  if !(arec.active) then begin
+    finalise call_alarm arec;
+    arec.f ();
   end;
 ;;
 
 let create_alarm f =
-  let a = { active = true; f = f } in
-  finalise call_alarm a;
+  let a = ref true in
+  let arec = { active = ref true; f = f } in
+  finalise call_alarm arec;
   a
 ;;
 
-let delete_alarm a = a.active <- false;;
+let delete_alarm a = a := false;;
