@@ -6,15 +6,17 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../LICENSE.     *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: obj.mli,v 1.17 1999/11/17 18:58:26 xleroy Exp $ *)
+(* $Id: obj.mli,v 1.23 2001/12/07 13:40:56 xleroy Exp $ *)
 
-(* Module [Obj]: operations on internal representations of values *)
+(** Operations on internal representations of values.
 
-(* Not for the casual user. *)
+   Not for the casual user. 
+*)
 
 type t
 
@@ -22,6 +24,7 @@ external repr : 'a -> t = "%identity"
 external obj : t -> 'a = "%identity"
 external magic : 'a -> 'b = "%identity"
 external is_block : t -> bool = "obj_is_block"
+external is_int : t -> bool = "%obj_is_int"
 external tag : t -> int = "obj_tag"
 external size : t -> int = "%obj_size"
 external field : t -> int -> t = "%obj_field"
@@ -40,8 +43,7 @@ val double_tag : int
 val double_array_tag : int
 val final_tag : int
 
-(* The following two functions are deprecated.  Use module [Marshal]
-   instead. *)
+(** The following two functions are deprecated.  Use module {!Marshal} instead. *)
 
 val marshal : t -> string
 val unmarshal : string -> int -> t * int
