@@ -6,11 +6,12 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../LICENSE.     *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: parsing.ml,v 1.13 1999/12/08 08:21:57 garrigue Exp $ *)
+(* $Id: parsing.ml,v 1.15 2001/12/07 13:40:56 xleroy Exp $ *)
 
 (* The parsing engine *)
 
@@ -50,7 +51,9 @@ type parse_tables =
     tablesize : int;
     table : string;
     check : string;
-    error_function : string -> unit }
+    error_function : string -> unit;
+    names_const : string;
+    names_block : string }
 
 exception YYexit of Obj.t
 exception Parse_error
