@@ -6,11 +6,12 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../LICENSE.     *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pervasives.ml,v 1.42 2001/02/20 10:18:55 xleroy Exp $ *)
+(* $Id: pervasives.ml,v 1.46 2001/12/07 13:40:56 xleroy Exp $ *)
 
 (* type 'a option = None | Some of 'a *)
 
@@ -116,6 +117,15 @@ let neg_infinity =
 let nan =
   float_of_bytes "\127\240\000\000\000\000\000\001"
               (* 0x7F F0 00 00 00 00 00 01 *)
+let max_float =
+  float_of_bytes "\127\239\255\255\255\255\255\255"
+              (* 0x7f ef ff ff ff ff ff ff *)
+let min_float =
+  float_of_bytes "\000\016\000\000\000\000\000\000"
+              (* 0x00 10 00 00 00 00 00 00 *)
+let epsilon_float =
+  float_of_bytes "\060\176\000\000\000\000\000\000"
+              (* 0x3c b0 00 00 00 00 00 00 *)
 type fpclass =
     FP_normal
   | FP_subnormal
@@ -188,8 +198,8 @@ let rec (@) l1 l2 =
 type in_channel
 type out_channel
 
-external open_descriptor_out: int -> out_channel = "caml_open_descriptor"
-external open_descriptor_in: int -> in_channel = "caml_open_descriptor"
+external open_descriptor_out: int -> out_channel = "caml_open_descriptor_out"
+external open_descriptor_in: int -> in_channel = "caml_open_descriptor_in"
 
 let stdin = open_descriptor_in 0
 let stdout = open_descriptor_out 1
@@ -215,6 +225,15 @@ let open_out_bin name =
 
 external flush : out_channel -> unit = "caml_flush"
 
+external out_channels_list : unit -> out_channel list 
+                           = "caml_out_channels_list"
+
+let flush_all () = 
+  let rec iter = function
+      [] -> ()
+    | a::l -> flush a; iter l
+  in iter (out_channels_list ())
+
 external unsafe_output : out_channel -> string -> int -> int -> unit
                        = "caml_output"
 
@@ -345,7 +364,7 @@ external decr: int ref -> unit = "%decr"
 
 external sys_exit : int -> 'a = "sys_exit"
 
-let exit_function = ref (fun () -> flush stdout; flush stderr)
+let exit_function = ref flush_all
 
 let at_exit f =
   let g = !exit_function in
@@ -357,3 +376,6 @@ let exit retcode =
   do_at_exit ();
   sys_exit retcode
 
+external register_named_value: string -> 'a -> unit = "register_named_value"
+
+let _ = register_named_value "Pervasives.do_at_exit" do_at_exit
