@@ -6,11 +6,12 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../LICENSE.     *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: random.ml,v 1.14 2001/06/15 19:09:53 doligez Exp $ *)
+(* $Id: random.ml,v 1.16 2001/12/07 13:40:58 xleroy Exp $ *)
 
 (* "Linear feedback shift register" random number generator. *)
 (* References: Robert Sedgewick, "Algorithms", Addison-Wesley *)
@@ -61,19 +62,15 @@ let int bound =
 let float bound = rawfloat () *. bound
 
 (* Simple initialisation.  The seed is an integer.
-   Two seeds that are close enough will not produce uncorrelated
-   pseudo-random sequences.
 *)
 let init seed =
-  let st = ref seed in
-  let mdg () =
-    st := !st + 1;
-    let d = Digest.string (string_of_int !st) in
+  let mdg i =
+    let d = Digest.string (string_of_int i ^ string_of_int seed) in
     (Char.code d.[0] + (Char.code d.[1] lsl 8) + (Char.code d.[2] lsl 16))
     lxor (Char.code d.[3] lsl 22)
   in
   for i = 0 to 54 do
-    state.(i) <- (mdg ())
+    state.(i) <- (mdg i)
   done;
   index := 0
 
