@@ -6,11 +6,12 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../LICENSE.     *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: sort.ml,v 1.6 1999/11/17 18:58:29 xleroy Exp $ *)
+(* $Id: sort.ml,v 1.8 2001/12/07 13:40:58 xleroy Exp $ *)
 
 (* Merging and sorting *)
 
@@ -63,6 +64,9 @@ let array cmp arr =
       end;
       let pivot = unsafe_get arr mid in
       let i = ref (lo + 1) and j = ref (hi - 1) in
+      if not (cmp pivot (unsafe_get arr hi))
+         || not (cmp (unsafe_get arr lo) pivot)
+      then raise (Invalid_argument "Sort.array");
       while !i < !j do
         while not (cmp pivot (unsafe_get arr !i)) do incr i done;
         while not (cmp (unsafe_get arr !j) pivot) do decr j done;
