@@ -6,11 +6,12 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../LICENSE.     *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: stack.ml,v 1.5 1999/11/17 18:58:30 xleroy Exp $ *)
+(* $Id: stack.ml,v 1.8 2001/12/07 13:40:58 xleroy Exp $ *)
 
 type 'a t = { mutable c : 'a list }
 
@@ -20,6 +21,8 @@ let create () = { c = [] }
 
 let clear s = s.c <- []
 
+let copy s = { c = s.c }
+
 let push x s = s.c <- x :: s.c
 
 let pop s =
@@ -27,6 +30,11 @@ let pop s =
     hd::tl -> s.c <- tl; hd
   | []     -> raise Empty
 
+let top s =
+  match s.c with
+    hd::_ -> hd
+  | []     -> raise Empty
+
 let length s = List.length s.c
 
 let iter f s = List.iter f s.c
