@@ -6,18 +6,19 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../LICENSE.     *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: string.ml,v 1.20 2000/12/28 13:06:29 weis Exp $ *)
+(* $Id: string.ml,v 1.22 2001/12/07 13:40:59 xleroy Exp $ *)
 
 (* String operations *)
 
 external length : string -> int = "%string_length"
 external get : string -> int -> char = "%string_safe_get"
 external set : string -> int -> char -> unit = "%string_safe_set"
-external create: int -> string = "create_string"
+external create : int -> string = "create_string"
 external unsafe_get : string -> int -> char = "%string_unsafe_get"
 external unsafe_set : string -> int -> char -> unit = "%string_unsafe_set"
 external unsafe_blit : string -> int -> string -> int -> int -> unit
@@ -56,6 +57,9 @@ let blit s1 ofs1 s2 ofs2 len =
   then invalid_arg "String.blit"
   else unsafe_blit s1 ofs1 s2 ofs2 len
 
+let iter f a =
+  for i = 0 to length a - 1 do f(unsafe_get a i) done
+
 let concat sep l =
   match l with
     [] -> ""
