@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: dumpobj.ml,v 1.23 2000/04/06 13:05:11 xleroy Exp $ *)
+(* $Id: dumpobj.ml,v 1.26 2001/02/19 20:27:48 maranget Exp $ *)
 
 (* Disassembler for executable and .cmo object files *)
 
@@ -227,6 +227,7 @@ type shape =
   | Uint_Uint
   | Disp
   | Uint_Disp
+  | Sint_Disp
   | Getglobal
   | Getglobal_Uint
   | Setglobal
@@ -366,7 +367,19 @@ let op_shapes = [
   opGEINT, Nothing;
   opOFFSETINT, Sint;
   opOFFSETREF, Sint;
+  opISINT, Nothing;
   opGETMETHOD, Nothing;
+  opBEQ, Sint_Disp;
+  opBNEQ, Sint_Disp;
+  opBLTINT, Sint_Disp;
+  opBLEINT, Sint_Disp;
+  opBGTINT, Sint_Disp;
+  opBGEINT, Sint_Disp;
+  opULTINT, Nothing;
+  opUGEINT, Nothing;
+  opBULTINT, Uint_Disp;
+  opBUGEINT, Uint_Disp;
+  opRERAISE, Nothing;
   opSTOP, Nothing;
   opEVENT, Nothing;
   opBREAK, Nothing;
@@ -393,6 +406,9 @@ let print_instr ic =
   | Uint_Disp
      -> print_int (inputu ic); print_string ", ";
         let p = currpc ic in print_int (p + inputs ic)
+  | Sint_Disp
+     -> print_int (inputs ic); print_string ", ";
+        let p = currpc ic in print_int (p + inputs ic)
   | Getglobal -> print_getglobal_name ic
   | Getglobal_Uint
      -> print_getglobal_name ic; print_string ", "; print_int (inputu ic)
