@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: toploop.mli,v 1.15 2001/07/03 11:04:10 xleroy Exp $ *)
+(* $Id: toploop.mli,v 1.19 2001/08/07 12:12:27 ddr Exp $ *)
 
 open Format
 
@@ -69,7 +69,7 @@ val remove_printer : Path.t -> unit
 val max_printer_depth: int ref
 val max_printer_steps: int ref
 
-(* Hooks for an external parser *)
+(* Hooks for external parsers and printers *)
 
 val parse_toplevel_phrase : (Lexing.lexbuf -> Parsetree.toplevel_phrase) ref
 val parse_use_file : (Lexing.lexbuf -> Parsetree.toplevel_phrase list) ref
@@ -77,6 +77,11 @@ val print_location : formatter -> Location.t -> unit
 val print_warning : Location.t -> formatter -> Warnings.t -> unit
 val input_name : string ref
 
+val print_out_value : (formatter -> Outcometree.out_value -> unit) ref
+val print_out_type : (formatter -> Outcometree.out_type -> unit) ref
+val print_out_sig_item : (formatter -> Outcometree.out_sig_item -> unit) ref
+val print_out_phrase : (formatter -> Outcometree.out_phrase -> unit) ref
+
 (* Used by Trace module *)
 
 val may_trace : bool ref
