@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: btype.mli,v 1.8 2001/04/19 08:34:20 garrigue Exp $ *)
+(* $Id: btype.mli,v 1.10 2001/11/23 04:35:47 garrigue Exp $ *)
 
 (* Basic operations on core types *)
 
@@ -61,6 +61,12 @@ val iter_type_expr: (type_expr -> unit) -> type_expr -> unit
 val iter_row: (type_expr -> unit) -> row_desc -> unit
         (* Iteration on types in a row *)
 
+val copy_type_desc: (type_expr -> type_expr) -> type_desc -> type_desc
+        (* Copy on types *)
+val copy_row:
+    (type_expr -> type_expr) -> row_desc -> bool -> type_expr -> row_desc
+val copy_kind: field_kind -> field_kind
+
 val save_desc: type_expr -> type_desc -> unit
         (* Save a type description *)
 val cleanup_types: unit -> unit
