@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: ctype.ml,v 1.104 2001/06/08 15:24:34 garrigue Exp $ *)
+(* $Id: ctype.ml,v 1.115 2001/11/23 14:28:21 garrigue Exp $ *)
 
 (* Operations on core types *)
 
@@ -661,14 +661,6 @@ let rec copy ty =
     ty.desc <- Tsubst t;
     t.desc <-
       begin match desc with
-        Tvar ->
-          Tvar
-      | Tarrow (l, t1, t2, c) ->
-          let c =
-            if commu_repr c = Cok then Cok else Clink (ref Cunknown) in
-          Tarrow (l, copy t1, copy t2, c)
-      | Ttuple tl ->
-          Ttuple (List.map copy tl)
       | Tconstr (p, tl, _) ->
           begin match find_repr p !(!abbreviations) with
             Some ty when repr ty != t -> (* XXX Commentaire... *)
@@ -688,15 +680,6 @@ let rec copy ty =
                               Mcons _ -> Mlink !abbreviations
                             | abbrev  -> abbrev))
           end
-      | Tobject (t1, {contents = name}) ->
-          let name' =
-            match name with
-              None ->
-                None
-            | Some (p, tl) ->
-                Some (p, List.map copy tl)
-          in
-          Tobject (copy t1, ref name')
       | Tvariant row0 ->
           let row = row_repr row0 in
           let more = repr row.row_more in
@@ -707,45 +690,16 @@ let rec copy ty =
               ty.desc <- Tsubst ty2; (* avoid Tlink in the new type *)
               Tlink ty2
           | _ ->
-              (* We shall really check the level on the row variable *)
-              if more.level <> generic_level then Tvariant row0 else
-              (* We create a new copy *)
-              let fields =
-                List.map
-                  (fun (l,fi) -> l,
-                    match row_field_repr fi with
-                      Rpresent (Some ty) -> Rpresent(Some(copy ty))
-                    | Reither(c, l, m, _) ->
-                        Reither(c, List.map copy l, m, ref None)
-                    | fi -> fi)
-                  row.row_fields
-              and name =
-                may_map (fun (p,l) -> p, List.map copy l) row.row_name in
-              let var =
-                Tvariant { row_fields = fields; row_more = newvar();
-                           row_bound = List.map copy row.row_bound;
-                           row_closed = row.row_closed; row_name = name }
-              in
-              (* Remember it for other occurences *)
+              (* If the row variable is not generic, we must keep it *)
+              let keep = more.level <> generic_level in
+              (* Register new type first for recursion *)
               save_desc more more.desc;
               more.desc <- ty.desc;
-              var
+              (* Return a new copy *)
+              let more' = if keep then more else newvar () in
+              Tvariant (copy_row copy row keep more')
           end
-      | Tfield (label, kind, t1, t2) ->
-          begin match field_kind_repr kind with
-            Fpresent ->
-              Tfield (label, Fpresent, copy t1, copy t2)
-          | Fvar _ (* {contents = None} *) ->
-              Tfield (label, Fvar (ref None), copy t1, copy t2)
-          | Fabsent ->
-              assert false
-          end
-      | Tnil ->
-          Tnil
-      | Tlink t -> (* Actually unused *)
-          Tlink (copy t)
-      | Tsubst _ ->
-          assert false
+      | _ -> copy_type_desc copy desc
       end;
     t
 
@@ -1008,24 +962,24 @@ let rec non_recursive_abbrev env ty0 ty =
         begin try
           non_recursive_abbrev env ty0 (try_expand_head env ty)
         with Cannot_expand ->
+          if !Clflags.recursive_types then () else
           iter_type_expr (non_recursive_abbrev env ty0) ty
         end
     | Tobject _ | Tvariant _ ->
         ()
     | _ ->
+        if !Clflags.recursive_types then () else
         iter_type_expr (non_recursive_abbrev env ty0) ty
   end
 
 let correct_abbrev env ident params ty =
-  if not !Clflags.recursive_types then begin
-    let ty0 = newgenvar () in
-    visited := [];
-    non_recursive_abbrev env ty0
-      (subst env generic_level
-         (ref (Mcons (Path.Pident ident, ty0, ty0, Mnil))) None
-         [] [] ty);
-    visited := []
-  end
+  let ty0 = newgenvar () in
+  visited := [];
+  non_recursive_abbrev env ty0
+    (subst env generic_level
+       (ref (Mcons (Path.Pident ident, ty0, ty0, Mnil))) None
+       [] [] ty);
+  visited := []
 
 let rec occur_rec env visited ty0 ty =
   if ty == ty0  then raise Occur;
@@ -1321,6 +1275,13 @@ and unify_row env row1 row2 =
   in
   let empty fields =
     List.for_all (fun (_,f) -> row_field_repr f = Rabsent) fields in
+  if (row1.row_closed || row2.row_closed)
+  && (empty r1 || row2.row_closed) && (empty r2 || row1.row_closed)
+  && List.for_all
+      (fun (_,f1,f2) ->
+        row_field_repr f1 = Rabsent || row_field_repr f2 = Rabsent)
+      pairs
+  then raise (Unify []);
   let name =
     if row1.row_name <> None && (row1.row_closed || empty r2) &&
       (not row2.row_closed || keep (fun f1 f2 -> f1, f2) && empty r1)
@@ -1348,50 +1309,50 @@ and unify_row env row1 row2 =
   begin try
     rm1.desc <- Tlink (more row1 r2);
     rm2.desc <- Tlink (more row2 r1);
-    List.iter
-      (fun (l,f1,f2) ->
-        let f1 = row_field_repr f1 and f2 = row_field_repr f2 in
-        if f1 == f2 then () else
-        match f1, f2 with
-          Rpresent(Some t1), Rpresent(Some t2) -> unify env t1 t2
-        | Rpresent None, Rpresent None -> ()
-        | Reither(c1, tl1, m1, e1), Reither(c2, tl2, m2, e2) ->
-            if e1 == e2 then () else
-            let tl = tl1 @ tl2 in
-            let tl =
-              if not (m1 || m2) then
-                List.fold_right
-                  (fun t tl ->
-                    let t = repr t in if List.memq t tl then tl else t::tl)
-                  tl []
-              else match tl with
-              | [] -> []
-              | t1 :: tl ->
-                  if c1 || c2 then raise (Unify []);
-                  List.iter (unify env t1) tl;
-                  [t1]
-            in
-            let f = Reither(c1 || c2, tl, m1 || m2, ref None) in
-            e1 := Some f; e2 := Some f
-        | Reither(false, tl, _, e1), Rpresent(Some t2) ->
-            e1 := Some f2;
-            (try List.iter (fun t1 -> unify env t1 t2) tl
-             with exn -> e1 := None; raise exn)
-        | Rpresent(Some t1), Reither(false, tl, _, e2) ->
-            e2 := Some f1;
-            (try List.iter (unify env t1) tl
-             with exn -> e2 := None; raise exn)
-        | Reither(true, [], _, e1), Rpresent None -> e1 := Some f2
-        | Rpresent None, Reither(true, [], _, e2) -> e2 := Some f1
-        | Reither(_, _, false, e1), Rabsent -> e1 := Some f2
-        | Rabsent, Reither(_, _, false, e2) -> e2 := Some f1
-        | Rabsent, Rabsent -> ()
-        | _ -> raise (Unify []))
-      pairs
+    List.iter (fun (l,f1,f2) -> unify_row_field env f1 f2) pairs
   with exn ->
     rm1.desc <- md1; rm2.desc <- md2; raise exn
   end
 
+and unify_row_field env f1 f2 =
+  let f1 = row_field_repr f1 and f2 = row_field_repr f2 in
+  if f1 == f2 then () else
+  match f1, f2 with
+    Rpresent(Some t1), Rpresent(Some t2) -> unify env t1 t2
+  | Rpresent None, Rpresent None -> ()
+  | Reither(c1, tl1, m1, e1), Reither(c2, tl2, m2, e2) ->
+      if e1 == e2 then () else
+      if m1 || m2 then begin
+        match tl1 @ tl2 with [] -> ()
+        | t1 :: tl ->
+            if c1 || c2 then raise (Unify []);
+            List.iter (unify env t1) tl
+      end;
+      let tl1 = List.map repr tl1 and tl2 = List.map repr tl2 in
+      let rec remq tl = function [] -> []
+        | ty :: tl' ->
+            if List.memq ty tl then remq tl tl' else ty :: remq tl tl'
+      in
+      let tl2' = remq tl2 tl1 and tl1' = remq tl1 tl2 in
+      let e = ref None in
+      let f1 = Reither(c1 || c2, tl1', m1 || m2, e)
+      and f2 = Reither(c1 || c2, tl2', m1 || m2, e) in
+      e1 := Some f1; e2 := Some f2
+  | Reither(false, tl, _, e1), Rpresent(Some t2) ->
+      e1 := Some f2;
+      (try List.iter (fun t1 -> unify env t1 t2) tl
+      with exn -> e1 := None; raise exn)
+  | Rpresent(Some t1), Reither(false, tl, _, e2) ->
+      e2 := Some f1;
+      (try List.iter (unify env t1) tl
+      with exn -> e2 := None; raise exn)
+  | Reither(true, [], _, e1), Rpresent None -> e1 := Some f2
+  | Rpresent None, Reither(true, [], _, e2) -> e2 := Some f1
+  | Reither(_, _, false, e1), Rabsent -> e1 := Some f2
+  | Rabsent, Reither(_, _, false, e2) -> e2 := Some f1
+  | Rabsent, Rabsent -> ()
+  | _ -> raise (Unify [])
+
 let unify env ty1 ty2 =
   try
     unify env ty1 ty2
@@ -1623,15 +1584,17 @@ and moregen_row inst_nongen type_pairs env row1 row2 =
       | Reither(false, tl1, _, e1), Rpresent(Some t2) ->
           e1 := Some f2;
           List.iter (fun t1 -> moregen inst_nongen type_pairs env t1 t2) tl1
-      | Reither(c1, tl1, _, e1), Reither(c2, tl2, _, e2) ->
+      | Reither(c1, tl1, _, e1), Reither(c2, tl2, m2, e2) ->
           if c1 && not c2 then raise(Unify []);
-          e1 := Some f2;
-          begin match tl2 with
-            [t2] when tl1 <> [] -> List.iter
-                (fun t1 -> moregen inst_nongen type_pairs env t1 t2) tl1
-          | _ ->
-              if List.length tl1 <> List.length tl2 then raise (Unify []);
-              List.iter2 (moregen inst_nongen type_pairs env) tl1 tl2
+          e1 := Some (Reither (c2, [], m2, e2));
+          if List.length tl1 = List.length tl2 then
+            List.iter2 (moregen inst_nongen type_pairs env) tl1 tl2
+          else begin match tl2 with
+            t2 :: _ ->
+              List.iter (fun t1 -> moregen inst_nongen type_pairs env t1 t2)
+                tl1
+          | [] ->
+              if tl1 <> [] then raise (Unify [])
           end
       | Reither(true, [], _, e1), Rpresent None -> e1 := Some f2
       | Reither(_, _, _, e1), Rabsent -> e1 := Some f2
@@ -1776,9 +1739,18 @@ and eqtype_row rename type_pairs subst env row1 row2 =
       match row_field_repr f1, row_field_repr f2 with
         Rpresent(Some t1), Rpresent(Some t2) ->
           eqtype rename type_pairs subst env t1 t2
-      | Reither(c1, tl1, _, _), Reither(c2, tl2, _, _)
-        when c1 = c2 && List.length tl1 = List.length tl2 ->
-          List.iter2 (eqtype rename type_pairs subst env) tl1 tl2
+      | Reither(true, [], _, _), Reither(true, [], _, _) ->
+          ()
+      | Reither(false, t1::tl1, _, _), Reither(false, t2::tl2, _, _) ->
+          eqtype rename type_pairs subst env t1 t2;
+          if List.length tl1 = List.length tl2 then
+            (* if same length allow different types (meaning?) *)
+            List.iter2 (eqtype rename type_pairs subst env) tl1 tl2
+          else begin
+            (* otherwise everything must be equal *)
+            List.iter (eqtype rename type_pairs subst env t1) tl2;
+            List.iter (fun t1 -> eqtype rename type_pairs subst env t1 t2) tl1
+          end
       | Rpresent None, Rpresent None -> ()
       | Rabsent, Rabsent -> ()
       | _ -> raise (Unify []))
@@ -2159,22 +2131,26 @@ let rec build_subtype env visited loops posi onlyloop t =
       with Not_found -> build_subtype env visited loops posi onlyloop t'
       end
   | Tconstr(p, tl, abbrev) ->
-      let decl = Env.find_type p env in
-      let tl' =
-        List.map2
-          (fun (co,cn) t ->
-            if cn then
-              if co then (t, false)
-              else build_subtype env visited loops (not posi) onlyloop t
-            else
-              if co then build_subtype env visited loops posi onlyloop t
-              else (newvar(), true))
-          decl.type_variance tl
-      in
-      if List.exists snd tl' then
-        (newconstr p (List.map fst tl'), true)
-      else
+      begin try
+        let decl = Env.find_type p env in
+        let tl' =
+          List.map2
+            (fun (co,cn) t ->
+              if cn then
+                if co then (t, false)
+                else build_subtype env visited loops (not posi) onlyloop t
+              else
+                if co then build_subtype env visited loops posi onlyloop t
+                else (newvar(), true))
+            decl.type_variance tl
+        in
+        if List.exists snd tl' then
+          (newconstr p (List.map fst tl'), true)
+        else
+          (t, false)
+      with Not_found ->
         (t, false)
+      end
   | Tvariant row ->
       if List.memq t visited then (t, false) else
       let visited = t :: visited in
@@ -2285,18 +2261,22 @@ let rec subtype_rec env trace t1 t2 cstrs =
     | (_, Tconstr(p2, tl2, abbrev2)) when generic_abbrev env p2 ->
         subtype_rec env trace t1 (expand_abbrev env t2) cstrs
     | (Tconstr(p1, tl1, _), Tconstr(p2, tl2, _)) when Path.same p1 p2 ->
-        let decl = Env.find_type p1 env in
-        List.fold_left2
-          (fun cstrs (co, cn) (t1, t2) ->
-            if co then
-              if cn then
-                (trace, newty2 t1.level (Ttuple[t1]),
-                 newty2 t2.level (Ttuple[t2])) :: cstrs 
-              else subtype_rec env ((t1, t2)::trace) t1 t2 cstrs
-            else
-              if cn then subtype_rec env ((t2, t1)::trace) t2 t1 cstrs
-              else cstrs)
-          cstrs decl.type_variance (List.combine tl1 tl2)
+        begin try
+          let decl = Env.find_type p1 env in
+          List.fold_left2
+            (fun cstrs (co, cn) (t1, t2) ->
+              if co then
+                if cn then
+                  (trace, newty2 t1.level (Ttuple[t1]),
+                   newty2 t2.level (Ttuple[t2])) :: cstrs 
+                else subtype_rec env ((t1, t2)::trace) t1 t2 cstrs
+              else
+                if cn then subtype_rec env ((t2, t1)::trace) t2 t1 cstrs
+                else cstrs)
+            cstrs decl.type_variance (List.combine tl1 tl2)
+        with Not_found ->
+          (trace, t1, t2)::cstrs
+        end
     | (Tobject (f1, _), Tobject (f2, _))
               when opened_object f1 && opened_object f2 ->
         (* Same row variable implies same object. *)
@@ -2357,6 +2337,7 @@ let subtype env ty1 ty2 =
   TypePairs.clear subtypes;
   (* Build constraint set. *)
   let cstrs = subtype_rec env [(ty1, ty2)] ty1 ty2 [] in
+  TypePairs.clear subtypes;
   (* Enforce constraints. *)
   function () ->
     List.iter
@@ -2364,9 +2345,7 @@ let subtype env ty1 ty2 =
          try unify env t1 t2 with Unify trace ->
            raise (Subtype (expand_trace env (List.rev trace0),
                            List.tl (List.tl trace))))
-      (List.rev cstrs);
-    TypePairs.clear subtypes
-
+      (List.rev cstrs)
 
                               (*******************)
                               (*  Miscellaneous  *)
@@ -2498,14 +2477,6 @@ let rec nondep_type_rec env id ty =
     ty.desc <- Tsubst ty';
     ty'.desc <-
       begin match desc with
-        Tvar ->
-          fatal_error "Ctype.nondep_type_rec"
-      | Tarrow(l, t1, t2, c) ->
-          let c =
-            if commu_repr c = Cok then Cok else Clink (ref Cunknown) in
-          Tarrow(l, nondep_type_rec env id t1, nondep_type_rec env id t2, c)
-      | Ttuple tl ->
-          Ttuple(List.map (nondep_type_rec env id) tl)
       | Tconstr(p, tl, abbrev) ->
           if Path.isfree id p then
             begin try
@@ -2539,53 +2510,19 @@ let rec nondep_type_rec env id ty =
               ty.desc <- Tsubst ty2; (* avoid Tlink in the new type *)
               Tlink ty2
           | _ ->
-              (* We create a new copy *)
-              let bound = ref [] in
-              let fields =
-                List.map
-                  (fun (l,fi) -> l,
-                    match row_field_repr fi with
-                      Rpresent (Some ty) ->
-                        Rpresent(Some (nondep_type_rec env id ty))
-                    | Reither(c, l, m, _) ->
-                        let l = List.map (nondep_type_rec env id) l in
-                        bound := l @ !bound;
-                        Reither(c, l, m, ref None)
-                    | fi -> fi)
-                  row.row_fields
-              and name =
-                match row.row_name with
-                  Some (p,l) when Path.isfree id p ->
-                    Some (p, List.map (nondep_type_rec env id) l)
-                | _ -> None
-              in
-              let var =
-                Tvariant { row_fields = fields; row_more = newgenvar();
-                           row_bound = !bound;
-                           row_closed = row.row_closed; row_name = name }
-              in
-              (* Remember it for other occurences *)
+              let static = static_row row in
+              (* Register new type first for recursion *)
               save_desc more more.desc;
               more.desc <- ty.desc;
-              var
+              let more' = if static then newgenvar () else more in
+              (* Return a new copy *)
+              let row = copy_row (nondep_type_rec env id) row true more' in
+              match row.row_name with
+                Some (p, tl) when Path.isfree id p ->
+                  Tvariant {row with row_name = None}
+              | _ -> Tvariant row
           end
-      | Tfield(label, kind, t1, t2) ->
-          begin match field_kind_repr kind with
-            Fpresent ->
-              Tfield(label, Fpresent, nondep_type_rec env id t1,
-                                      nondep_type_rec env id t2)
-          | Fvar _ (* {contents = None} *) as k ->
-              Tfield(label, k, nondep_type_rec env id t1,
-                               nondep_type_rec env id t2)
-          | Fabsent ->
-              assert false
-          end
-      | Tnil ->
-          Tnil
-      | Tlink ty ->                    (* Actually unused *)
-          Tlink(nondep_type_rec env id ty)
-      | Tsubst _ ->
-          assert false
+      | _ -> copy_type_desc (nondep_type_rec env id) desc
       end;
     ty'
 
