@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: datarepr.mli,v 1.6 1999/11/17 18:58:53 xleroy Exp $ *)
+(* $Id: datarepr.mli,v 1.8 2000/03/21 14:43:23 xleroy Exp $ *)
 
 (* Compute constructor and label descriptions from type declarations,
    determining their representation. *)
@@ -25,7 +25,7 @@ val exception_descr:
   Path.t -> type_expr list -> constructor_description
 val label_descrs:
   type_expr -> (string * mutable_flag * type_expr) list ->
-    (string * label_description) list
+    record_representation -> (string * label_description) list
 
 exception Constr_not_found
 
