@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: includemod.ml,v 1.28 2000/03/06 22:11:58 weis Exp $ *)
+(* $Id: includemod.ml,v 1.29 2000/12/28 13:07:27 weis Exp $ *)
 
 (* Inclusion checks for the module language *)
 
@@ -118,7 +118,7 @@ let simplify_structure_coercion cc =
   try
     List.iter
       (fun (n, c) ->
-        if n <> !pos or c <> Tcoerce_none then raise Exit;
+        if n <> !pos || c <> Tcoerce_none then raise Exit;
         incr pos)
       cc;
     Tcoerce_none
