@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: mtype.ml,v 1.19 1999/11/25 16:47:47 xleroy Exp $ *)
+(* $Id: mtype.ml,v 1.20 2000/09/06 10:21:06 garrigue Exp $ *)
 
 (* Operations on module types *)
 
@@ -46,13 +46,9 @@ and strengthen_sig env sg p =
       let newdecl =
         match decl.type_manifest with
           None ->
-            { type_params = decl.type_params;
-              type_arity = decl.type_arity;
-              type_kind = decl.type_kind;
-              type_manifest = Some(Btype.newgenty(
-                                   Tconstr(Pdot(p, Ident.name id, nopos),
-                                           decl.type_params,
-                                           ref Mnil))) }
+            { decl with type_manifest =
+                Some(Btype.newgenty(Tconstr(Pdot(p, Ident.name id, nopos),
+                                            decl.type_params, ref Mnil))) }
         | _ -> decl in
       Tsig_type(id, newdecl) :: strengthen_sig env rem p
   | (Tsig_exception(id, d) as sigelt) :: rem ->
