@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: parmatch.ml,v 1.30 2001/04/23 09:29:22 maranget Exp $ *)
+(* $Id: parmatch.ml,v 1.35 2001/11/12 11:02:40 maranget Exp $ *)
 
 (* Detection of partial matches and unused match cases. *)
 
@@ -176,7 +176,7 @@ let simple_match_args p1 p2 =
 let rec normalize_pat q = match q.pat_desc with
   | Tpat_any | Tpat_constant _ -> q
   | Tpat_var _ -> make_pat Tpat_any q.pat_type q.pat_env
-  | Tpat_alias (p,_) -> normalize_pat q
+  | Tpat_alias (p,_) -> normalize_pat p
   | Tpat_tuple (args) ->
       make_pat (Tpat_tuple (omega_list args)) q.pat_type q.pat_env
   | Tpat_construct  (c,args) ->
@@ -189,7 +189,7 @@ let rec normalize_pat q = match q.pat_desc with
   | Tpat_record (largs) ->
       make_pat (Tpat_record (List.map (fun (lbl,_) -> lbl,omega) largs))
         q.pat_type q.pat_env
-  | Tpat_or (_,_) -> fatal_error "Parmatch.normalize_pat"
+  | Tpat_or _ -> fatal_error "Parmatch.normalize_pat"
 
 
 (*
@@ -202,7 +202,7 @@ let discr_pat q pss =
   let rec acc_pat acc pss = match pss with
     ({pat_desc = Tpat_alias (p,_)}::ps)::pss -> 
         acc_pat acc ((p::ps)::pss)
-  | ({pat_desc = Tpat_or (p1,p2)}::ps)::pss ->
+  | ({pat_desc = Tpat_or (p1,p2,_)}::ps)::pss ->
         acc_pat acc ((p1::ps)::(p2::ps)::pss)
   | ({pat_desc = (Tpat_any | Tpat_var _)}::_)::pss ->
         acc_pat acc pss
@@ -282,7 +282,7 @@ let filter_one q pss =
   let rec filter_rec = function
       ({pat_desc = Tpat_alias(p,_)}::ps)::pss -> 
         filter_rec ((p::ps)::pss)
-    | ({pat_desc = Tpat_or(p1,p2)}::ps)::pss ->
+    | ({pat_desc = Tpat_or(p1,p2,_)}::ps)::pss ->
         filter_rec ((p1::ps)::(p2::ps)::pss)
     | (p::ps)::pss ->
         if simple_match q p
@@ -300,7 +300,7 @@ let filter_extra pss =
   let rec filter_rec = function
       ({pat_desc = Tpat_alias(p,_)}::ps)::pss -> 
         filter_rec ((p::ps)::pss)
-    | ({pat_desc = Tpat_or(p1,p2)}::ps)::pss ->
+    | ({pat_desc = Tpat_or(p1,p2,_)}::ps)::pss ->
         filter_rec ((p1::ps)::(p2::ps)::pss)
     | ({pat_desc = (Tpat_any | Tpat_var(_))} :: qs) :: pss ->
         qs :: filter_rec pss
@@ -333,7 +333,7 @@ let filter_all pat0 pss =
   let rec filter_rec env = function
     ({pat_desc = Tpat_alias(p,_)}::ps)::pss ->
       filter_rec env ((p::ps)::pss)
-  | ({pat_desc = Tpat_or(p1,p2)}::ps)::pss ->
+  | ({pat_desc = Tpat_or(p1,p2,_)}::ps)::pss ->
       filter_rec env ((p1::ps)::(p2::ps)::pss)
   | ({pat_desc = (Tpat_any | Tpat_var(_))}::_)::pss ->
       filter_rec env pss
@@ -344,7 +344,7 @@ let filter_all pat0 pss =
   and filter_omega env = function
     ({pat_desc = Tpat_alias(p,_)}::ps)::pss ->
       filter_omega env ((p::ps)::pss)
-  | ({pat_desc = Tpat_or(p1,p2)}::ps)::pss ->
+  | ({pat_desc = Tpat_or(p1,p2,_)}::ps)::pss ->
       filter_omega env ((p1::ps)::(p2::ps)::pss)
   | ({pat_desc = (Tpat_any | Tpat_var(_))}::ps)::pss ->
       filter_omega
@@ -450,7 +450,7 @@ let rec pat_of_constrs ex_pat = function
     pat_desc=
       Tpat_or
         (pat_of_constr ex_pat cstr,
-         pat_of_constrs ex_pat rem)}
+         pat_of_constrs ex_pat rem, None)}
 
 (* Sends back a pattern that complements constructor tags all_tag *)
 let complete_constrs p all_tags = match p.pat_desc with
@@ -518,7 +518,7 @@ let build_other env =  match env with
     | pat::other_pats ->
         List.fold_left
           (fun p_res pat ->
-            make_pat (Tpat_or (pat, p_res)) p.pat_type p.pat_env)
+            make_pat (Tpat_or (pat, p_res, None)) p.pat_type p.pat_env)
           pat other_pats
     end
 | ({pat_desc = Tpat_constant(Const_char _)} as p,_) :: _ ->
@@ -633,7 +633,7 @@ let rec satisfiable tdefs build pss qs =
   | _ ->
     match qs with
       [] -> Rnone
-    | {pat_desc = Tpat_or(q1,q2)}::qs ->
+    | {pat_desc = Tpat_or(q1,q2,_)}::qs ->
         begin match satisfiable tdefs build pss (q1::qs) with
         | Rnone -> satisfiable tdefs build pss (q2::qs)
         | r -> r
@@ -685,18 +685,6 @@ let has_guard act =
     Texp_when(_, _) -> true
   | _ -> false
 
-(*
-  Build up a working pattern matrix.
-    1- Retain minimal patterns (for optimizing when catch all's  are here)
-    2- Forget about guarded patterns
-*)
-
-let rec initial_matrix = function
-    [] -> []
-  | (pat, act) :: rem ->
-       if has_guard act
-       then initial_matrix rem
-       else [pat] :: initial_matrix rem
 
 (* p less_equal means, forall B,  V matches q implies V mactches p *)
 let rec le_pat p q =
@@ -704,7 +692,7 @@ let rec le_pat p q =
   | Tpat_var _,_ -> true | Tpat_any, _ -> true
   | Tpat_alias(p,_), _ -> le_pat p q
   | _, Tpat_alias(q,_) -> le_pat p q
-  | _, Tpat_or(q1,q2) -> le_pat p q1 && le_pat p q2
+  | _, Tpat_or(q1,q2,_) -> le_pat p q1 && le_pat p q2
   | Tpat_constant(c1), Tpat_constant(c2) -> c1 = c2
   | Tpat_construct(c1,ps), Tpat_construct(c2,qs) ->
       c1.cstr_tag = c2.cstr_tag && le_pats ps qs
@@ -800,7 +788,7 @@ let rec pretty_val ppf v = match v.pat_desc with
       fprintf ppf "@[[| %a |]@]" (pretty_vals " ;") vs
   | Tpat_alias (v,x) ->
       fprintf ppf "@[(%a@ as %a)@]" pretty_val v Ident.print x
-  | Tpat_or (v,w)    ->
+  | Tpat_or (v,w,_)    ->
       fprintf ppf "@[(%a|@,%a)@]" pretty_or v pretty_or w
 
 and pretty_car ppf v = match v.pat_desc with
@@ -820,7 +808,7 @@ and pretty_arg ppf v = match v.pat_desc with
 |  _ -> pretty_val ppf v
 
 and pretty_or ppf v = match v.pat_desc with
-| Tpat_or (v,w) ->
+| Tpat_or (v,w,_) ->
     fprintf ppf "%a|@,%a" pretty_or v pretty_or w
 | _ -> pretty_val ppf v
 
@@ -847,12 +835,12 @@ let top_pretty ppf v =
 
 let rec compat p q =
   match p.pat_desc,q.pat_desc with
-  | Tpat_alias (p,_),_     -> compat p q
-  | _,Tpat_alias (q,_)     -> compat p q
+  | Tpat_alias (p,_),_      -> compat p q
+  | _,Tpat_alias (q,_)      -> compat p q
   | (Tpat_any|Tpat_var _),_ -> true
   | _,(Tpat_any|Tpat_var _) -> true
-  | Tpat_or (p1,p2),_       -> compat p1 q || compat p2 q
-  | _,Tpat_or (q1,q2)       -> compat p q1 || compat p q2    
+  | Tpat_or (p1,p2,_),_     -> compat p1 q || compat p2 q
+  | _,Tpat_or (q1,q2,_)     -> compat p q1 || compat p q2    
   | Tpat_constant c1, Tpat_constant c2 -> c1=c2
   | Tpat_tuple ps, Tpat_tuple qs -> compats ps qs
   | Tpat_construct (c1,ps1), Tpat_construct (c2,ps2) ->
@@ -883,12 +871,12 @@ and compats ps qs = match ps,qs with
 *)
 
 let rec lub p q = match p.pat_desc,q.pat_desc with
-| Tpat_alias (p,_),_     -> lub p q
-| _,Tpat_alias (q,_)     -> lub p q
+| Tpat_alias (p,_),_      -> lub p q
+| _,Tpat_alias (q,_)      -> lub p q
 | (Tpat_any|Tpat_var _),_ -> q
 | _,(Tpat_any|Tpat_var _) -> p
-| Tpat_or (p1,p2),_       -> orlub p1 p2 q
-| _,Tpat_or (q1,q2)       -> orlub q1 q2 p (* Thanks god, lub is commutative *)
+| Tpat_or (p1,p2,_),_     -> orlub p1 p2 q
+| _,Tpat_or (q1,q2,_)     -> orlub q1 q2 p (* Thanks god, lub is commutative *)
 | Tpat_constant c1, Tpat_constant c2 when c1=c2 -> p
 | Tpat_tuple ps, Tpat_tuple qs ->
     let rs = lubs ps qs in
@@ -917,7 +905,7 @@ and orlub p1 p2 q =
   try    
     let r1 = lub p1 q in
     try
-      {q with pat_desc=(Tpat_or (r1,lub p2 q))}
+      {q with pat_desc=(Tpat_or (r1,lub p2 q,None))}
     with
     | Empty -> r1
   with
@@ -962,39 +950,128 @@ and lubs ps qs = match ps,qs with
     Ok, note that by contrast, unused clause check still can be avoided at
         user request.
 *)
+(*
+  Build up a working pattern matrix.
+    1- Retain minimal patterns (for optimizing when catch all's  are here)
+    2- Forget about guarded patterns
+*)
+
+let rec initial_matrix = function
+    [] -> []
+  | (pat, act) :: rem ->
+       if has_guard act
+       then
+         initial_matrix rem
+       else
+         [pat] :: initial_matrix rem
+
+exception NoGuard
+
+let rec initial_all no_guard = function
+  | [] ->
+      if no_guard then
+        raise NoGuard
+      else
+       []
+  | (pat, act) :: rem ->
+      ([pat], pat.pat_loc) :: initial_all (no_guard && not (has_guard act)) rem
+
+
+let rec do_filter_var = function
+  | (_::ps,loc)::rem -> (ps,loc)::do_filter_var rem
+  | _ -> []
+
+let do_filter_one q pss =
+  let rec filter_rec = function
+    | ({pat_desc = Tpat_alias(p,_)}::ps,loc)::pss -> 
+        filter_rec ((p::ps,loc)::pss)
+    | ({pat_desc = Tpat_or(p1,p2,_)}::ps,loc)::pss ->
+          filter_rec ((p1::ps,loc)::(p2::ps,loc)::pss)
+    | (p::ps,loc)::pss ->
+          if simple_match q p
+          then (simple_match_args q p @ ps, loc) :: filter_rec pss
+          else filter_rec pss
+    | _ -> [] in
+    filter_rec pss
+
+(* Check whether value v can be matched, considering guarded clauses *) 
+let rec do_match pss qs = match qs with
+| [] ->
+    begin match pss  with
+    | ([],loc)::_ -> Some loc
+    | _ -> None
+    end
+| q::qs -> match q with
+  | {pat_desc = Tpat_or (q1,q2,_)} ->
+      begin match do_match pss (q1::qs) with
+      | None -> do_match pss (q2::qs)
+      | r -> r
+      end
+  | {pat_desc = Tpat_any} ->
+      do_match (do_filter_var pss) qs
+  | _ ->
+      let q0 = normalize_pat q in
+      do_match (do_filter_one q0 pss) (simple_match_args q0 q @ qs)
+
+    
+let check_partial_all v casel =
+  try
+    let pss =
+      get_mins
+        (fun (p,_) (q,_) -> le_pats p q)
+        (initial_all true casel) in
+    do_match pss [v]
+  with
+  | NoGuard -> None
 
 let check_partial tdefs loc casel =
-    let pss = get_mins le_pats (initial_matrix casel) in    
-    match pss with
-    | [] ->
+  let pss = get_mins le_pats (initial_matrix casel) in    
+  match pss with
+  | [] ->
         (*
           This can occur
-            - For empty matches generated by ocamlp4
-            - when all patterns have guards
-           Then match should be considered non-exhaustive
-           (cf. matching.ml) no warning is issued,
-           users should know what they do 
-         *)
-        Partial
-    | ps::_  ->
-        match satisfiable tdefs true pss (omega_list ps) with
-        | Rnone -> Total
-        | Rok ->
-            Location.prerr_warning loc (Warnings.Partial_match "");
-            Partial
-        | Rsome [v] ->
-            let errmsg =
-              try
-                let buf = Buffer.create 16 in
-                let fmt = formatter_of_buffer buf in
-                top_pretty fmt v;
-                Buffer.contents buf
-              with _ ->
-                "" in
-            Location.prerr_warning loc (Warnings.Partial_match errmsg);
-            Partial
-        | _ ->
-            fatal_error "Parmatch.check_partial"
+          - For empty matches generated by ocamlp4 (no warning)
+          - when all patterns have guards (then casel <> [], warning)
+          Then match should be considered non-exhaustive,
+          otherwise compilation of PM is broken.
+          *)
+      begin match casel with
+      | [] -> ()
+      | _  ->
+          Location.prerr_warning loc
+            (Warnings.Other
+               "Bad style, all clauses in this pattern-matching are guarded.")
+      end ;
+      Partial
+  | ps::_  ->
+      match satisfiable tdefs true pss (omega_list ps) with
+      | Rnone -> Total
+      | Rok ->            
+          Location.prerr_warning loc (Warnings.Partial_match "");
+          Partial
+      | Rsome [v] ->
+          let errmsg =
+            try
+              let buf = Buffer.create 16 in
+              let fmt = formatter_of_buffer buf in
+              top_pretty fmt v;
+              begin match check_partial_all v casel with
+              | None -> ()
+              | Some _ ->
+                  (* This is ``Some l'', where l is the location of
+                     a possibly matching clause.
+                     I forget about l, because printing two locations
+                     is a plain in the top-level *)                  
+                  Buffer.add_string buf
+                   "\n(However, some guarded clause may match this value.)"
+              end ;
+              Buffer.contents buf
+            with _ ->
+              "" in          
+          Location.prerr_warning loc (Warnings.Partial_match errmsg) ;
+          Partial
+      | _ ->
+          fatal_error "Parmatch.check_partial"
 
 let location_of_clause = function
     pat :: _ -> pat.pat_loc
