@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: typedtree.mli,v 1.28 2000/12/04 15:37:04 doligez Exp $ *)
+(* $Id: typedtree.mli,v 1.29 2001/09/25 09:54:16 garrigue Exp $ *)
 
 (* Abstract syntax tree after typing *)
 
@@ -35,7 +35,7 @@ and pattern_desc =
   | Tpat_variant of label * pattern option * row_desc
   | Tpat_record of (label_description * pattern) list
   | Tpat_array of pattern list
-  | Tpat_or of pattern * pattern
+  | Tpat_or of pattern * pattern * Path.t option
 
 type partial = Partial | Total
 type optional = Required | Optional
