@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: typemod.ml,v 1.43 2001/07/23 15:35:49 xleroy Exp $ *)
+(* $Id: typemod.ml,v 1.48 2001/09/28 22:55:27 garrigue Exp $ *)
 
 (* Type-checking of the module language *)
 
@@ -263,11 +263,13 @@ let check_nongen_schemes env str =
 
 (* Extract the list of "value" identifiers bound by a signature.
    "Value" identifiers are identifiers for signature components that
-   correspond to a run-time value: values, exceptions, modules, classes *)
+   correspond to a run-time value: values, exceptions, modules, classes.
+   Note: manifest primitives do not correspond to a run-time value! *)
 
 let rec bound_value_identifiers = function
     [] -> []
-  | Tsig_value(id, decl) :: rem -> id :: bound_value_identifiers rem
+  | Tsig_value(id, {val_kind = Val_reg}) :: rem ->
+      id :: bound_value_identifiers rem
   | Tsig_exception(id, decl) :: rem -> id :: bound_value_identifiers rem
   | Tsig_module(id, mty) :: rem -> id :: bound_value_identifiers rem
   | Tsig_class(id, decl) :: rem -> id :: bound_value_identifiers rem
@@ -412,7 +414,10 @@ and type_structure env sstr =
         let (path, mty) = type_module_path env loc lid in
         let sg = extract_sig_open env loc mty in
         type_struct (Env.open_signature path sg env) srem
-    | {pstr_desc = Pstr_class cl} :: srem ->
+    | {pstr_desc = Pstr_class cl; pstr_loc = loc} :: srem ->
+         List.iter
+           (fun {pci_name = name} -> check "type" loc type_names name)
+           cl;
         let (classes, new_env) = Typeclass.class_declarations env cl in
         let (str_rem, sig_rem, final_env) = type_struct new_env srem in
         (Tstr_class
@@ -432,7 +437,10 @@ and type_structure env sstr =
                   Tsig_type(i'', d''); Tsig_type(i''', d''')])
               classes [sig_rem]),
          final_env)
-    | {pstr_desc = Pstr_class_type cl} :: srem ->
+    | {pstr_desc = Pstr_class_type cl; pstr_loc = loc} :: srem ->
+        List.iter
+          (fun {pci_name = name} -> check "type" loc type_names name)
+          cl;
         let (classes, new_env) = Typeclass.class_type_declarations env cl in
         let (str_rem, sig_rem, final_env) = type_struct new_env srem in
         (Tstr_cltype
