@@ -10,11 +10,12 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: typemod.mli,v 1.17 1999/11/17 18:58:58 xleroy Exp $ *)
+(* $Id: typemod.mli,v 1.18 2000/03/06 22:12:08 weis Exp $ *)
 
 (* Type-checking of the module language *)
 
 open Types
+open Format
 
 val type_module:
         Env.t -> Parsetree.module_expr -> Typedtree.module_expr
@@ -45,4 +46,4 @@ type error =
 
 exception Error of Location.t * error
 
-val report_error: error -> unit
+val report_error: formatter -> error -> unit
