@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: types.mli,v 1.13 2000/03/21 14:43:24 xleroy Exp $ *)
+(* $Id: types.mli,v 1.16 2001/04/19 08:34:21 garrigue Exp $ *)
 
 (* Representation of types and declarations *)
 
@@ -25,7 +25,7 @@ type type_expr =
 
 and type_desc =
     Tvar
-  | Tarrow of label * type_expr * type_expr
+  | Tarrow of label * type_expr * type_expr * commutable
   | Ttuple of type_expr list
   | Tconstr of Path.t * type_expr list * abbrev_memo ref
   | Tobject of type_expr * (Path.t * type_expr list) option ref
@@ -44,8 +44,10 @@ and row_desc =
 
 and row_field =
     Rpresent of type_expr option
-  | Reither of bool * type_expr list * row_field option ref
-        (* true denotes a constant constructor *)
+  | Reither of bool * type_expr list * bool * row_field option ref
+        (* 1st true denotes a constant constructor *)
+        (* 2nd true denotes a tag in a pattern matching, and
+           is erased later *)
   | Rabsent
 
 and abbrev_memo =
@@ -58,6 +60,11 @@ and field_kind =
   | Fpresent
   | Fabsent
 
+and commutable =
+    Cok
+  | Cunknown
+  | Clink of commutable ref
+
 (* Maps of methods and instance variables *)
 
 module Meths : Map.S with type key = string
@@ -116,7 +123,8 @@ type type_declaration =
   { type_params: type_expr list;
     type_arity: int;
     type_kind: type_kind;
-    type_manifest: type_expr option }
+    type_manifest: type_expr option;
+    type_variance: (bool * bool) list } (* covariant, contravariant *)
 
 and type_kind =
     Type_abstract
