@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: ccomp.ml,v 1.10 2000/04/16 14:34:58 xleroy Exp $ *)
+(* $Id: ccomp.ml,v 1.11 2000/12/28 13:07:41 weis Exp $ *)
 
 (* Compiling C files and building C libraries *)
 
@@ -61,7 +61,7 @@ let create_archive archive file_list =
       let r1 =
         command(Printf.sprintf "ar rc %s %s"
                                    archive (String.concat " " file_list)) in
-      if r1 <> 0 or String.length Config.ranlib = 0
+      if r1 <> 0 || String.length Config.ranlib = 0
       then r1
       else command(Config.ranlib ^ " " ^ archive)
 
