@@ -10,12 +10,13 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: clflags.ml,v 1.36 2001/02/19 20:27:52 maranget Exp $ *)
+(* $Id: clflags.ml,v 1.39 2001/10/30 09:32:32 xleroy Exp $ *)
 
 (* Command-line parameters *)
 
 let objfiles = ref ([] : string list)   (* .cmo and .cma files *)
-and ccobjs = ref ([] : string list)     (* .o, .a and -lxxx files *)
+and ccobjs = ref ([] : string list)     (* .o, .a, .so and -cclib -lxxx *)
+and dllibs = ref ([] : string list)     (* .so and -dllib -lxxx *)
 
 let compile_only = ref false            (* -c *)
 and exec_name = ref "a.out"             (* -o *)
@@ -30,7 +31,7 @@ and link_everything = ref false         (* -linkall *)
 and custom_runtime = ref false          (* -custom *)
 and output_c_object = ref false         (* -output-obj *)
 and ccopts = ref ([] : string list)     (* -ccopt *)
-and classic = ref true                  (* -labels *)
+and classic = ref false                 (* -nolabels *)
 and nopervasives = ref false            (* -nopervasives *)
 and preprocessor = ref(None : string option) (* -pp *)
 and thread_safe = ref false             (* -thread *)
@@ -44,6 +45,7 @@ and gprofile = ref false                (* -p *)
 and c_compiler = ref Config.bytecomp_c_compiler (* -cc *)
 and c_linker = ref Config.bytecomp_c_linker (* -cc *)
 and no_auto_link = ref false            (* -noautolink *)
+and dllpaths = ref ([] : string list)   (* -dllpath *)
 let dump_parsetree = ref false          (* -dparsetree *)
 and dump_rawlambda = ref false          (* -drawlambda *)
 and dump_lambda = ref false             (* -dlambda *)
