@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: config.mli,v 1.20 2001/05/28 15:14:18 xleroy Exp $ *)
+(* $Id: config.mli,v 1.24 2001/10/30 09:32:32 xleroy Exp $ *)
 
 (* System configuration *)
 
@@ -87,3 +87,5 @@ val ext_asm: string
         (* Extension for assembler files, e.g. [.s] under Unix. *)
 val ext_lib: string
         (* Extension for library files, e.g. [.a] under Unix. *)
+val ext_dll: string
+        (* Extension for dynamically-loaded libraries, e.g. [.so] under Unix.*)
