@@ -12,7 +12,7 @@
 
 /* Based on public-domain code from Berkeley Yacc */
 
-/* $Id: skeleton.c,v 1.10 2001/07/12 12:54:23 doligez Exp $ */
+/* $Id: skeleton.c,v 1.11 2001/11/05 13:34:42 xleroy Exp $ */
 
 #include "defs.h"
 
@@ -38,7 +38,9 @@ char *define_tables[] =
   "    tablesize=yytablesize;",
   "    table=yytable;",
   "    check=yycheck;",
-  "    error_function=parse_error }",
+  "    error_function=parse_error;",
+  "    names_const=yynames_const;",
+  "    names_block=yynames_block }",
   0
 };
 
